Package: survdt
Title: Improved Methods for Survival Analysis under Double Truncation
Version: 0.9.0
Authors@R: c(
    person("Omar", "Vazquez", email = "omar.vazquez@pennmedicine.upenn.edu", 
           role = c("aut", "cre", "cph"), 
           comment = c(ORCID = "0009-0009-4137-3333")),
    person(c("Sharon", "X."), "Xie", role = "ths", 
           comment = c(ORCID = "0000-0002-7886-4103")))
Description: Contains existing and novel methods for nonparametric analysis and 
    Cox regression analysis with doubly truncated survival data, as described in 
    Vazquez and Xie (2025) <doi:10.1007/s10985-025-09650-5>. Includes survival 
    curves and hazard estimates through nonparametric maximum likelihood 
    estimation, various tests for detecting group differences or non-ignorable 
    sampling bias, and inverse probability weighted Cox regression with several 
    options of nonparametric weights. Also implements diagnostics for key 
    modeling assumptions such as quasi-independent truncation and the positivity
    assumption. Closed-form standard errors are available for all estimates, i.e.
    bootstrapping is not required.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: ggplot2, igraph, stats, survival
Depends: R (>= 2.10)
LazyData: true
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-03 21:12:41 UTC; ovazquez
Author: Omar Vazquez [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0009-4137-3333>),
  Sharon X. Xie [ths] (ORCID: <https://orcid.org/0000-0002-7886-4103>)
Maintainer: Omar Vazquez <omar.vazquez@pennmedicine.upenn.edu>
Repository: CRAN
Date/Publication: 2026-05-06 19:30:02 UTC
Built: R 4.5.2; ; 2026-05-06 20:37:59 UTC; unix

Package: survdt
Title: Improved Methods for Survival Analysis under Double Truncation
Version: 0.9.0
Authors@R: c(
    person("Omar", "Vazquez", email = "omar.vazquez@pennmedicine.upenn.edu", 
           role = c("aut", "cre", "cph"), 
           comment = c(ORCID = "0009-0009-4137-3333")),
    person(c("Sharon", "X."), "Xie", role = "ths", 
           comment = c(ORCID = "0000-0002-7886-4103")))
Description: Contains existing and novel methods for nonparametric analysis and 
    Cox regression analysis with doubly truncated survival data, as described in 
    Vazquez and Xie (2025) <doi:10.1007/s10985-025-09650-5>. Includes survival 
    curves and hazard estimates through nonparametric maximum likelihood 
    estimation, various tests for detecting group differences or non-ignorable 
    sampling bias, and inverse probability weighted Cox regression with several 
    options of nonparametric weights. Also implements diagnostics for key 
    modeling assumptions such as quasi-independent truncation and the positivity
    assumption. Closed-form standard errors are available for all estimates, i.e.
    bootstrapping is not required.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: ggplot2, igraph, stats, survival
Depends: R (>= 2.10)
LazyData: true
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-03 21:12:41 UTC; ovazquez
Author: Omar Vazquez [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0009-4137-3333>),
  Sharon X. Xie [ths] (ORCID: <https://orcid.org/0000-0002-7886-4103>)
Maintainer: Omar Vazquez <omar.vazquez@pennmedicine.upenn.edu>
Repository: CRAN
Date/Publication: 2026-05-06 19:30:02 UTC
Built: R 4.5.2; ; 2026-05-06 21:11:41 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",Survdt)
S3method("[",Survdt2)
S3method(as.data.frame,Survdt)
S3method(as.data.frame,Survdt2)
S3method(coef,coxph_indtrunc)
S3method(confint,coxph_indtrunc)
S3method(is.na,Survdt)
S3method(is.na,Survdt2)
S3method(length,Survdt)
S3method(length,Survdt2)
S3method(plot,Survdt)
S3method(plot,Survdt2)
S3method(plot,coxph_pos_indtrunc)
S3method(plot,ignorability_indtrunc_test)
S3method(plot,npmle)
S3method(plot,quasiindep_strat_test)
S3method(plot,samplediff_indtrunc_test)
S3method(print,coxph_indtrunc)
S3method(print,coxph_pos_indtrunc)
S3method(print,ignorability_indtrunc_test)
S3method(print,npmle)
S3method(print,quasiindep_ctau_test)
S3method(print,quasiindep_strat_test)
S3method(print,samplediff_indtrunc_test)
S3method(residuals,coxph_indtrunc)
S3method(vcov,coxph_indtrunc)
export(Survdt)
export(Survdt2)
export(coxph_indtrunc)
export(is.Survdt)
export(is.Survdt2)
export(is.npmle)
export(npmle)
export(plot_coxsurv)
export(positivity_sens_indtrunc)
export(survdt2to1)
export(test_ignorability_indtrunc)
export(test_quasiindep_covariates)
export(test_quasiindep_ctau)
export(test_samplediff_indtrunc)
export(time_split)

#' Cox regression under quasi-independent double truncation.
#'
#' @description
#' Computes an inverse probability weighted Cox regression estimator using
#' nonparametric weights (see [npmle()]).
#' Includes options for robust estimators with time-varying weights, which
#' introduce an event time stabilization weight (see below).
#'
#' @param formula A model formula with a `Survdt` or `Survdt2` response.
#'   Subjects with any missing values are discarded.
#'   See [Survdt()] and [Survdt2()].
#' @param data An optional data frame used to evaluate `formula` and
#' `strata_formula`.
#' @param strata_formula An optional one-sided formula with stratification
#'   variables on the right hand side, to fit a stratified Cox model.
#' @param ipw_type Type of inverse probability weights.
#'   1. `"W-asurv"` (the default): stabilized robust time-varying weights based
#'      on the NPMLE survival function.
#'   2. `"W-surv"`: robust time-varying weights based on the NPMLE survival function.
#'   3. `"W-1"`: standard weights with no time-varying component (Rennert and Xie, 2018)
#'   4. `"W-a"`: stabilized weights (Mandel et al., 2018)
#'   5. For custom time-varying weights, provide a function that takes a numerical
#'      vector of times as input and returns a numerical vector of positive weights.
#' @param se Whether to calculate standard errors (default `TRUE`).
#' @param robust_se Whether to calculate standard errors that are robust to
#'   model misspecification (default `TRUE`).
#'   Ignored if `se`=`FALSE`.
#' @param basehaz Whether to return the baseline hazard estimate
#'   (default `FALSE`).
#' @param restr_times An optional numerical vector of length two giving a time
#'   window to restrict the score function.
#'   Values of `NA` are replaced with the minimum and maximum observed event
#'   time respectively, i.e. no restriction.
#'   The default setting is no restriction.
#' @param npmle_fit An optional pre-computed `npmle` object fit on the complete
#'   cases.
#'   If it was fit with `ainv`=`FALSE` (or `infl`=`FALSE` when `se`=`TRUE`) then
#'   it is ignored and a new NPMLE is fit.
#'   See [npmle()].
#' @param robust_infl A square matrix of influence values for the event weight,
#'   with one column per individual and one row per follow up time, both in the
#'   order that the subject appears in the response variable.
#'   Only used if `ipw_type` is a function and `robust_se`=`TRUE`.
#' @param simple_labels Whether to drop the factor names from the stratum labels
#'   (Default `FALSE`).
#' @param keep_x Whether to include the model matrix in the `cox_fit` component
#'   of the return value (Default `FALSE`).
#' @param ... Additional arguments passed to other methods.
#' @param object,x A `coxph_indtrunc` object.
#' @param parm A numeric or character vector specifying a set of model
#'   parameters.
#'   Defaults to all identifiable (non-missing) coefficients.
#' @param level Confidence level for confidence intervals.
#' @param type The type of residuals to return.
#' @param per_subject Whether to return per-subject residuals as opposed to
#'   per-observation residuals, for models with `Survdt2` data.
#'   Defaults to `TRUE` for deviance residuals and `FALSE` otherwise.
#'
#' @details
#' Given a sample of doubly truncated data with event times \eqn{T_i}, truncation
#' times \eqn{L_i} and \eqn{R_i}, and potentially time-varying covariates
#' \eqn{X_i(t)}, let \eqn{N_i(t)=1(T_i \leq t)} be the usual counting process
#' and \eqn{Y_i(t)=1(T_i \geq t)} be the at-risk process.
#'
#' [coxph_indtrunc()] fits a Cox model by maximizing an inverse probability
#' weighted partial likelihood, where the double truncation bias is accounted
#' for through nonparametric estimates of
#' \eqn{a(t)^{-1}=P(L<T \leq R)/P(L<T \leq R|T=t)} (see [npmle()]).
#' Assuming `strata_formula=NULL` for sake of illustration, setting
#' `ipw_type="W-1"` specifies non-time-varying weights, so the model is fitted by
#' finding the root of the weighted score function
#' \deqn{\sum_{i}\int \frac{1}{\widehat{a}(T_i)}
#'       [X_i(t) - \overline{X}_{\widehat{a}}(t,\beta)]dN_i(t)}
#' \deqn{\text{where } \overline{X}_{\widehat{a}}(t,\beta)
#'       = \frac{\sum_j X_j(t)\widehat{a}(T_j)^{-1}Y_j(t)\exp(X_j(t)^T\beta)}
#'              {\sum_j \widehat{a}(T_j)^{-1}Y_j(t)\exp(X_j(t)^T\beta)}.}
#'
#' More generally, [coxph_indtrunc()] allows for time-varying weights with a
#' stabilization component \eqn{\widehat{w}(t)} that determines the relative
#' importance of each event time.
#' This can improve robustness and numerical convergence without introducing any
#' bias, in contrast to weight truncation, for example.
#' The weighted score function becomes
#' \deqn{\sum_{i}\int \frac{\widehat{w}(t)}{\widehat{a}(T_i)}
#'       [X_i(t) - \overline{X}_{\widehat{a}}(t,\beta)]dN_i(t).}
#' Several choices of time-varying weights are available:
#' * `ipw_type="W-surv"`: \eqn{\widehat{w}(t)=\widehat{S}(t-)}, where
#'    \eqn{\widehat{S}(t)} is the marginal survival function estimated by
#'    [npmle()].
#'    Can increase robustness to outliers.
#' * `ipw_type="W-a"`: \eqn{\widehat{w}(t)=\widehat{a}(t)}.
#'    Can reduce the influence of extreme inverse probability weights.
#' * `ipw_type="W-asurv"`: \eqn{\widehat{w}(t)=\widehat{a}(t)\widehat{S}(t-)}.
#'    Combines the benefits of both weights.
#' * Custom stabilization weights \eqn{\widehat{w}(t)} can be specified by
#'   supplying a function as the input to `ipw_type`.
#'   If `robust_se=TRUE`, this also requires the influence values for
#'   \eqn{\widehat{w}(t)} to be supplied in `robust_infl`, in order to account
#'   for its variability in the standard errors.
#'
#' @section Stratified Cox model:
#'
#' A stratified Cox model is specified through `strata_formula`.
#' The formula should look like `~svar1 + svar2`... where `svar1` and `svar2` are
#' factor variables defining the strata.
#' This allows the baseline hazard to differ across strata.
#'
#' @section Restricted time window in score function:
#' The user can choose to restrict the time range of the score function during
#' estimation.
#' This may be useful when extreme event times have very large weights.
#' Setting the input argument `restr_times=c(t1,t2)` will restrict the range of
#' the integrals in the score function to the interval \eqn{[}`t1`, `t2`\eqn{]}
#' instead of the entire event time range.
#' Notably, this time restriction will not introduce any bias in the fitted model.
#'
#' @returns
#' [coxph_indtrunc()] returns an object of class `coxph_indtrunc`, i.e. a list
#' with the following components:
#' \item{n}{The complete case sample size.}
#' \item{n_miss}{The number of individuals removed due to missing data.}
#' \item{n_event}{The number of observed events in the score function.}
#' \item{coef}{A four-column matrix containing the estimated coefficients,
#'   standard errors, z-statistics, and p-values. Coefficient values of `NA`
#'   indicate collinearity.}
#' \item{var_matrix}{The estimated variance matrix for the coefficient estimates.
#'   Values of `NA` indicate collinearity.}
#' \item{robust_se}{The input value for `robust_se`.}
#' \item{offset}{The offsets from the model formula for complete case observations.}
#' \item{ipw}{The estimated inverse probability of non-truncation for each
#'   complete case observation.}
#' \item{time_wt}{The time-varying component of the inverse probability weights
#'   for each complete case observation.}
#' \item{cc_indx}{A logical index vector for the complete cases in the original
#'   data.}
#' \item{id_indx}{A vector of ids matching each observation with a subject among
#'   the complete cases.}
#' \item{sort_indx}{A numerical vector to sort the complete cases into the order
#'   they are used in `cox_fit` (see below).}
#' \item{s}{The stratum value for each complete case observation if a stratified
#'   Cox model was fit.
#'   Otherwise `NULL`.}
#' \item{martingale_resid}{The weighted martingale residuals for the complete
#'   cases, with one entry per observation.}
#' \item{deviance_resid}{The weighted deviance residuals for the complete cases,
#'   with one entry per subject.}
#' \item{basehaz}{For input `basehaz`=`TRUE`, a list with first element `s`
#'   containing the stratum values and one additional element for each stratum,
#'   in the order of `s`.
#'   Each contains a matrix with following columns:
#'   `time`, `cumhaz` with the cumulative baseline hazard value (baseline at x=0),
#'   `var_logcumhaz` with the estimated variance of the log(cumhaz), and
#'   additional columns, labeled by predictor names, with the estimated covariance
#'   between the log(cumhaz) and the regression coefficients.
#'   For input `basehaz`=`FALSE`, `NULL`.}
#' \item{timerange}{A vector with the minimum and maximum times used in the
#' score function.}
#' \item{eventrange}{A vector with the unrestricted range of observed event times.}
#' \item{wt_name}{A string giving the name of the weights used in `ipw_type`.}
#' \item{cox_fit}{The `coxph` object used to obtain the estimates.
#'   Note that the data was sorted by strata and time before fitting the model,
#'   and the offset may include an additional term of `-log(time_wt)`.}
#' \item{strata_summary}{Some summary statistics for each stratum, computed
#'   prior to applying `restr_times`.}
#' \item{call_}{The call to [coxph_indtrunc()].}
#'
#' `coxph_indtrunc` objects have methods for `vcov` to extract the variance matrix
#' of estimated coefficients, `coef` to extract the estimated coefficients,
#' `confint` to compute confidence intervals for coefficients, `residuals` for
#' model diagnostics, and `print` to display a summary of the fitted model.
#'
#' `residuals` returns per-subject residuals in the order that the subject first
#' appeared in the original data.
#' The deviance residuals are computed as
#' \eqn{\text{sign}(\widehat{M}_{wi})\sqrt{
#' -2[\widehat{M}_{wi} +
#' \delta_{wi}\log(1 - \widehat{M}_{wi}/\delta_{wi})]}}
#' where \eqn{\widehat{M}_{wi}} is the weighted martingale residual and
#' \eqn{\delta_{wi}} is the weighted event indicator, following
#' Therneau et al. (1990).
#'
#' @export
#'
#' @seealso
#' * [plot_coxsurv()]: compute/plot estimates and pointwise confidence intervals
#'   for the conditional survival function or cumulative hazard from a fitted Cox
#'   model.
#' * [positivity_sens_indtrunc()]: sensitivity analysis for positivity violations
#'   in the Cox model.
#' * [test_quasiindep_covariates()]: test for dependence between truncation times
#'   and the event time or covariates.
#'   The nonparametric weights used by [coxph_indtrunc()] requires quasi-independence.
#' * `vignette("cox-regression-quasiindep", package = "survdt")`: an
#'   illustration of Cox regression and related diagnostics applied to the
#'   [aids] data, if vignettes were built when installing [survdt].
#'
#' @examples
#' coxph_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
#'                data = aids)
#'
#' @references
#'
#' Mandel, M., de Uña-Álvarez, J., Simon, D. K., & Betensky, R. A. (2018).
#' Inverse probability weighted Cox regression for doubly truncated data.
#' *Biometrics*, **74**(2), 481--487.
#'
#' Rennert, L., & Xie, S. X. (2018). Cox regression model with doubly truncated
#' data. *Biometrics*, **74**(2), 725--733.
#'
#' Therneau, T. M., Grambsch, P. M., & Fleming, T. R. (1990). Martingale-based
#' residuals for survival models. *Biometrika*, **77**(1), 147--160.
#'
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
coxph_indtrunc <- function(formula, data = NULL, strata_formula = NULL,
                           ipw_type = c("W-asurv", "W-surv", "W-1", "W-a"),
                           se = TRUE, robust_se = TRUE, basehaz = FALSE,
                           restr_times = NULL, npmle_fit = NULL,
                           robust_infl = NULL, simple_labels = FALSE,
                           keep_x = FALSE){
  # check input
  if (missing(formula))
    stop('\n  argument "formula" is missing, with no default')

  fn_call <- match.call()
  vals <- prep_data_for_fit(formula, data, strata_formula, simple_labels)
  if (is.Survdt2(vals$y))
    y_npmle <- survdt2to1(vals$y[vals$complete_cases, ])
  else
    y_npmle <- vals$y[vals$complete_cases, ]
  n <- length(y_npmle)
  s <- NULL
  if (!is.null(vals$s)) s <- vals$s[vals$sort_indx]

  if (!is.null(s)){
    strata_vals <- unique(s)
    n_strata <- length(strata_vals)
    if (n_strata < 2)
      stop(paste0("Only one stratum in complete cases.\n ",
                  "Either update the strata formula or drop it altogether"))

    if (is.Survdt2(vals$y)){
      event_rows <- vals$sorted_cc[vals$y[vals$sorted_cc, "event",
                                          drop = TRUE] == 1]
    }else{
      event_rows <- vals$sorted_cc
    }

    n_unique_strata <- stats::aggregate(vals$y[event_rows, vals$time_col,
                                               drop = TRUE] ~ s,
                                        FUN = function(x) c(length(unique(x))))
    small_strata <- n_unique_strata[n_unique_strata[, 2, drop = TRUE] < 2, 1,
                                    drop = TRUE]
    if (length(n_unique_strata) > 5)
      n_unique_strata <- c(as.character(n_unique_strata[1:5]), "...")
    if (length(small_strata) > 0)
      warning(paste0("Less than two unique event times found in stratum: ",
                     paste0(small_strata, collapse = "; ")))
    strata_summary <- data.frame(strata = n_unique_strata[, 1, drop = TRUE],
                                 strata_size =
                                   sapply(n_unique_strata[, 1, drop = TRUE],
                                          FUN = function(x) sum(s == x)),
                                 unique_event_times =
                                   n_unique_strata[, 2, drop = TRUE])
  }else{
    strata_vals <- NULL
    n_strata <- 1
    strata_summary <- NULL
  }

  if (is.null(restr_times)) restr_times <- c(NA_real_, NA_real_)
  stopifnot((is.vector(ipw_type) && is.character(ipw_type)) ||
              is.function(ipw_type),
            is.logical(se), is.vector(se), length(se) == 1, all(!is.na(se)),
            is.logical(robust_se), is.vector(robust_se), length(robust_se) == 1,
            all(!is.na(robust_se)), is.logical(basehaz), is.vector(basehaz),
            length(basehaz) == 1, all(!is.na(basehaz)), is.numeric(restr_times),
            is.vector(restr_times), length(restr_times) == 2,
            is.null(npmle_fit) || is.npmle(npmle_fit),
            is.null(robust_infl) || is.matrix(robust_infl),
            is.vector(keep_x), is.logical(keep_x), length(keep_x) == 1,
            all(!is.na(keep_x)))
  robust_se <- ifelse(se == TRUE, robust_se, FALSE)

  # time window
  eventrange <- range(y_npmle[, "time", drop = TRUE])
  time_lims <- eventrange
  if (!is.na(restr_times[1]) && (restr_times[1] >= time_lims[2]))
    stop("restr_times[1] must be less than the maximum observed event time")
  if (!is.na(restr_times[2]) && (restr_times[2] <= time_lims[1]))
    stop("restr_times[2] must be greater than the minimum observed event time")
  if (all(!is.na(restr_times)) && (restr_times[1] >= restr_times[2]))
    stop("Must have restr_times[1]<restr_times[2]")
  test_window <- ifelse(is.na(restr_times), time_lims,
                        c(max(restr_times[1], time_lims[1]),
                          min(restr_times[2], time_lims[2])))
  ltrunc_indx <-
    vals$y[vals$sorted_cc, vals$time_col, drop = TRUE] < test_window[1]
  rcens_indx <-
    test_window[2] < vals$y[vals$sorted_cc, vals$time_col, drop = TRUE]
  n_events <- sum(!ltrunc_indx[vals$event_indx] & !rcens_indx[vals$event_indx])
  if (n_events == 0) stop("No observed event times in restricted window")

  if (is.character(ipw_type)){
    ipw_type <- match.arg(ipw_type)
    wt_name <- ipw_type
  }else{
    if ((robust_se == TRUE) && is.null(robust_infl))
      stop(paste0("Supplying a function for ipw_type with ",
                  "robust_se=TRUE requires a matrix of influence values in ",
                  "robust_infl"))
    time_wt <- ipw_type(y_npmle[, "time", drop = TRUE])
    if (!is.numeric(time_wt) || !is.vector(time_wt))
      stop("Function ipw_type did not return a numerical vector")
    if (length(time_wt) != length(y_npmle))
      stop(paste0("Function ipw_type should return ",
                  "one value per time input. Output length is ", length(time_wt),
                  " but ", length(y_npmle), " times requested"))

    if (any(is.na(time_wt)))
      stop("Missing values in output of function ipw_type")
    if (any(time_wt < 0))
      stop("Negative values in output of function ipw_type")
    if (robust_se == TRUE){
      stopifnot(nrow(robust_infl) == ncol(robust_infl))
      if (nrow(robust_infl) != vals$n_init)
        stop(paste0("Matrix robust_infl has dimension ", nrow(robust_infl),
                    ", does not match number of subjects ", vals$n_init))
      robust_infl <- robust_infl[vals$complete_id_nums, vals$complete_id_nums,
                                 drop = FALSE]
      stopifnot(!any(is.na(robust_infl)), !any(is.infinite(robust_infl)))
    }
    wt_name <- paste0("user-supplied function ", format(substitute(ipw_type)))
  }

  # npmle
  if (!is.null(npmle_fit)){
    stopifnot(is.npmle(npmle_fit))
    if (nrow(npmle_fit[["masses"]]) != vals$n)
      stop(paste0("npmle_fit sample size (", nrow(npmle_fit[["masses"]]), ") ",
                  "does not match the number of complete cases in the model ",
                  "frame (", vals$n, ")"))
    if (is.null(npmle_fit[["ainv"]]) ||
        (is.null(npmle_fit[["infl_ainv"]]) && (se == TRUE)))
      npmle_fit <- NULL
    else{
      if (any(abs(y_npmle[, "time", drop = TRUE] -
                  npmle_fit[["ainv"]][, "time", drop = TRUE]) > 1e-9))
        stop("Follow up times in formula response and npmle_fit do not match")
    }
  }
  if (is.null(npmle_fit)){
    npmle_fit <- npmle(y_npmle, ainv = TRUE, se = se, infl = se)
    if (!npmle_fit[["converged"]]) stop("Fitted npmle did not converge")
  }
  ipw <- npmle_fit[["ainv"]][, "ainv", drop = TRUE]

  # time-varying weights
  if (is.character(ipw_type)){
    calculate_infl <- (se == TRUE) && (robust_se == TRUE)
    if (ipw_type == "W-1"){
      time_wt <- rep(1, n)
    }else if (ipw_type == "W-a"){
      time_wt <- 1 / ipw
      if (calculate_infl == TRUE){
        robust_infl <- apply(npmle_fit[["infl_ainv"]], MARGIN = 2,
                             FUN = function(x) {-x / (ipw^2)})
      }
    }else{
      cdf_time_match <- pmin(findInterval(y_npmle[, "time", drop = TRUE],
                                          npmle_fit[["cdf"]][,
                                                             "time",
                                                             drop = TRUE],
                                          left.open = TRUE) + 1,
                             nrow(npmle_fit[["cdf"]]))
      lcontsurv <- c(1, 1 - npmle_fit[["cdf"]][,
                                               "cdf",
                                               drop = TRUE])[cdf_time_match]

      if (ipw_type == "W-surv"){
        time_wt <- lcontsurv
        if (calculate_infl == TRUE){
          robust_infl <- -rbind(0, npmle_fit[["infl_cdf"]])[cdf_time_match, ,
                                                            drop = FALSE]
        }
      }else if (ipw_type == "W-asurv"){
        time_wt <- lcontsurv / ipw
        if (calculate_infl == TRUE){
          robust_infl <- sapply(seq_len(n), FUN = function(i){
            (-npmle_fit[["infl_ainv"]][, i, drop = TRUE] *
               lcontsurv / (ipw^2)) -
              (c(0,
                 npmle_fit[["infl_cdf"]][, i, drop = TRUE])[cdf_time_match] /
                 ipw)
          })
        }
      }
    }
  }
  time_wt <- time_wt + 1e-7
  # time_wt and ipw are in original data order, one entry per id

  X <- vals$X[vals$sorted_cc, , drop = FALSE]
  ofst <- vals$ofst[vals$sorted_cc]
  n_miss <- vals$n_init - vals$n
  expanded_time_wt <- time_wt[vals$match_id_indx[vals$sorted_cc]]
  ofst_wt <- ofst - log(expanded_time_wt)
  expanded_ipw <- ipw[vals$match_id_indx[vals$sorted_cc]]
  outer_wt <- expanded_ipw * expanded_time_wt

  if (is.Survdt2(vals$y)){
    y_coxph <- survival::Surv(vals$y[vals$sorted_cc, "start", drop = TRUE],
                              vals$y[vals$sorted_cc, "stop", drop = TRUE],
                              as.numeric((vals$y[vals$sorted_cc, "event",
                                                 drop = TRUE] == 1) &
                                           !ltrunc_indx & !rcens_indx))
  }else{
    y_coxph <- survival::Surv(vals$y[vals$sorted_cc, "time", drop = TRUE],
                              as.numeric(!ltrunc_indx & !rcens_indx))
  }
  n_event <- sum(y_coxph[, ncol(y_coxph), drop = TRUE])


  # fit cox model
  offset <- stats::offset
  strata <- survival::strata
  if (is.null(s)){
    cox_fit <- survival::coxph(y_coxph ~ X + offset(ofst_wt), weights = outer_wt,
                               na.action = stats::na.fail, ties = "breslow",
                               robust = FALSE, x = keep_x)
  }else{
    cox_fit <- survival::coxph(y_coxph ~ X + offset(ofst_wt) + strata(s),
                               weights = outer_wt, na.action = stats::na.fail,
                               ties = "breslow", robust = FALSE, x = keep_x)
  }

  # std errors
  p <- length(cox_fit[["coefficients"]])
  coef_na <- is.na(cox_fit[["coefficients"]])
  effective_p <- sum(!coef_na)
  if (effective_p == 0)
    stop("All coefficient estimates are NA. Check predictors for collinearity.")
  var_matrix <- matrix(NA, nrow = p, ncol = p)
  if (se == TRUE){
    # effective_p x n
    var_components <- t(stats::residuals(cox_fit, "score", weighted = TRUE,
                                         collapse = vals$id[vals$sorted_cc]))
    var_components <- var_components[!coef_na, , drop = FALSE]
    var_components <- var_components +
      (var_components %*%
         diag(1 / ipw, length(ipw)) %*%
         npmle_fit[["infl_ainv"]] / n)
    time_varying_weights <- is.function(ipw_type) ||
      (is.character(ipw_type) && (ipw_type != "W-1"))

    if ((robust_se == TRUE) && (time_varying_weights == TRUE)){
      delta_indx <- as.logical(y_coxph[, ncol(y_coxph), drop = TRUE])
      event_id_indx <- vals$complete_id_nums %in%
        (vals$id[vals$sorted_cc])[delta_indx]
      sch_resids <- t(stats::residuals(cox_fit, "schoenfeld",
                                       weighted = TRUE))[!coef_na, ,
                                                         drop = FALSE]
      var_components <- var_components +
        (sch_resids[, order(vals$sorted_cc[delta_indx]), drop = FALSE] %*%
           diag(1 / time_wt[event_id_indx], sum(event_id_indx)) %*%
           robust_infl[event_id_indx, , drop = FALSE] / n)
    }

    meat <- apply(var_components, MARGIN = 2, FUN = function(x) x %o% x)
    if (effective_p == 1){
      meat <- sum(meat)
    }else{
      meat <- rowSums(meat)
    }
    var_matrix[!coef_na, !coef_na] <-
      cox_fit[["var"]][!coef_na, !coef_na, drop = FALSE] %*%
      matrix(meat, nrow = effective_p) %*%
      cox_fit[["var"]][!coef_na, !coef_na, drop = FALSE]
  }

  ests <- cbind(coef = cox_fit[["coefficients"]], se = sqrt(diag(var_matrix)),
                zstat = cox_fit[["coefficients"]] / sqrt(diag(var_matrix)),
                pval = stats::pnorm(abs(cox_fit[["coefficients"]] /
                                          sqrt(diag(var_matrix))),
                                    lower.tail = FALSE) * 2)
  rownames(ests) <- colnames(X)
  rownames(var_matrix) <- colnames(X)
  colnames(var_matrix) <- colnames(X)

  # baseline hazard
  bhaz <- NULL
  if (basehaz == TRUE){
    if (is.null(s)) s <- factor(rep(1, nrow(y_coxph)))
    stratas <- unique(s)
    bhaz <- list(s = stratas)

    wexps <- exp(c(X[, !coef_na, drop = FALSE] %*%
                     cox_fit[["coefficients"]][!coef_na]) + ofst) *
      expanded_ipw

    for (i in seq_along(stratas)){
      ys <- unclass(cox_fit[["y"]])[s == stratas[i], , drop = FALSE]
      if (is.Survdt2(vals$y)){
        ys[, ncol(ys)] <- vals$y[vals$sorted_cc[s == stratas[i]],
                                 "event",
                                 drop = TRUE]
      }else{
        ys[, ncol(ys)] <- rep(1, nrow(ys))
      }

      strata_indx <- which(s == stratas[i])
      strata_ipw <- expanded_ipw[strata_indx]
      strata_ids <- (vals$id[vals$sorted_cc])[strata_indx]
      strata_uniqueids <- unique(strata_ids)
      strata_wexps <- wexps[strata_indx]

      etimes <- sort(unique(ys[ys[, ncol(ys), drop = TRUE] == 1,
                               ncol(ys) - 1, drop = TRUE]))
      n_etime_strata <- length(etimes)
      # row time, column observation window
      if (ncol(ys) == 3){
        atrisk <- apply(ys, MARGIN = 1,
                        FUN = function(z) (z[1] < etimes) & (etimes <= z[2]))
      }else{
        atrisk <- sapply(ys[, 1, drop = TRUE], FUN = function(z) z >= etimes)
      }
      s0 <- c(atrisk %*% strata_wexps)
      hazincr <- sapply(etimes, FUN = function(tj) {
        sum(strata_ipw[(ys[, ncol(ys), drop = TRUE] == 1) &
                         (ys[, ncol(ys) - 1, drop = TRUE] == tj)])
      }) / s0

      bhaz[[i+1]] <- matrix(NA, ncol = 3 + effective_p, nrow = length(etimes))
      colnames(bhaz[[i+1]]) <- c("time", "cumhaz", "var_logcumhaz",
                                 colnames(X)[!coef_na])
      bhaz[[i+1]][, "time"] <- etimes
      bhaz[[i+1]][, "cumhaz"] <- cumsum(hazincr)

      if (se == TRUE){
        xavg <- diag(1 / s0, length(s0)) %*%
          (atrisk %*% diag(strata_wexps, length(strata_wexps)) %*%
             X[strata_indx, !coef_na, drop = FALSE])

        match_etimes <- match(ys[, ncol(ys) - 1, drop = TRUE],
                              etimes,
                              nomatch = NA_integer_)
        dM_infl <- sapply(strata_uniqueids, FUN = function(j){
          id_indx <- which(strata_ids == j)
          match_id_times <- c(apply(atrisk[, id_indx, drop = FALSE], MARGIN = 1,
                                    FUN = function(z) {
                                      colindx <- utils::tail(which(z), 1)
                                      ifelse(length(colindx) == 0, NA, colindx)
                                      }))
          nomatch <- is.na(match_id_times)
          x <- rep(0, length(etimes))
          if (any(!nomatch)){
            event_id <- id_indx[which(ys[id_indx,
                                         ncol(ys),
                                         drop = TRUE] == 1)[1]]
            if (!is.na(event_id)){
              x[match_etimes[event_id]] <- (strata_ipw[event_id])[1]
            }
            x <- x - ifelse(nomatch,
                            0,
                            (strata_wexps[id_indx[match_id_times]] * hazincr))
            x <- n * x / s0
          }
          cumsum(x)
        })

        # row time, col individual
        infl_cumhaz <- apply(xavg, MARGIN = 2,
                             FUN = function(z) cumsum(z * hazincr)) %*%
          cox_fit[["var"]][!coef_na, !coef_na, drop = FALSE] %*%
          var_components * (-n)

        strata_indivs <- match(strata_uniqueids, vals$complete_id_nums)
        infl_cumhaz[, strata_indivs] <- infl_cumhaz[, strata_indivs, drop = FALSE] +
          dM_infl
        infl_cumhaz <- infl_cumhaz +
          (dM_infl %*%
             diag(1 / (ipw[strata_indivs] * n), length(strata_indivs)) %*%
             npmle_fit[["infl_ainv"]][strata_indivs, , drop = FALSE])

        bhaz[[i+1]][, "var_logcumhaz"] <- rowMeans(infl_cumhaz^2) /
          (bhaz[[i+1]][, "cumhaz"]^2 * n)
        bhaz[[i+1]][, colnames(X[, !coef_na, drop = FALSE])] <-
          diag(1 / (bhaz[[i+1]][, "cumhaz", drop = TRUE] * n),
               length(bhaz[[i+1]][, "cumhaz", drop = TRUE])) %*%
          infl_cumhaz %*%
          t(var_components) %*%
          cox_fit[["var"]][!coef_na, !coef_na, drop = FALSE]
      }
    }
  }

  # residuals
  mresid <- stats::residuals(cox_fit, type = "martingale", weighted = TRUE)
  rsums <- rowsum(cbind(mresid,
                        y_coxph[, ncol(y_coxph), drop = TRUE] * outer_wt,
                        y_coxph[, ncol(y_coxph), drop = TRUE]),
                  vals$id[vals$sorted_cc]) # returned in original data order
  subj_mresid <- rsums[, 1, drop = TRUE]
  subj_wt <- ifelse(rsums[, 2, drop = TRUE] == 0, 1, rsums[, 2, drop = TRUE])
  subj_event <- rsums[, 3, drop = TRUE]
  subj_dresid <- sign(subj_mresid) *
    sqrt(-2 * (subj_mresid +
                 ifelse(subj_event > 0,
                        subj_wt * log(1 - subj_mresid / subj_wt),
                        0)))

  unsort_indx <- order(vals$sort_indx)
  res <- list(n = n, n_miss = n_miss, n_event = n_event, coef = ests,
              var_matrix = var_matrix, robust_se = robust_se,
              offset = ofst[unsort_indx], ipw = expanded_ipw[unsort_indx],
              time_wt = expanded_time_wt[unsort_indx],
              cc_indx = vals$complete_cases,
              id_indx = vals$id[vals$complete_cases],
              sort_indx = vals$sort_indx, s = s,
              martingale_resid = mresid[unsort_indx],
              deviance_resid = subj_dresid, basehaz = bhaz,
              timerange = test_window, eventrange = eventrange,
              wt_name = wt_name, cox_fit = cox_fit,
              strata_summary = strata_summary, call_ = fn_call)
  class(res) <- "coxph_indtrunc"
  res
}


#' Plot survival or hazard estimates from a Cox model fit
#'
#' @description
#' Given a fitted Cox model from [coxph_indtrunc()], plot the conditional survival
#' or cumulative hazard estimates along with pointwise confidence intervals.
#' The baseline hazard and its standard errors are computed by [plot_coxsurv()]
#' if they were not included in the `coxph_indtrunc` fit.
#'
#' @param object A `coxph_indtrunc` object. See [coxph_indtrunc()].
#' @param newdata A data frame with the covariate values for the plot.
#'   The original model formula is applied to extract predictors.
#'   Rows that are duplicated or have missing values are discarded.
#' @param ci Whether to plot pointwise confidence intervals (default `TRUE`).
#'   Will re-fit the model if `basehaz` or its standard errors were not computed.
#' @param level Confidence level for the pointwise confidence intervals
#'   (default 0.95).
#' @param ctype Type of confidence interval.
#'   * `"plain"`: linear confidence from a normal approximation.
#'   * `"log"`: back-transformed confidence intervals from the natural log scale.
#'     Guaranteed to be non-negative.
#'   * `"log-log"` (the default): for the survival function, back-transformed
#'     confidence intervals from the (natural) log minus log scale, which are
#'     guaranteed to be between zero and one.
#'     For the cumulative hazard, the same confidence intervals as `"log"` are
#'     used.
#' @param target Which quantity to plot.
#'   * `"survival"` (the default): the conditional survival function.
#'   * `"cumhaz"`: the conditional cumulative hazard.
#'
#' @details
#' In order to construct the linear predictors at the requested covariate values,
#' the design matrix obtained from evaluating the original model formula on
#' `newdata` should have the same column names as the design matrix for the
#' fitted model.
#' Issues can occur for categorical variables that were converted to factors by
#' [coxph_indtrunc()] for estimation.
#' If an error occurs, check that each categorical variable in `newdata` is a
#' factor variable that is not missing any levels used to fit the model.
#'
#' @returns
#' After displaying the plot, a list with two elements is returned invisibly.
#' The first element `ests` is a data frame with the columns `time`, `est`,
#' `lower`, and `upper`, containing the point estimates and their computed
#' confidence intervals.
#' The second element `plot` is the plotted object returned by [ggplot2::ggplot()].
#' Lines are colored based on the covariate value.
#'
#' @export
#'
#' @seealso [coxph_indtrunc()]
#'
#' @examples
#' fit <- coxph_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
#'                       data = aids,
#'                       basehaz = TRUE)
#' plot_coxsurv(fit, newdata = aids[c(1,40), ])
#'
#' plot_coxsurv(fit, newdata = aids[c(1,40), ],
#'              target = "cumhaz")
plot_coxsurv <- function(object, newdata, ci = TRUE, level = 0.95,
                         ctype = c("log-log", "log", "plain"),
                         target = c("survival", "cumhaz")){
  # check input
  if (!inherits(object, "coxph_indtrunc"))
    stop("Model object must be of class coxph_indtrunc.")
  stopifnot(is.matrix(newdata) || is.data.frame(newdata))
  if (is.matrix(newdata)) newdata <- as.data.frame(newdata)
  ctype <- match.arg(ctype)
  target <- match.arg(target)
  stopifnot(is.logical(ci), is.vector(ci), all(!is.na(ci)),
            is.numeric(level), is.vector(level), all(!is.na(level)))
  stopifnot(nrow(newdata) > 0, length(ci) == 1, length(level) == 1)
  stopifnot(level > 0, level < 1)
  q <- stats::qnorm(1-(1-level)/2)

  # Survdt2 response not supported
  if (ncol(object[["cox_fit"]]$y) > 2)
    stop("plot_coxsurv does not currently support time-varying covariates")

  # check for basehaz
  if (is.null(object[["basehaz"]]) ||
      ((ci == TRUE) &&
       (!("var_logcumhaz" %in% colnames(object[["basehaz"]][[2]])) ||
        (any(is.na(object[["basehaz"]][[2]][, "var_logcumhaz"])))))){

    new_call <- object[["call_"]]
    new_call$basehaz <- TRUE
    new_call$se <- ci
    message(paste0("Obtaining the baseline hazard ",
                   ifelse(ci == TRUE, "with standard errors ", ""),
                   "by calling:\n ",
                   paste0(format(new_call), collapse = "\n")))
    object$basehaz <- eval.parent(new_call, n = 1)[["basehaz"]]
  }

  # evaluate model formulas with newdata
  newdata <- newdata[!duplicated(newdata), , drop = FALSE]
  formula <- eval.parent(object[["call_"]]$formula, n = 1)
  strata_formula <- eval.parent(object[["call_"]]$strata_formula, n = 1)
  simple_labels <- eval.parent(object[["call_"]]$simple_labels, n = 1)
  if (is.null(simple_labels))
    simple_labels <- formals(coxph_indtrunc)[["simple_labels"]]

  # get strata
  if (!is.null(strata_formula)){
    strata_vars <- stats::model.frame(stats::update(strata_formula, ~.-1),
                                      data = newdata, na.action = stats::na.pass)
    complete_cases_s <- rowSums(is.na(strata_vars)) == 0
    sdf_names <- c(apply(strata_vars[1, , drop = FALSE],
                         MARGIN = 1, FUN = names))
    if (simple_labels == TRUE)
      factor_names <- rep("", length(sdf_names))
    else
      factor_names <- sdf_names
    s <- apply(strata_vars, MARGIN = 1, FUN = function(x) {
      paste0(factor_names, x, sep = "", collapse = ", ")
    })
  }else{
    complete_cases_s <- TRUE
    s <- NULL
  }

  # main formula
  main_terms <-
    stats::delete.response(stats::terms(stats::update(formula, ~.+1)))
  mf <- stats::model.frame(main_terms,
                           data = newdata,
                           na.action = stats::na.pass)

  if (sum(sapply(mf, FUN = function(x) any(is.infinite(x)))) > 0)
    stop("Infinite values occurred when evaluating 'formula' in newdata")
  ofst <- stats::model.offset(mf)
  X <- stats::model.matrix(mf, data = newdata)
  X <- X[, -which(attr(X, "assign") == 0), drop = FALSE]
  coef_na <- is.na(object[["coef"]][, "coef", drop = TRUE])
  pred_match_indx <- match(rownames(object[["coef"]])[!coef_na],
                           colnames(X),
                           nomatch = NA)
  if (all(is.na(pred_match_indx)))
    stop("Names of predictors constructed from newdata did not match ",
         "any identifiable parameters")
  X <- X[, pred_match_indx[!is.na(pred_match_indx)], drop = FALSE]
  if (ncol(X) != sum(!coef_na))
    stop("Missing some parameter names in predictors constructed from newdata")

  # missing values
  complete_cases <- (rowSums(is.na(X)) == 0) & complete_cases_s
  if (!is.null(ofst)) complete_cases <- complete_cases & !is.na(ofst)
  n <- sum(complete_cases)
  if (n == 0) stop("All observations in newdata produced missing values")
  X <- X[complete_cases, , drop = FALSE]
  if (is.null(ofst))
    ofst <- rep(0, n)
  else
    ofst <- ofst[complete_cases]

  if (!is.null(s)) {
    s <- s[complete_cases]
    s <- factor(s, levels = levels(object[["s"]]))
    if (any(is.na(s)))
      stop("newdata contained stratum values not found in original fit")
    unique_indx <- !duplicated(as.data.frame(cbind(X, ofst, s)))
    s <- s[unique_indx]

    n_strata <- length(object[["basehaz"]][[1]])
    strata_sizes <- sapply(object[["basehaz"]][[1]],
                           FUN = function(sj) sum(s == sj))
    strata_etimes <- sapply(object[["basehaz"]][-1], FUN = nrow)
    total_times <- sum(strata_sizes * strata_etimes)
  } else{
    unique_indx <- !duplicated(as.data.frame(cbind(X, ofst)))
    s <- rep(object[["basehaz"]][[1]][1], sum(unique_indx))
    n_strata <- 1
    strata_etimes <- nrow(object[["basehaz"]][[2]])
    total_times <- sum(unique_indx) * strata_etimes
  }
  X <- X[unique_indx, , drop = FALSE]
  ofst <- ofst[unique_indx]
  n <- nrow(X)

  # get labels for plot
  rows <- which(complete_cases)[unique_indx]
  maindf_names <- c(apply(mf[1, , drop = FALSE], MARGIN = 1, FUN = names))
  pltlabels <- c(apply(mf[rows, , drop = FALSE], MARGIN = 1,
                       FUN = function(z) {
                         paste0(maindf_names, z, sep = "", collapse = ", ")
                       }))
  if (n_strata > 1){
    keep_sdfcol <- !(sdf_names %in% maindf_names)
    if (any(keep_sdfcol)){
      pltlabels <- paste0(pltlabels, ", ",
                          c(apply(strata_vars[rows, , drop = FALSE], MARGIN = 1,
                                  FUN = function(z) {
                                    paste0(sdf_names[keep_sdfcol], z[keep_sdfcol],
                                           sep = "", collapse = ", ")
                                  })))
    }
  }

  # get bhaz & std errors
  exps <- exp(c(X %*% object[["coef"]][!coef_na, "coef", drop = TRUE]) + ofst)
  event_time <- rep(NA, total_times + n)
  cumhaz <- rep(NA, total_times + n)
  se_logcumhaz <- rep(NA, total_times + n)
  subject <- rep(NA, total_times + n)
  label2 <- rep(NA, total_times + n)
  pos <- 0
  for (j in seq_len(n_strata)){
    strata_indx <- s == object[["basehaz"]][[1]][j]
    n_in_s <- sum(strata_indx)
    if (n_in_s == 0) next
    posindx <- pos + seq_len((strata_etimes[j] + 1) * n_in_s)
    pos <- pos + (strata_etimes[j] + 1) * n_in_s

    event_time[posindx] <- rep(c(object[["basehaz"]][[j + 1]][1,
                                                              "time",
                                                              drop = TRUE],
                                 object[["basehaz"]][[j + 1]][,
                                                              "time",
                                                              drop = TRUE]),
                               n_in_s)
    cumhaz[posindx] <- rep(c(0,
                             object[["basehaz"]][[j + 1]][,
                                                          "cumhaz",
                                                          drop = TRUE]),
                           n_in_s) *
      rep(exps[strata_indx], each = strata_etimes[j] + 1)
    subject[posindx] <- rep(pltlabels[strata_indx], each = strata_etimes[j] + 1)
    label2[posindx] <- rep(which(strata_indx), each = strata_etimes[j] + 1)

    if (ci == TRUE){
      var_matrix <- object[["var_matrix"]][!coef_na, !coef_na, drop = FALSE]
      se_logcumhaz[posindx] <-
        c(apply(X[strata_indx, , drop = FALSE],
                MARGIN = 1,
                FUN = function(z) {
                  sqrt(c(t(z) %*% var_matrix %*% z) +
                         c(NA,
                           object[["basehaz"]][[j + 1]][,
                                                        "var_logcumhaz",
                                                        drop = TRUE]) +
                         c(NA,
                           2 *
                             object[["basehaz"]][[j + 1]][,
                                                          colnames(X),
                                                          drop = FALSE] %*%
                             z))
                }))
    }
  }

  # generate plot
  ci_transform <- ifelse(ctype == "plain", "linear", ctype)
  if (target == "cumhaz"){
    ylab <- "Cumulative hazard"
    ests <- data.frame(x_number = label2,
                       x_label = subject,
                       time = event_time,
                       est = cumhaz,
                       se = se_logcumhaz * cumhaz,
                       lower = rep(NA, length(event_time)),
                       upper = rep(NA, length(event_time)))

    ci_transform <- ifelse(ctype == "log-log", "log", ci_transform)
  }else{
    ylab <- "Survival function"
    ests <- data.frame(x_number = label2,
                       x_label = subject,
                       time = event_time,
                       est = exp(-cumhaz),
                       se = se_logcumhaz * cumhaz * exp(-cumhaz),
                       lower = rep(NA, length(event_time)),
                       upper = rep(NA, length(event_time)))
  }


  if (ci == TRUE){
    if (ci_transform == "linear"){
      ests$lower <- ests$est - q * ests$se
      ests$upper <- ests$est + q * ests$se
    }else if (ci_transform == "log"){
      ests$lower <- ests$est * exp(-q * ests$se / abs(ests$est))
      ests$upper <- ests$est * exp(q * ests$se / abs(ests$est))
    }else if (ci_transform == "log-log"){
      ests$upper <- ests$est^exp(-q * ests$se /
                                   abs(ests$est * log(abs(ests$est))))
      ests$lower <- ests$est^exp(q * ests$se /
                                   abs(ests$est * log(abs(ests$est))))
    }

    startindx <- which(cumhaz == 0)
    if (target == "survival"){
      ests$lower[startindx] <- ests$lower[startindx + 1]
      ests$upper[startindx] <- pmax(1, ests$upper[startindx + 1])
    }else{
      ests$lower[startindx] <- pmin(0, ests$lower[startindx + 1])
      ests$upper[startindx] <- ests$upper[startindx + 1]
    }
  }

  plt <- with(ests,
              ggplot2::ggplot(ests,
                              ggplot2::aes(x = time,
                                           color = x_label,
                                           group = x_number)))
  if (ci == TRUE){
    plt <- with(ests,
                plt +
                  ggplot2::geom_step(ggplot2::aes(y = lower),
                                     linetype = "dashed",
                                     direction = "hv") +
                  ggplot2::geom_step(ggplot2::aes(y = upper),
                                     linetype = "dashed",
                                     direction = "hv"))
  }
  plt <- with(ests,
              plt +
                ggplot2::geom_step(ggplot2::aes(y = est), direction = "hv",
                                   key_glyph = "rect") +
                ggplot2::labs(x = "Time", y = ylab, color = "") +
                ggplot2::scale_color_viridis_d(begin = 0, end = 0.8) +
                ggplot2::coord_cartesian(xlim = c(min(ests$time) / 1.01,
                                                  max(ests$time) * 1.01),
                                         ylim = c(0, max(ests$est) * 1.01),
                                         expand = FALSE) +
                ggplot2::theme_classic() +
                ggplot2::theme(legend.position = "bottom",
                               legend.key.size = ggplot2::unit(0.9, "lines")))

  print(plt)
  invisible(list(ests = ests, plot = plt))
}

#' @rdname coxph_indtrunc
#' @export
vcov.coxph_indtrunc <- function(object, ...){
  if (is.null(object[["var_matrix"]]))
    stop("Standard errors not available for fitted model. Refit with se=TRUE.")
  object[["var_matrix"]]
}

#' @rdname coxph_indtrunc
#' @export
coef.coxph_indtrunc <- function(object, ...){
  coefs <- object[["coef"]][, "coef", drop = TRUE]
  names(coefs) <- rownames(object[["coef"]])
  coefs
}

#' @rdname coxph_indtrunc
#' @export
confint.coxph_indtrunc <- function(object, parm, level = 0.95, ...){
  if (missing(parm)) parm <- rownames(object[["coef"]])
  stopifnot(is.vector(parm), is.numeric(parm) | is.character(parm),
            is.vector(level), is.numeric(level), length(level) == 1,
            all(!is.na(level)), all(level < 1), all(level > 0))
  if (is.numeric(parm) && any((parm < 1) | (parm > nrow(object[["coef"]]))))
    stop("Numerical parm contains indices that are out of bounds")
  if (is.character(parm) && !all(parm %in% rownames(object[["coef"]])))
    stop("Character vector parm contains names that do not match coefficients")
  parm <- unique(parm)
  p <- 1 - ((1 - level) / 2)
  q <- stats::qnorm(p)
  ci <- cbind(object[["coef"]][parm, "coef", drop = TRUE] -
                (q * object[["coef"]][parm, "se", drop = TRUE]),
              object[["coef"]][parm, "coef", drop = TRUE] +
                (q * object[["coef"]][parm, "se", drop = TRUE]))
  rownames(ci) <- rownames(object[["coef"]][parm, , drop = FALSE])
  colnames(ci) <- paste0(c((1 - p), p) * 100, "%")
  ci
}

#' @rdname coxph_indtrunc
#' @export
residuals.coxph_indtrunc <- function(object, type = c("martingale", "deviance"),
                                     per_subject = type == "deviance", ...){
  type <- match.arg(type)
  per_subject <- per_subject
  stopifnot(is.logical(per_subject), is.vector(per_subject),
            length(per_subject) == 1, all(!is.na(per_subject)))
  if (type == "martingale"){
    if (per_subject == TRUE)
      resids <- c(rowsum(object[["martingale_resid"]], object[["id_indx"]]))
    else
      resids <- object[["martingale_resid"]]

  }else{
    if (per_subject == TRUE)
      resids <- object[["deviance_resid"]]
    else{
      y <- object[["cox_fit"]][["y"]]
      event_indx <- y[order(object[["sort_indx"]]) , ncol(y), drop = TRUE]
      obs_mresid <- object[["martingale_resid"]]
      obs_wt <- object[["cox_fit"]][["time_wt"]] * event_indx
      if (any(obs_wt == 0)) obs_wt[obs_wt == 0] <- 1
      resids <- sign(obs_mresid) *
        sqrt(-2 * (obs_mresid +
                     ifelse(event_indx > 0,
                            obs_wt * log(1 - obs_mresid / obs_wt),
                            0)))
    }
  }
  resids
}

#' @rdname coxph_indtrunc
#' @export
print.coxph_indtrunc <- function(x, ...){
  timewt_name <- ifelse(x[["wt_name"]] == "W-1", "none", x[["wt_name"]])
  cat(paste0("\n",
             "IPW Cox model under quasi-independent truncation.",
             "\n",
             "\n",
             "  Call: ", paste0(format(x[["call_"]]),
                                collapse = paste0("\n",
                                                  paste0(rep(" ", 8),
                                                         collapse = ""))),
             "\n",
             "\n",
             "  Model was fit on ", x[["n"]], " complete samples (",
             x[["n_miss"]], " dropped due to missingness).",
             "\n",
             "  Min event time: ", x[["eventrange"]][1],
             "\n",
             "  Max event time: ", x[["eventrange"]][2],
             "\n",
             "  ", x[["n_event"]], " observed events in the score function, ",
                  "time range [", x[["timerange"]][1], ", ",
                                  x[["timerange"]][2], "].",
             "\n",
             "  Inverse probability of non-truncation weights estimated ",
                "nonparametrically.",
             "\n",
             "  Time-varying weight component: ", timewt_name,
             "\n",
             "\n",
             "Model estimates:",
             "\n"))
  print(x[["coef"]])
  cat("\n")

  if (!is.null(x[["strata_summary"]])){
    sort_indx <- order(x[["strata_summary"]][, "strata_size", drop = TRUE],
                       decreasing = TRUE)
    strata_summary <- x[["strata_summary"]][sort_indx,
                                            c("strata",
                                              "strata_size",
                                              "unique_event_times"),
                                            drop = FALSE]
    colnames(strata_summary) <- c("stratum", "size", "unique event times")

    cat(paste0("Summary of the ", nrow(x[["strata_summary"]]), " strata:",
               "\n"))

    if (nrow(x[["strata_summary"]]) > 10){
      print(strata_summary[1:10, , drop = FALSE],
            row.names = FALSE)
      cat("  ...")
      cat("\n\n")
    }else{
      print(strata_summary, row.names = FALSE)
      cat("\n")
    }
  }

  invisible(x)
}

#' Sensitivity analysis for positivity violations in a Cox model with
#' quasi-independent double truncation.
#'
#' @description
#' Computes inverse probability weighted sensitivity analysis estimators for the
#' Cox model using nonparametric weights (see [npmle()]).
#' Given a grid of values for the potential amount of baseline probability mass
#' lost due to zero sampling probability, shows how a positivity violation could
#' influence the results of a fitted Cox model.
#'
#' @param formula A model formula with a `Survdt` response.
#'   Subjects with missing values are discarded.
#'   Time-varying covariates are not supported.
#'   See [Survdt()].
#' @param data An optional data frame used to evaluate `formula` and
#'   `strata_formula`.
#' @param strata_formula An optional one-sided formula with stratification
#'   variables on the right hand side, for stratified Cox models.
#' @param trunc_mass A numerical vector containing the grid of truncated
#'   baseline mass values to use in the sensitivity analysis.
#' @param se Whether to calculate (robust) standard errors (default `TRUE`).
#' @param level Confidence level for constructing sensitivity intervals
#'   (default 0.95).
#' @param robust_infl A square matrix of influence values for the event weight,
#'   with one column per individual and one row per follow up time, both in the
#'   order that they appear in the data.
#'   Only used if `ipw_type` is a function and `se`=`TRUE`.
#' @param max_iter Maximum number of iterations before non-convergence
#'   is declared at a given truncated mass (default 100).
#' @param error Tolerance for determining numerical convergence of coefficient
#'   estimates (default `1e-7`).
#' @param beta_init Optional numerical vector with initial values for each
#'   coefficient estimate.
#' @param center_covariates Whether to center the covariates before the
#'   sensitivity analysis (default `FALSE`).
#'   Covariates with values of 0/1 are not centered.
#' @param x A `coxph_pos_indtrunc` object.
#' @param hazratio Whether to plot the hazard ratios exp(beta) and their
#'   sensitivity intervals instead of the regression coefficients beta (default
#'   `FALSE`).
#' @param dropNA Whether to remove truncation masses where the estimator did not
#'   converge, resulting in missing values, from the plot (default `TRUE`).
#'
#' @inheritParams coxph_indtrunc
#'
#' @details
#' Consider a sample of doubly truncated data with event times \eqn{T_i},
#' truncation times \eqn{L_i} and \eqn{R_i}, and covariates \eqn{X_i}.
#'
#' A positivity violation occurs when the sampling probability
#' \eqn{P(L<T \leq R|T=t)} is zero for \eqn{t\leq a} and \eqn{t > b}, where the
#' observation bounds \eqn{a}
#' and \eqn{b} lie within the event time support.
#' The truncated baseline probability mass is defined as the amount of probability
#' mass right-truncated by the positivity violation at the baseline covariate
#' level, conditional on not being left-truncated by the positivity violation,
#' i.e. \eqn{P(T> b|X=0,T > a)}.
#' Note that the interpretation of this truncated mass depends on how the baseline
#' covariate value is defined, e.g. whether the covariates are centered.
#'
#' At each specified truncated mass, [positivity_sens_indtrunc()] fits a Cox model
#' that accounts for the positivity violation through an adjustment to the
#' at-risk process (see Vazquez and Xie, 2025).
#' This modified at-risk process is inserted into the weighted score function from
#' [coxph_indtrunc()].
#'
#' The end result of the sensitivity analysis is a series of adjusted coefficient
#' estimators at the grid of truncated masses, which show how sensitive the Cox
#' model estimates are to potential positivity violations, as well as sensitivity
#' intervals giving a range of plausible values for the coefficients at the
#' user-supplied confidence `level` for each truncated mass.
#'
#' In general, the sensitivity analysis estimators may not converge at
#' some truncated mass values.
#' Usually this is due to large weights, near-zero risk scores
#' \eqn{\exp(X_i^T\beta)}, and/or truncated masses too close to one.
#' Some options to consider are:
#' * Changing the baseline covariate value by shifting the covariates or reordering
#'   factor levels.
#' * Using time-varying weights (see `ipw_type`).
#' * Restricting the time window in the score function through `restr_times`
#'   (see below).
#' * Altering the range and step size of `trunc_mass`.
#'
#' @inheritSection coxph_indtrunc Stratified Cox model
#'
#' @inheritSection coxph_indtrunc Restricted time window in score function
#'
#' @returns
#' [positivity_sens_indtrunc()] returns an object of class `coxph_pos_indtrunc`,
#' i.e. a list with the following components:
#' \item{n}{The complete case sample size.}
#' \item{n_miss}{The number of individuals removed due to missing data.}
#' \item{n_event}{The number of observed events used in the score function.}
#' \item{level}{The user-specified confidence level for constructing sensitivity
#'   intervals.}
#' \item{trunc_mass}{The truncated baseline masses supplied in `trunc_mass`
#'   (sorted).
#'   The baseline covariate values are provided in `col_centerings`.}
#' \item{coef}{A matrix of coefficient estimates with `j`th column corresponding
#'   to `trunc_mass[j]`.
#'   Missing values indicate non-convergence or collinearity.}
#' \item{se}{A matrix of standard errors with `j`th column corresponding to
#'   `trunc_mass[j]`.}
#' \item{lower,upper}{Matrices containing the lower and upper sensitivity
#'   interval endpoints, with `j`th column corresponding to `trunc_mass[j]`.}
#' \item{timerange}{A vector with the minimum and maximum times used in the
#'   score function.}
#' \item{eventrange}{A vector with the unrestricted range of observed event times.}
#' \item{wt_name}{A string giving the name of the weights used in `ipw_type`.}
#' \item{col_centerings}{A numerical vector with the values used to center the
#'   columns of the design matrix if `center_covariates`=`TRUE`.
#'   Otherwise, a vector of zeros.}
#' \item{strata_summary}{Some summary statistics for each stratum, computed prior
#'   to applying `restr_times`.}
#' \item{call_}{The call to [positivity_sens_indtrunc()].}
#'
#' The `print` method for `coxph_pos_indtrunc` objects displays a summary of the
#' sensitivity analysis results, while the `plot` method returns a visualization
#' of the estimators and their sensitivity intervals across different truncated
#' mass values, generated through [ggplot2::ggplot()].
#'
#' @export
#'
#' @seealso
#' * [coxph_indtrunc()]: fitting a Cox model under quasi-independent truncation.
#' * `vignette("cox-regression-quasiindep", package = "survdt")`: an illustration
#'   of this sensitivity analysis applied to the [aids] data, if vignettes were
#'   built when installing [survdt].
#'
#' @examples
#' sensfit <- positivity_sens_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
#'                                     data = aids,
#'                                     trunc_mass = seq(0, 0.11, by = 0.02),
#'                                     ipw_type = "W-asurv")
#' sensfit
#' plot(sensfit)
#'
#' @references
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
positivity_sens_indtrunc <- function(formula, data = NULL, strata_formula = NULL,
                                     trunc_mass = seq(0, 0.9, by = 0.1),
                                     ipw_type = c("W-asurv", "W-surv", "W-1", "W-a"),
                                     se = TRUE, level = 0.95, restr_times = NULL,
                                     npmle_fit = NULL,
                                     robust_infl = NULL, max_iter = 100,
                                     error = 1e-7, beta_init = NULL,
                                     center_covariates = FALSE,
                                     simple_labels = FALSE){
  fn_call <- match.call()

  # check input
  if (missing(formula))
    stop('\n  argument "formula" is missing, with no default')
  stopifnot(inherits(formula, "formula"))
  if (length(formula) < 3) stop("The formula is missing a LHS or a RHS")
  if (!is.null(data) && is.matrix(data)) data <- as.data.frame(data)
  stopifnot(is.null(data) || is.data.frame(data))
  if (!is.null(strata_formula)){
    stopifnot(inherits(strata_formula, "formula"))
    if (length(strata_formula) != 2)
      stop("The strata formula should have the form: ~var1 + var2 + ...")
    strata_vars <- stats::model.frame(stats::update(strata_formula, ~.-1),
                                      data = data,
                                      na.action = stats::na.pass)
    if (attr(attr(strata_vars, "terms"), "response") != 0)
      stop("The strata formula should not have a response variable")
    if (length(strata_vars) == 0)
      stop("strata formula is missing stratification variables on RHS")
    complete_cases_s <- rowSums(is.na(strata_vars)) == 0
    factor_names <- c(apply(strata_vars[1, , drop = FALSE],
                            MARGIN = 1, FUN = names))
    stopifnot(is.logical(simple_labels), is.vector(simple_labels),
              length(simple_labels) == 1, all(!is.na(simple_labels)))
    if (simple_labels == TRUE) factor_names <- rep("", length(factor_names))
    s <- apply(strata_vars, MARGIN = 1, FUN = function(x) {
      paste0(factor_names, x, sep = "", collapse = ", ")
    })
  }else{
    complete_cases_s <- TRUE
    s <- NULL
  }

  mf <- stats::model.frame(stats::update(formula, ~.+1), data = data,
                           na.action = stats::na.pass)
  if (attr(attr(mf, "terms"), "response") == 0)
    stop("formula is missing a response")
  if (sum(sapply(mf, FUN = function(x) any(is.infinite(x)))) > 0)
    stop("Infinite values occurred when evaluating 'formula'")
  y <- mf[[attr(attr(mf, "terms"), "response")]]
  ofst <- stats::model.offset(mf)
  if (is.Survdt2(y))
    stop("Response should be a Survdt object with one observation",
         " per subject\n (not Survdt2, time-varying covariates are",
         " not supported).\n See ?survdt::survdt2to1")
  stopifnot(is.Survdt(y))
  if (!is.null(s) && (length(s) != length(y)))
    stop("Strata formula terms and main formula terms have different lengths")


  if (is.null(restr_times)) restr_times <- c(NA_real_, NA_real_)
  stopifnot(is.vector(trunc_mass), is.numeric(trunc_mass),
            all(!is.na(trunc_mass)), all(trunc_mass < 1), all(trunc_mass>=0),
            length(trunc_mass) != 0,
            (is.vector(ipw_type) && is.character(ipw_type)) ||
              is.function(ipw_type),
            is.logical(se), is.vector(se), length(se) == 1, all(!is.na(se)),
            is.numeric(level), is.vector(level), length(level) == 1,
            all(!is.na(level)), is.numeric(restr_times), is.vector(restr_times),
            is.null(npmle_fit) || is.npmle(npmle_fit),
            is.null(robust_infl) || is.matrix(robust_infl),
            is.numeric(max_iter), is.vector(max_iter), length(max_iter) == 1,
            all(!is.na(max_iter)), max_iter > 1,
            is.numeric(error), is.vector(error), length(error) == 1,
            all(!is.na(error)), error > 0,
            is.null(beta_init) ||
              (is.numeric(beta_init) && is.vector(beta_init)),
            is.logical(center_covariates), is.vector(center_covariates),
            length(center_covariates) == 1, all(!is.na(center_covariates)))
  stopifnot(length(restr_times) == 2, 0 < level, level < 1)
  trunc_mass <- sort(trunc_mass)
  max_iter <- floor(max_iter)

  # missing values
  n_init <- length(y)
  complete_cases <- (rowSums(is.na(mf)) == 0) & complete_cases_s
  n_miss <- sum(!complete_cases)
  n <- sum(complete_cases)
  if (n == 0) stop("All individuals have missing values")
  X <- stats::model.matrix(mf, data = data)
  X <- X[, -which(attr(X, "assign") == 0), drop = FALSE]
  if (ncol(X) == 0) stop("No covariates provided in RHS of formula")

  if (!is.null(s)) {
    if (length(unique(s)) != length(unique(s[complete_cases])))
      warning("Dropping strata with no complete cases")
    s <- s[complete_cases]
    s <- factor(s, levels = unique(s[order(c(strata_vars[complete_cases, 1,
                                                         drop = TRUE])[1:n])]))
    sort_indx <- order(s, y[complete_cases, "time", drop = TRUE])
    s <- s[sort_indx]
    strata_vals <- unique(s)
    n_strata <- length(strata_vals)
    if (n_strata < 2)
      stop(paste0("Only one stratum in complete cases.\n Either update the",
                  " strata formula or drop it altogether"))

    n_unique_strata <- stats::aggregate(y[, "time", drop = TRUE] ~ s,
                                        FUN = function(x) length(unique(x)))
    small_strata <- n_unique_strata[n_unique_strata[, 2, drop = TRUE] < 2, 1,
                                    drop = TRUE]
    if (length(n_unique_strata) > 5)
      n_unique_strata <- c(as.character(n_unique_strata[1:5]), "...")
    if (length(small_strata) > 0)
      warning(paste0("Less than two unique event times found in stratum: ",
                     paste0(small_strata, collapse = "; ")))
    strata_summary <- data.frame(strata = n_unique_strata[, 1, drop = TRUE],
                                 strata_size =
                                   sapply(n_unique_strata[, 1, drop = TRUE],
                                          FUN = function(x) sum(s == x)),
                                 unique_event_times =
                                   n_unique_strata[, 2, drop = TRUE])
  }else{
    sort_indx <- order(y[complete_cases, "time", drop = TRUE])
    strata_vals <- NULL
    n_strata <- 1
    strata_summary <- NULL
  }
  sorted_cc <- which(complete_cases)[sort_indx]
  y <- y[sorted_cc, ]
  X <- X[sorted_cc, , drop = FALSE]
  if (is.null(ofst)){
    ofst <- rep(0, n)
  }else{
    ofst <- ofst[sorted_cc]
  }

  # time window
  eventrange <- range(y[, "time", drop = TRUE])
  time_lims <- eventrange
  if (!is.na(restr_times[1]) && (restr_times[1] >= time_lims[2]))
    stop("restr_times[1] must be less than the maximum observed event time")
  if (!is.na(restr_times[2]) && (restr_times[2] <= time_lims[1]))
    stop("restr_times[2] must be greater than the minimum observed event time")
  if (all(!is.na(restr_times)) && (restr_times[1] >= restr_times[2]))
    stop("Must have restr_times[1]<restr_times[2]")
  test_window <- ifelse(is.na(restr_times), time_lims,
                        c(max(restr_times[1], time_lims[1]),
                          min(restr_times[2], time_lims[2])))
  ltrunc_indx <- y[, "time", drop = TRUE] < test_window[1]
  rcens_indx <- test_window[2] < y[, "time", drop = TRUE]
  n_events <- sum(!ltrunc_indx & !rcens_indx)
  if (n_events == 0) stop("No observed event times in restricted window")

  # initial coefficient value
  if (is.null(beta_init)) beta_init <- rep(0, ncol(X))
  stopifnot(all(!is.na(beta_init)), all(!is.infinite(beta_init)))
  if (length(beta_init) != ncol(X))
    stop(paste0("Length of beta_init does not match the number of predictors (",
                ncol(X), ")"))

  # check custom weights
  if (is.character(ipw_type)){
    ipw_type <- match.arg(ipw_type)
    wt_name <- ipw_type
  }else{
    time_wt <- ipw_type(y[, "time", drop = TRUE])

    if (length(time_wt) != length(y))
      stop("Length of output from function ipw_type does not match input")
    if (any(is.na(time_wt) | is.infinite(time_wt)))
      stop("Missing or infinite values found in output of function ipw_type")
    if (any(time_wt <= 0))
      stop("Function ipw_type did not return strictly positive values")

    if (se == TRUE){
      if (is.null(robust_infl))
        stop("Must supply robust_infl for custom event weights when se=TRUE")
      if ((nrow(robust_infl) != n_init) || (ncol(robust_infl) != n_init)){
        stop("Matrix robust_infl should have one row & column per individual")
      }
      robust_infl <- robust_infl[sorted_cc, sorted_cc, drop = FALSE]
      if (any(is.na(robust_infl) | is.infinite(robust_infl)))
        stop("Missing or infinite values found in robust_infl")
    }
    wt_name <- paste0("user-supplied function ", format(substitute(ipw_type)))
  }

  # npmle
  if (!is.null(npmle_fit)){
    stopifnot(is.npmle(npmle_fit))
    if (nrow(npmle_fit[["masses"]]) != n)
      stop(paste0("npmle_fit sample size (", nrow(npmle_fit[["masses"]]), ") ",
                  "does not match the number of complete cases ",
                  "in the model frame (", n, ")"))
    if (any(abs(y[, "time", drop = TRUE] -
                npmle_fit[["masses"]]$time[sort_indx]) >
            1e-9))
      stop("Follow up times in formula response and npmle_fit do not match")

    if (se == FALSE){
      if (is.null(npmle_fit[["ainv"]])){
        npmle_fit <- npmle(with(npmle_fit[["masses"]],
                                Survdt(time, ltrunc, rtrunc)),
                           se = FALSE, ainv = TRUE, infl = FALSE, max_iter = 0,
                           init = npmle_fit[["masses"]]$f,
                           convergence_warning = FALSE,
                           check_identifiability = FALSE)
      }
      ipw <- npmle_fit[["ainv"]][sort_indx, "ainv", drop = TRUE]
    }else{
      if (is.null(npmle_fit[["infl_ainv"]])){
        infl_vals <- npmle_infl(npmle_fit[["masses"]], ainv = TRUE,
                                L_F = npmle_fit[["infl_cdf"]],
                                already_sorted = FALSE)
        npmle_fit$infl_cdf <- infl_vals$L_F[, sort_indx, drop = FALSE]
        npmle_fit$infl_ainv <- infl_vals$L_ainv[, sort_indx, drop = FALSE]
        npmle_fit$infl_ainv <- npmle_fit[["infl_ainv"]][sort_indx, ,
                                                        drop = FALSE]
        ipw <- 1 / infl_vals$a[sort_indx]
        infl_vals <- NULL
      }else{
        npmle_fit$infl_cdf <- npmle_fit[["infl_cdf"]][, sort_indx,
                                                      drop = FALSE]
        npmle_fit$infl_ainv <- npmle_fit[["infl_ainv"]][, sort_indx,
                                                        drop = FALSE]
        npmle_fit$infl_ainv <- npmle_fit[["infl_ainv"]][sort_indx, ,
                                                        drop = FALSE]
        ipw <- npmle_fit[["ainv"]][sort_indx, "ainv", drop = TRUE]
      }
    }
  }else{
    npmle_fit <- npmle(y, ainv = TRUE, se = se, infl = se)
    if (!npmle_fit[["converged"]]) stop("Fitted npmle did not converge")
    ipw <- npmle_fit[["ainv"]][, "ainv", drop = TRUE]
  }

  # time-varying weights
  if (is.character(ipw_type)){
    robust_infl <- NULL
    calculate_infl <- se == TRUE
    if (ipw_type == "W-1"){
      time_wt <- rep(1, n)
    }else if (ipw_type == "W-a"){
      time_wt <- 1 / ipw
      if (calculate_infl == TRUE){
        robust_infl <- apply(npmle_fit[["infl_ainv"]], MARGIN = 2,
                             FUN = function(x) {-x / (ipw^2)})
      }
    }else{
      cdf_time_match <- pmin(findInterval(y[, "time", drop = TRUE],
                                          npmle_fit[["cdf"]][,
                                                             "time",
                                                             drop = TRUE],
                                          left.open = TRUE) + 1,
                             nrow(npmle_fit[["cdf"]]))
      lcontsurv <- c(1,
                     1 - npmle_fit[["cdf"]][,
                                            "cdf",
                                            drop = TRUE])[cdf_time_match]

      if (ipw_type == "W-surv"){
        time_wt <- lcontsurv
        if (calculate_infl == TRUE){
          robust_infl <- -rbind(0,
                                npmle_fit[["infl_cdf"]])[cdf_time_match,
                                                         ,
                                                         drop = FALSE]
        }
      }else if (ipw_type == "W-asurv"){
        time_wt <- lcontsurv / ipw
        if (calculate_infl == TRUE){
          robust_infl <- sapply(seq_len(n), FUN = function(i){
            (-npmle_fit[["infl_ainv"]][, i] * lcontsurv / (ipw^2)) -
              (c(0,
                 npmle_fit[["infl_cdf"]][,
                                         i,
                                         drop = TRUE])[cdf_time_match] / ipw)
          })
        }
      }
    }
  }
  time_wt <- time_wt + 1e-7

  # fit sensitivity analysis estimators
  p <- ncol(X)
  col_centerings <- rep(0, p)
  if (center_covariates == TRUE){
    col_centerings <- c(apply(X, MARGIN = 2, FUN = function(x) {
      ifelse(all(x %in% c(0, 1)), 0, mean(x))
    }))
    X <- scale(X, center = col_centerings, scale = FALSE)
  }
  var_names <- colnames(X)
  if (is.null(var_names)) var_names <- paste0("X", 1:p)
  names(col_centerings) <- var_names

  coef_ests <- matrix(NA, ncol = length(trunc_mass), nrow = p)
  stderrs <- matrix(NA, ncol = length(trunc_mass), nrow = p)
  for (j in seq_along(trunc_mass)){
    ests <- tryCatch(coxph_rtrunc(y = y, X = X, s = s, ofst = ofst,
                                  ipw = ipw, time_wt = time_wt,
                                  trunc_mass = trunc_mass[j],
                                  ltrunc_indx = ltrunc_indx,
                                  rcens_indx = rcens_indx,
                                  max_iter = max_iter, error = error,
                                  beta_init = beta_init,
                                  ipw_infl = npmle_fit[["infl_ainv"]],
                                  time_wt_infl = robust_infl),
                     error = function(e) {
                       paste0("\n  Sensitivity analysis failed at ",
                              "trunc_mass ", trunc_mass[j],
                              " with error message ",
                              "\n  '",
                              e$message,
                              "'\n  ",
                              ifelse(is.null(e$call), "",
                                     paste0("in ", e$call, ".\n")),
                              "\n",
                              "See ?positivity_sens_indtrunc for ",
                              "suggestions to improve convergence.")
                     })

    if ((length(ests) == 1) && is.character(ests)){
      warning(ests)
      ests <- NULL
    }

    if (!is.null(ests) && any(!is.na(ests$coef))){
      coef_ests[, j] <- ests$coef
      stderrs[, j] <- ests$se
      beta_init <- ifelse(is.na(ests$coef), 0, ests$coef) # improve convergence
    }else{
      break
    }
  }

  # sensitivity intervals
  q <- stats::qnorm(1 - (1 - level) / 2)
  if (length(trunc_mass) == 1){
    lower <- coef_ests - q * stderrs
    upper <- coef_ests + q * stderrs
  }else{
    lower <- t(apply(coef_ests - q * stderrs, MARGIN = 1, FUN = cummin))
    upper <- t(apply(coef_ests + q * stderrs, MARGIN = 1, FUN = cummax))
  }

  rownames(coef_ests) <- var_names
  rownames(stderrs) <- var_names
  rownames(lower) <- var_names
  rownames(upper) <- var_names
  res <- list(n = n, n_miss = n_miss, n_event = n_events, level = level,
              trunc_mass = trunc_mass, coef = coef_ests, se = stderrs,
              lower = lower, upper = upper, timerange = test_window,
              eventrange = eventrange, wt_name = wt_name,
              col_centerings = col_centerings, strata_summary = strata_summary,
              call_ = fn_call)
  class(res) <- "coxph_pos_indtrunc"
  res
}

# internal function to extract data from formulas
prep_data_for_fit <- function(formula, data = NULL, strata_formula = NULL,
                              simple_labels = FALSE, drop_NA = TRUE,
                              check_post_event = TRUE, min_time = "sample",
                              strata_levels = NULL, ...){
  stopifnot(inherits(formula, "formula"))
  if (length(formula) < 3) stop("The formula is missing a LHS or a RHS")
  if (!is.null(data) && is.matrix(data)) data <- as.data.frame(data)
  stopifnot(is.null(data) || is.data.frame(data))
  if (!is.null(strata_formula)){
    stopifnot(is.logical(simple_labels), is.vector(simple_labels),
              length(simple_labels) == 1, all(!is.na(simple_labels)))
    stopifnot(inherits(strata_formula, "formula"))
    if (length(strata_formula) != 2)
      stop("The strata formula should have the form: ~var1 + var2 + ...")
    strata_vars <- stats::model.frame(stats::update(strata_formula, ~.-1),
                                      data = data, na.action = stats::na.pass)
    if (attr(attr(strata_vars, "terms"), "response") != 0)
      stop("The strata formula should not have a response variable")
    if (length(strata_vars) == 0)
      stop("The strata formula is missing stratification variables on RHS")
    complete_cases_s <- rowSums(is.na(strata_vars)) == 0
    if (!drop_NA && any(!complete_cases_s))
      stop("Missing values found in strata")
    factor_names <- c(apply(strata_vars[1, , drop = FALSE],
                            MARGIN = 1, FUN = names))
    if (simple_labels == TRUE) factor_names <- rep("", length(factor_names))
    s <- apply(strata_vars, MARGIN = 1, FUN = function(x) {
      paste0(factor_names, x, sep = "", collapse = ", ")
    })
  }else{
    complete_cases_s <- TRUE
    s <- NULL
  }

  mf <- stats::model.frame(stats::update(formula, ~.+1), data = data,
                           na.action = stats::na.pass)
  if (attr(attr(mf, "terms"), "response") == 0)
    stop("formula is missing a response")
  if (sum(sapply(mf, FUN = function(x) any(is.infinite(x)))) > 0)
    stop("Infinite values occurred when evaluating 'formula'")
  y <- mf[[attr(attr(mf, "terms"), "response")]]
  ofst <- stats::model.offset(mf)
  stopifnot(is.Survdt(y) || is.Survdt2(y))
  if (!is.null(s) && (length(s) != length(y)))
    stop("Strata formula terms and main formula terms have different lengths")
  complete_cases <- (rowSums(is.na(mf)) == 0) & complete_cases_s
  time_col <- "time"
  if (is.Survdt2(y)){
    time_col <- "stop"
    complete_cases <- !(y[, "id", drop = TRUE] %in%
                          y[!complete_cases, "id", drop = TRUE])
    id <- y[, "id", drop = TRUE]
    event_indx <- y[, "event", drop = TRUE] == 1

    # check observation windows
    event_wndws <- y[complete_cases & (y[, "event", drop = TRUE] == 1), ]
    if (check_post_event &&
        any(y[complete_cases, "start", drop = TRUE] >=
            event_wndws[match(id[complete_cases],
                              event_wndws[, "id", drop = TRUE]),
                        "stop", drop = TRUE]))
      stop(paste0("Observation windows for a given id should not extend past",
                  " the event time"))

    min_required_time <- min_time
    if (min_required_time == "sample") {
      min_required_time <- min(event_wndws[, "stop", drop = TRUE])
    }
    starts_before_min_time <- y[complete_cases, "start", drop = TRUE] <
      min_required_time
    if (any(stats::aggregate(starts_before_min_time ~ id[complete_cases],
                             FUN = sum)[, 2] == 0))
      stop(paste0("Observation windows for any given id should extend back past",
                  " the ",
                  ifelse(min_time == "sample",
                         "minimum observed event time",
                         paste0("minimum time ", min_time))))
    complete_id_nums <-
      as.numeric(unique(factor(id, levels = unique(id))[complete_cases]))
    id <- as.numeric(factor(id, levels = unique(id)))
  }else{
    id <- seq_len(length(y))
    complete_id_nums <- id[complete_cases]
    event_indx <- rep(TRUE, length(y))
  }
  match_id_indx <- match(id, complete_id_nums)

  # missing values
  n_init <- length(unique(id))
  if (!drop_NA && any(!complete_cases))
    stop("Missing values found evaluating model formula")
  n <- length(unique(id[complete_cases]))
  if (n == 0) stop("All individuals have missing values")
  X <- stats::model.matrix(mf, data = data)
  X <- X[, -which(attr(X, "assign") == 0), drop = FALSE]
  if (ncol(X) == 0) stop("No covariates provided in RHS of formula")

  if (!is.null(s)) {
    if (length(unique(s)) != length(unique(s[complete_cases])))
      warning("Dropping strata with no complete cases")
    s <- s[complete_cases]

    if (is.null(strata_levels)) {
      strata_levels <- unique(s[order(c(strata_vars[complete_cases, 1,
                                                    drop = TRUE])[1:n])])
    }

    s <- factor(s, levels = strata_levels)
    sort_indx <- order(s, y[complete_cases, time_col, drop = TRUE])
  }else{
    sort_indx <- order(y[complete_cases, time_col, drop = TRUE])
  }
  sorted_cc <- which(complete_cases)[sort_indx]
  if (is.null(ofst)) ofst <- rep(0, length(sorted_cc))

  # s may be NULL; ofst is numerical vector
  list(y = y, X = X, ofst = ofst, s = s, n = n, n_init = n_init,
       complete_id_nums = complete_id_nums, id = id,
       match_id_indx = match_id_indx, sorted_cc = sorted_cc,
       sort_indx = sort_indx, time_col = time_col,
       complete_cases = complete_cases, complete_cases_s = complete_cases_s,
       event_indx = event_indx)
}

#' @rdname positivity_sens_indtrunc
#' @export
print.coxph_pos_indtrunc <- function(x, ...){
  timewt_name <- ifelse(x[["wt_name"]] == "W-1", "none", x[["wt_name"]])
  cat(paste0("\n",
             "Sensitivity analysis for positivity violations in a Cox model",
             " under quasi-\n independent truncation.",
             "\n",
             "\n",
             "  Call: ", paste0(format(x[["call_"]]),
                                collapse = paste0("\n",
                                                  paste0(rep(" ", 8),
                                                         collapse = ""))),
             "\n",
             "\n",
             "  Model was fit on ", x[["n"]], " complete samples (",
             x[["n_miss"]], " dropped due to missingness).",
             "\n",
             "  Min event time: ", x[["eventrange"]][1],
             "\n",
             "  Max event time: ", x[["eventrange"]][2],
             "\n",
             "  ", x[["n_event"]], " observed events in the score function, ",
             "time range [", x[["timerange"]][1], ", ",
             x[["timerange"]][2], "].",
             "\n",
             "  Inverse probability of non-truncation weights estimated ",
             "nonparametrically.",
             "\n",
             "  Time-varying weight component: ", timewt_name,
             "\n",
             "\n",
             "Sensitivity analysis estimates:",
             "\n"))

  coefmat <- x[["coef"]]
  colnames(coefmat) <- paste0("trunc_mass=", x[["trunc_mass"]])
  print(coefmat)
  cat("\n")

  if (!is.null(x[["strata_summary"]])){
    sort_indx <- order(x[["strata_summary"]][, "strata_size", drop = TRUE],
                       decreasing = TRUE)
    strata_summary <- x[["strata_summary"]][sort_indx,
                                            c("strata",
                                              "strata_size",
                                              "unique_event_times"),
                                            drop = FALSE]
    colnames(strata_summary) <- c("stratum", "size", "unique event times")

    cat(paste0("Summary of the ", nrow(x[["strata_summary"]]), " strata:",
               "\n"))

    if (nrow(x[["strata_summary"]]) > 10){
      print(strata_summary[1:10, , drop = FALSE],
            row.names = FALSE)
      cat("  ...")
      cat("\n\n")
    }else{
      print(strata_summary, row.names = FALSE)
      cat("\n")
    }
  }
  invisible(x)
}



#' @rdname positivity_sens_indtrunc
#' @export
plot.coxph_pos_indtrunc <- function(x, parm, level, hazratio = FALSE,
                                    dropNA = TRUE, ...){
  if (missing(parm)) parm <- rownames(x[["coef"]])
  if (missing(level)) level <- x[["level"]]
  stopifnot(is.vector(parm), is.numeric(parm) | is.character(parm),
            is.numeric(level), is.vector(level), all(!is.na(level)),
            length(level) == 1, all(level > 0), all(level < 1),
            is.vector(hazratio), is.logical(hazratio), length(hazratio) == 1,
            all(!is.na(hazratio)), is.vector(dropNA), is.logical(dropNA),
            length(dropNA) == 1, all(!is.na(dropNA)))

  if (is.numeric(parm) && any((parm < 1) | (parm > nrow(x[["coef"]]))))
    stop("Numerical parm contains indices that are out of bounds")
  if (is.character(parm) && !all(parm %in% rownames(x[["coef"]])))
    stop("Character vector parm contains names that do not match coefficients")
  parm <- unique(parm)

  q <- stats::qnorm(1-(1-level)/2)
  n_mass <- ncol(x[["coef"]])
  if (level != x[["level"]]){
    if (n_mass == 1){
      x$lower <- x[["coef"]] - q * x[["se"]]
      x$upper <- x[["coef"]] + q * x[["se"]]
    }else{
      x$lower <- t(apply(x[["coef"]] - q * x[["se"]], MARGIN = 1, FUN = cummin))
      x$upper <- t(apply(x[["coef"]] + q * x[["se"]], MARGIN = 1, FUN = cummax))
    }
  }

  ests <- data.frame(trunc_mass = rep(x[["trunc_mass"]], each = length(parm)),
                     coef = c(x[["coef"]][parm, ]),
                     se = c(x[["se"]][parm, ]),
                     lower = c(x[["lower"]][parm, ]),
                     upper = c(x[["upper"]][parm, ]),
                     parm = rep(rownames(x[["coef"]][parm, 1, drop = FALSE]),
                                n_mass))
  ylab <- "Coefficient"
  if (hazratio == TRUE){
    ylab <- "Hazard ratio"
    ests$coef <- exp(ests$coef)
    ests$lower <- exp(ests$lower)
    ests$upper <- exp(ests$upper)
  }
  if (dropNA == TRUE) ests <- ests[!is.na(ests$coef), , drop = FALSE]
  plt <- ggplot2::ggplot(ests)
  if (any(!is.na(ests$upper))){
    if (nrow(ests) == 1){
      plt <- with(ests,
                  plt +
                    ggplot2::geom_errorbar(ggplot2::aes(x = trunc_mass,
                                                        ymin = lower,
                                                        ymax = upper)))
    }else{
      plt <- with(ests,
                  plt +
                    ggplot2::geom_line(ggplot2::aes(x = trunc_mass, y = lower),
                                       linetype = "dashed") +
                    ggplot2::geom_line(ggplot2::aes(x = trunc_mass, y = upper),
                                       linetype = "dashed"))
    }
  }
  if (nrow(ests) > 1){
    plt <- with(ests,
                plt +
                  ggplot2::geom_line(ggplot2::aes(x = trunc_mass, y = coef),
                                     linetype = "dotted"))
  }
  with(ests, plt +
         ggplot2::geom_point(ggplot2::aes(x = trunc_mass, y = coef)) +
         ggplot2::facet_wrap(~parm, scales = "free_y") +
         ggplot2::labs(x = "Truncated mass", y = ylab) +
         ggplot2::theme_classic())
}


# internal function to compute the sensitivity analysis estimator
coxph_rtrunc <- function(y, X, s = NULL, ofst = NULL, ipw, time_wt, trunc_mass,
                         ltrunc_indx = NULL, rcens_indx = NULL,
                         max_iter, error, beta_init = NULL, ipw_infl = NULL,
                         time_wt_infl = NULL){
  n <- nrow(X)
  p <- ncol(X)
  rownames(y) <- NULL
  rownames(X) <- NULL
  use_strata <- !is.null(s)
  if (is.null(ofst)) ofst <- rep(0, n)
  if (is.null(ltrunc_indx)) ltrunc_indx <- rep(FALSE, n)
  if (is.null(rcens_indx)) rcens_indx <- rep(FALSE, n)
  if (is.null(beta_init)) beta_init <- rep(0, p)

  # data sorted by event time and strata
  fit_indx <- !ltrunc_indx
  n_fit <- sum(fit_indx)
  outer_wt <- ipw * time_wt
  ofst_wt <- ofst - log(time_wt)
  y_coxph_aug <- survival::Surv(c(y[fit_indx, "time", drop = TRUE],
                                  rep(max(y[fit_indx, "time", drop = TRUE]), n)),
                                c(as.numeric(!rcens_indx[fit_indx]), rep(0, n)))
  X_aug <- rbind(X[fit_indx, , drop = FALSE], X[, , drop = FALSE])
  s_aug <- NULL
  if (use_strata) {
    s_num <- as.numeric(s)
    s_aug <- c(s_num[fit_indx], s_num)
  }
  ofst_aug <- c(ofst_wt[fit_indx], ofst_wt)

  offset <- stats::offset
  strata <- survival::strata
  beta_est <- beta_init
  coef_na <- is.na(beta_est)
  converged <- FALSE
  for (i in seq_len(max_iter)){
    exps <- exp(c(X %*% ifelse(coef_na, rep(0, p), beta_est)) + ofst)
    p_trunc <- trunc_mass^exps
    odds_trunc <- (p_trunc) / (1 - p_trunc)
    wt_aug <- c(outer_wt[fit_indx], outer_wt * odds_trunc)
    poswt <- c(rep(TRUE, n_fit), odds_trunc != 0)
    if (use_strata){
      augfit <- survival::coxph(y_coxph_aug ~ X_aug + offset(ofst_aug) +
                                  strata(s_aug), subset = poswt,
                                na.action = stats::na.fail, weights = wt_aug,
                                init = ifelse(coef_na, rep(0, p), beta_est),
                                robust = FALSE, ties = "breslow")
    }else{
      augfit <- survival::coxph(y_coxph_aug ~ X_aug + offset(ofst_aug),
                                subset = poswt, na.action = stats::na.fail,
                                weights = wt_aug,
                                init = ifelse(coef_na, rep(0, p), beta_est),
                                robust = FALSE, ties = "breslow")
    }
    beta_next <- stats::coef(augfit)
    coef_na_next <- is.na(beta_next)
    updated_coef_indx <- !coef_na_next & !coef_na
    if (all(coef_na_next))
      stop("Failed to update coefficients through augmented Cox model",
           call. = FALSE)

    if (any(updated_coef_indx)){
      # check convergence and exit without updating if achieved
      if (all(abs(beta_next[updated_coef_indx] -
                  beta_est[updated_coef_indx]) < error)){
        converged <- TRUE
        break
      }
    }

    beta_est <- beta_next
    coef_na <- coef_na_next
    next
  }

  if (!converged){
    stop("Algorithm reached max_iter without convergence",
         call. = FALSE)
  }

  coef_na <- is.na(beta_est)
  effective_p <- sum(!coef_na)
  stderrs <- rep(NA, p)
  if (!is.null(ipw_infl)){
    exps <- exp(c(X %*% ifelse(coef_na, rep(0, p), beta_est)) + ofst)
    p_trunc <- trunc_mass^exps
    odds_trunc <- (p_trunc) / (1 - p_trunc)
    wt_aug <- c(outer_wt[fit_indx], outer_wt * odds_trunc)
    posoddswt <- odds_trunc != 0
    poswt <- c(rep(TRUE, n_fit), posoddswt)
    if (use_strata){
      lastfit <- survival::coxph(y_coxph_aug ~ X_aug + offset(ofst_aug) +
                                   strata(s_aug), subset = poswt,
                                 na.action = stats::na.fail, weights = wt_aug,
                                 init = ifelse(coef_na, rep(0, p), beta_est),
                                 robust = FALSE, ties = "breslow",
                                 control = survival::coxph.control(iter.max = 0))
    }else{
      lastfit <- survival::coxph(y_coxph_aug ~ X_aug + offset(ofst_aug),
                                 subset = poswt, na.action = stats::na.fail,
                                 weights = wt_aug,
                                 init = ifelse(coef_na, rep(0, p), beta_est),
                                 robust = FALSE, ties = "breslow",
                                 control = survival::coxph.control(iter.max = 0))
    }

    # derivative
    adj_wts <- odds_trunc * (1 + odds_trunc) * exps^2 * ipw *
      ifelse(trunc_mass > 0, log(trunc_mass), 0)
    if (use_strata == FALSE) s <- rep(1, n)
    derivative <- matrix(0, nrow = effective_p, ncol = effective_p)
    for (stratum_val in unique(s)){
      strata_indx <- s == stratum_val
      adj1 <- c(t(adj_wts[strata_indx]) %*% X[strata_indx, !coef_na,
                                              drop = FALSE])
      adj2 <- t(X[strata_indx, !coef_na, drop = FALSE]) %*%
        diag(adj_wts[strata_indx], sum(strata_indx)) %*%
        X[strata_indx, !coef_na, drop = FALSE]

      unique_etimes <- unique(y[strata_indx & !ltrunc_indx & !rcens_indx, "time",
                                drop = TRUE])
      riskset_sums <- sapply(unique_etimes, FUN = function(ti){
        riskindx <- y[strata_indx, "time", drop = TRUE] >= ti
        time_match <- y[strata_indx, "time", drop = TRUE] == ti
        # sum over all strata members
        riskwts <- (riskindx + odds_trunc[strata_indx]) * exps[strata_indx] *
          ipw[strata_indx]
        outer_wt_derivative <- sum((outer_wt[strata_indx])[time_match])
        S0 <- sum(riskwts)
        S1 <- c(t(riskwts) %*% X[strata_indx, !coef_na, drop = FALSE])
        S2 <- t(X[strata_indx, !coef_na, drop = FALSE]) %*%
          diag(riskwts, length(riskwts)) %*%
          X[strata_indx, !coef_na, drop = FALSE]

        # contribution to derivative
        c(outer_wt_derivative *
            (((S2 + adj2) / S0) - (S1 %*% t(S1 + adj1) / S0^2)))
      })
      if (effective_p == 1)
        dentries <- sum(riskset_sums)
      else
        dentries <- rowSums(riskset_sums)

      derivative <- derivative + matrix(dentries,
                                        nrow = effective_p,
                                        ncol = effective_p)
    }

    # score variance components: {number of coefs without NA} x n
    score_terms <- matrix(0, nrow = effective_p, ncol = n)
    score_resids <- stats::residuals(lastfit, weighted = TRUE, type = "score",
                                     collapse = c(which(fit_indx),
                                                  which(posoddswt)))
    if (p == 1){
      score_indx <- as.numeric(names(score_resids))
      score_terms[1, score_indx] <- c(score_resids)
    }
    else{
      score_indx <- as.numeric(rownames(score_resids))
      score_terms[, score_indx] <- t(score_resids[, !coef_na, drop = FALSE])
    }
    score_terms <- score_terms +
      (score_terms %*% diag(1 / ipw, length(ipw)) %*% ipw_infl / n)

    if (!is.null(time_wt_infl)){
      sch_resids <- stats::residuals(lastfit,
                                     weighted = TRUE,
                                     type = "schoenfeld")
      sch_indx <- !ltrunc_indx & !rcens_indx
      score_terms <- score_terms +
        (t(sch_resids)[!coef_na, , drop = FALSE] %*%
           diag(1 / time_wt[sch_indx], sum(sch_indx)) %*%
           time_wt_infl[sch_indx, , drop = FALSE] / n)
    }

    meat <- apply(score_terms, MARGIN = 2, FUN = function(x) x %o% x)
    if (effective_p == 1){
      meat <- sum(meat)
    }else{
      meat <- rowSums(meat)
    }
    inv_derivative <- solve(derivative)
    var_est <- inv_derivative %*%
      matrix(meat, nrow = effective_p) %*%
      t(inv_derivative)
    stderrs[!coef_na] <- sqrt(diag(var_est))
  }
  list(coef = beta_est, se = stderrs)
}

#' Create a basic survival object for doubly truncated data.
#'
#' @description
#' Creates a survival object to contain the event times, left truncation times,
#' and right truncation times from a doubly truncated sample, with one observation
#' per subject.
#'
#' Typically used as the input for nonparametric analysis methods or as the
#' response variable in a regression model formula.
#'
#' @param time A vector of follow up times.
#' @param ltrunc A vector of left truncation times.
#' @param rtrunc A vector of right truncation times.
#' @param x Any R object.
#' @param i,j Row and columns index vectors. The result is a `Survdt` object if
#'   `j` is missing, otherwise `x` is treated as a matrix.
#' @param sort Whether to sort the data rows by follow up time (and `colorvar`,
#'   if provided) before plotting (default `TRUE`).
#' @param colorvar An optional vector used to color the dots in the plot. Will
#'   be coerced to factor before using.
#' @param ... Additional arguments passed to other methods.
#'
#' @details
#' For a given event time of interest \eqn{T}, a sample is doubly truncated when
#' it only includes subjects with \eqn{L_i<T_i \leq R_i} for some random
#' truncation times \eqn{L_i} and \eqn{R_i}.
#' Analyzing this type of data requires information on the \eqn{i}th subject's
#' event time \eqn{T_i}, left truncation time \eqn{L_i}, and right truncation
#' time \eqn{R_i}.
#'
#' [Survdt()] provides the basic data structure for organizing such data and
#' checking its validity in the `survdt` package.
#' Attempting to run [Survdt()] with inputs that do not satisfy \eqn{L<T \leq R}
#' will result in an error.
#'
#' @returns
#' * [Survdt()] returns an object of class `Survdt`, with columns `time`,
#'   `ltrunc`, and `rtrunc`.
#'   Indexing a `Survdt` object by rows will return another `Survdt` object,
#'   otherwise it is treated as a matrix.
#'   The length of a `Survdt` object is its sample size, and `is.na()` returns
#'   a logical index vector for the non-complete cases.
#'   Calling `as.data.frame()` on a `Survdt` object will return a data frame
#'   stripped of the `Survdt` class attribute.
#' * [is.Survdt()] returns `TRUE` if `x` inherits from the class `Survdt` and
#'   `FALSE` otherwise.
#' * The plot method for `Survdt` objects returns a simple visualization from
#'   [ggplot2::ggplot()] with x-axis time, y-axis subject, rectangles for the
#'   truncation intervals, and dots for the event times.
#'   The dots can optionally be colored by a factor variable supplied in
#'   `colorvar`.
#'   Incomplete cases are dropped from the plot (with a warning).
#'
#' @export
#'
#'
#' @seealso
#' * [npmle()], [test_samplediff_indtrunc()]: nonparametric estimation and
#'   analysis for doubly truncated data.
#' * [coxph_indtrunc()]: inverse probability weighted Cox regression.
#' * [Survdt2()]: to encode time-varying covariates; also [time_split()] for
#'   converting data from `Survdt` structure to `Survdt2`.
#'
#' @examples
#' y <- with(aids, Survdt(incu, ltrunc, rtrunc))
#' plot(y)
#' plot(y, colorvar = aids$age_gp)
#' npmle(y)
#'
Survdt <- function(time, ltrunc, rtrunc){
  # check input
  if (inherits(time, "difftime")) time <- as.numeric(time)
  if (inherits(ltrunc, "difftime")) ltrunc <- as.numeric(ltrunc)
  if (inherits(rtrunc, "difftime")) rtrunc <- as.numeric(rtrunc)

  stopifnot(is.numeric(time), is.vector(time), !any(is.infinite(time)),
            is.numeric(ltrunc), is.vector(ltrunc), !any(is.infinite(ltrunc)),
            is.numeric(rtrunc), is.vector(rtrunc), !any(is.infinite(rtrunc)))
  stopifnot(length(time) > 0, length(time) == length(ltrunc),
            length(time) == length(rtrunc))

  na_ltrunc <- is.na(ltrunc)
  na_rtrunc <- is.na(rtrunc)
  na_trunc <- na_ltrunc | na_rtrunc
  na_time <- is.na(time)
  if (any(!na_trunc) && any((ltrunc >= rtrunc)[!na_trunc]))
    stop("ltrunc must by < rtrunc")
  if (any(!na_ltrunc & !na_time) && any((ltrunc >= time)[!na_ltrunc & !na_time]))
    stop("Each event time must be >ltrunc")
  if (any(!na_rtrunc & !na_time) && any((time > rtrunc)[!na_rtrunc & !na_time]))
    stop("Each event time must be <=rtrunc")

  x <- cbind(time = time, ltrunc = ltrunc, rtrunc = rtrunc)
  class(x) <- "Survdt"
  x
}

#' @rdname Survdt
#' @export
`[.Survdt` <- function(x, i, j, ...){
  if (missing(j) && !missing(i)){
    x <- unclass(x)[i, , drop = FALSE]
    class(x) <- "Survdt"
    x
  }else{
    class(x) <- "matrix"
    NextMethod("[")
  }
}

#' @rdname Survdt
#' @export
is.na.Survdt <- function(x){
  as.vector(rowSums(is.na(unclass(x))) > 0)
}

#' @rdname Survdt
#' @export
is.Survdt <- function(x){
  inherits(x, "Survdt")
}

#' @rdname Survdt
#' @export
length.Survdt <- function(x){
  nrow(x)
}

#' @rdname Survdt
#' @export
as.data.frame.Survdt <- function(x, ...){
  as.data.frame(unclass(x), ...)
}

#' @rdname Survdt
#' @export
plot.Survdt <- function(x, colorvar = NULL, sort = TRUE, ...){
  stopifnot(is.logical(sort), is.vector(sort), all(!is.na(sort)))
  stopifnot(length(sort) == 1)
  hasc <- !is.null(colorvar)
  if (hasc) {
    colorlabel <- format(substitute(colorvar))
    colorvar <- as.factor(colorvar)
    stopifnot(length(colorvar) == length(x))
  }else
    colorlabel <- ""

  n <- length(x)
  cc_indx <- !is.na(x)
  x <- x[cc_indx, ]
  n_miss <- n - length(x)
  if (n_miss > 0){
    if (n_miss == n){
      stop("No complete observations available to plot.")
    }
    warning(paste0("omitted ", n_miss, " observations with missing values."))
  }
  if (hasc)
    colorvar <- colorvar[cc_indx]
  else
    colorvar <- rep(1, sum(cc_indx))


  if (sort == TRUE){
    sort_indx <- order(colorvar, x[, "time", drop = TRUE])

  }else{
    sort_indx <- seq_along(x[, "time", drop = TRUE])
  }

  plot_data <- cbind(rownum = seq_along(x[, "time", drop = TRUE]),
                     colorvar = colorvar[sort_indx],
                     as.data.frame(x)[sort_indx, , drop = FALSE])
  plt <- with(plot_data,
              ggplot2::ggplot(plot_data,
                              ggplot2::aes(x = time, y = rownum - 0.5)) +
                ggplot2::geom_rect(ggplot2::aes(xmin = ltrunc, xmax = rtrunc,
                                                ymin = rownum-1, ymax = rownum),
                                   fill = "gray"))
  if (hasc)
    plt <- with(plot_data,
                plt +
                  ggplot2::geom_point(ggplot2::aes(color = colorvar)) +
                  ggplot2::scale_color_viridis_d(begin = 0, end = 0.8, guide =
                          ggplot2::guide_legend(override.aes = list(size = 3))))
  else
    plt <- with(plot_data,
                plt +
                  ggplot2::geom_point())

  with(plot_data,
       plt +
         ggplot2::labs(x = "Time", y = "Subject", color = colorlabel) +
         ggplot2::coord_cartesian(expand = FALSE) +
         ggplot2::theme_classic() +
         ggplot2::theme(legend.position = "bottom"))
}

#' Split rows in a data frame into multiple observations at specified time
#' intervals
#'
#' @description
#' Splits each record in the data frame into a series of observation windows
#' based on the supplied cut times.
#' Can be used to reshape data with time-varying covariates to the appropriate
#' format for [Survdt2()].
#' The main workhorse is a modified call to [survival::survSplit()].
#'
#' @param data A data frame to be split.
#' @param cut A numeric vector of cut times.
#' @param start_name The name for the start time column to be created in the split
#'  data. If it matches a column in `data` (meaning the data was previously split)
#'  then `event_name` must also match a column in `data`.
#' @param stop_name The name of the existing event time or stop time column in
#'  `data`.
#' @param event_name The name for the event indicator column to be created in the
#'  split data. See `start_name` for special requirements.
#' @param id_name An optional name to create an id column in the split data. If
#'  this column already exists in `data` its values will be used as ids.
#'  If the data is already in (start, stop] format, do not use this argument.
#' @param origin The time that the follow up starts at. Defaults to zero.
#'
#' @details
#' A basic use case of [time_split()] is to take a data frame with one row per
#' subject and transform it into a data frame with multiple rows per subject,
#' each corresponding to a specific observation time window.
#' The event time information observed in each window is encoded by four
#' potentially new variables:
#' * `start`: the starting time for the observation window.
#' * `stop`: the end time for the observation window.
#' * `event`: equal to one when the subject's event occurred at `stop`, and zero
#'   otherwise.
#' * `id`: the label that identifies which subject the data corresponds to.
#'
#' In this use case, the name of the event time column in `data` should always be
#' provided, as well as the name for the new `id` column if one does not already
#' exist.
#' After `cut` is sorted, the \eqn{i}th subject's observation windows will be
#' \eqn{(}`origin`, `cut[1]`\eqn{]}, \eqn{(}`cut[1]`, `cut[2]`\eqn{]}, \eqn{\ldots},
#' \eqn{(}`cut[j]`, \eqn{T_i}\eqn{]} for some
#' `j`\eqn{\leq}`length(cut)`.
#'
#' Another use case of [time_split()] is to introduce additional splits in a
#' previously split data frame.
#' The names for the existing `start`, `stop`, and `event` columns should be
#' provided to [time_split()] in this case.
#' Requesting a new `id` variable for previously cut data will return an error,
#' since the new ids would not respect the prior groupings.
#'
#' @returns
#' A split data frame with (start, stop] records.
#' The new column names are taken from the input arguments of `time_split()`.
#'
#' @export
#'
#' @seealso
#' * [Survdt2()]: create survival objects with split data.
#' * [coxph_indtrunc()]: Cox regression with time-varying covariates.
#' * [survdt2to1()]: collapse data back to one row per subject.
#'
#' @examples
#' splitdat1 <- time_split(aids, cut = c(20, 40),
#'                         stop_name = "incu",
#'                         id_name = "id",
#'                         origin = -1)
#' tail(splitdat1)
#'
#' splitdat2 <- time_split(splitdat1, cut = c(10, 30),
#'                         start_name = "start",
#'                         stop_name = "incu",
#'                         event_name = "event",
#'                         id_name = "id",
#'                         origin = -1)
#' tail(splitdat2)
time_split <- function(data, cut, stop_name, start_name = "start",
                       event_name = "event", id_name = NULL, origin = 0){
  # check input
  if (is.matrix(data)) data <- as.data.frame(data)
  stopifnot(is.data.frame(data), is.numeric(cut), is.vector(cut), !any(is.na(cut)),
            is.character(start_name), is.vector(start_name), !any(is.na(start_name)),
            is.character(stop_name), is.vector(stop_name), !any(is.na(stop_name)),
            is.character(event_name), is.vector(event_name), !any(is.na(event_name)),
            is.character(start_name), is.vector(start_name), !any(is.na(start_name)),
            is.numeric(origin), is.vector(origin), !any(is.na(origin)))
  stopifnot(length(cut) > 0, length(start_name) == 1, length(stop_name) == 1,
            length(event_name) == 1, nrow(data) > 0, ncol(data) > 0)
  if (!is.null(id_name)){
    stopifnot(is.character(id_name), is.vector(id_name), !any(is.na(id_name)))
    stopifnot(length(id_name) == 1)
  }
  if (any(cut <= origin)) stop("All cut times must be >origin")

  colindx <- match(c(start_name, stop_name, event_name), colnames(data))
  if (is.na(colindx[2]))
    stop("No column in data matches stop_name (should be the stop or event time)")
  if (is.na(colindx)[1] != is.na(colindx[3]))
    stop(paste0("Must have (1) both start_name and event_name exist in data,\n",
                "or (2) neither exist in data"))
  min_time <- min(data[, stop_name, drop = TRUE], na.rm = TRUE)
  if (is.na(min_time)) stop("All stop times are NA")
  if (origin >= min_time)
    stop("Time origin must be strictly less than all stop times")

  # construct observation windows
  if (is.na(colindx[1])){
    form <- stats::reformulate(".",
                               response = paste0("Surv(",
                                                 stop_name,
                                                 ", ",
                                                 "rep(1, nrow(data))",
                                                 ")"))
  }else{
    form <- stats::reformulate(".",
                               response = paste0("Surv(",
                                                 start_name,
                                                 ", ",
                                                 stop_name,
                                                 ", ",
                                                 event_name,
                                                 ")"))
  }

  # survsplit
  Surv <- survival::Surv
  if (is.null(id_name) || (id_name %in% colnames(data))){
    res <- survival::survSplit(form, data = data, cut = cut, start = start_name,
                               zero = origin, end = stop_name, event = event_name)
  }else{
    if (!is.na(colindx[1]))
      stop("Cannot preserve ids in (start, stop] data; id_name not in data")

    res <- survival::survSplit(form, data = data, cut = cut, start = start_name,
                               zero = origin, end = stop_name, event = event_name,
                               id = id_name)


  }
  res
}

#' @keywords internal
#'
#' @section Datasets:
#' * [aids]: a doubly truncated sample of AIDS incubation times from patients
#'   with transfusion-acquired HIV.
#' * [nonprop_sample]: a simulated sample with non-proportional hazards and
#'   quasi-independent double truncation.
#'
#' @section Key functions:
#'
#' **Survival objects to hold doubly truncated data**
#' * [Survdt()]: one observation per subject.
#' * [Survdt2()]: for (start,stop] data with multiple observations per subject.
#' * [time_split()], [survdt2to1()]: convert between the two formats above.
#'
#' **Nonparametric analysis under double truncation**
#' * [npmle()]: nonparametric maximum likelihood estimation.
#'   The output has a `plot` method to plot the estimated survival function,
#'   cumulative hazard, and selection probabilities along with pointwise
#'   confidence intervals.
#'   The nonparametric maximum likelihood estimator (NPMLE) is used in the two
#'   tests that follow.
#' * [test_ignorability_indtrunc()]: test for non-ignorable sampling bias.
#' * [test_samplediff_indtrunc()]: test for any difference in event time
#'   distribution across groups.
#' * [test_quasiindep_ctau()]: test for dependence between truncation times and
#'   the event time.
#'   The NPMLE requires quasi-independence.
#'
#' **Cox regression under double truncation**
#' * [coxph_indtrunc()]: inverse probability weighted Cox regression with options
#'   for robust time-varying weights estimated nonparametrically.
#'   The output has common methods such as `confint`, `residuals`, and `vcov`.
#' * [plot_coxsurv()]: plot conditional survival functions or cumulative hazards
#'   from a fitted Cox model along with pointwise confidence intervals.
#' * [test_quasiindep_covariates()]: test for dependence between truncation times
#'   and the event time or covariates.
#'   The nonparametric inverse probability weighting used by [coxph_indtrunc()]
#'   requires quasi-independence.
#' * [positivity_sens_indtrunc()]: sensitivity analysis for positivity violations.
#'   Check how the estimates and confidence intervals may change under a range
#'   of values for the baseline probability mass potentially lost due to zero
#'   sampling probability.
#'
#' @section Vignettes:
#' If vignettes were built when installing [survdt], see
#' * `vignette("nonparametric-analysis", package = "survdt")`: an example of
#'   various nonparametric analyses applied to the [aids] data.
#' * `vignette("cox-regression-quasiindep", package = "survdt")`: an example of
#'   Cox regression and its assumption diagnostics applied to the [aids] data.
#'
#'
#' @references
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#' Simulated data with non-proportional hazards and independent double truncation
#'
#' @description
#' The event times were simulated from a Cox model with hazard function
#' \deqn{\lambda(t|x,\textit{group}) = \exp\{0.7\textit{group}\times 1(t \leq 0.6) - 0.5x\}\lambda_0(t-0.1)}
#' where \eqn{\lambda_0(t)} is the hazard function from a beta(2,2) distribution.
#' The covariates \eqn{\textit{group}} and \eqn{x} were independent bernoulli(0.5)
#' and uniform(0,1) random variates, respectively.
#'
#' The left truncation time had a uniform(0,0.6) distribution,
#' while the time between the left and right truncation time was an independent
#' uniform(0.1,0.7) random variate.
#' All times were rounded to two decimal places.
#'
#' @format
#' A data frame with 490 rows and 5 columns:
#' \describe{
#'   \item{`ltrunc`}{The left truncation time.}
#'   \item{`rtrunc`}{The right truncation time.}
#'   \item{`event_time`}{The event time.}
#'   \item{`group`}{A binary covariate with non-proportional hazards.}
#'   \item{`x`}{A continuous covariate with proportional hazards.}
#' }
"nonprop_sample"

#' AIDS incubation data
#'
#' @description
#' A doubly truncated dataset of AIDS incubation times from patients with
#' transfusion-acquired HIV collected by the Centers for Disease Control and
#' Prevention (CDC). The sample contains patients who were diagnosed between
#' the discovery of the disease (1982) and the end of study (July 1986), with
#' infection times determined retrospectively from a blood transfusion registry.
#'
#' @format
#' A data frame with 295 rows and 5 columns:
#' \describe{
#'   \item{`incu`}{The incubation time, i.e. months from HIV infection to AIDS
#'     diagnosis.}
#'   \item{`age`}{The age at HIV infection in years.}
#'   \item{`ltrunc`}{The time from HIV infection to the start of data collection
#'     (1982), in months.
#'     Note that `ltrunc` is negative for patients infected before 1982.}
#'   \item{`rtrunc`}{The time from HIV infection to the end of data collection
#'     (July 1986), in months.}
#'   \item{`age_gp`}{A factor with three `age` groups: `<=4`, `4-59`, and `>59`
#'     (the baseline level).}
#' }
#' @source
#' Wang, M. -C. (1989). A semiparametric model for randomly truncated data.
#' *Journal of the American Statistical Association*, **84**(407), 742--748.
"aids"

#' Create a survival object for use with time-varying covariates
#'
#' @description
#' Creates a survival object for doubly truncated data with the follow
#' up time for each subject split across multiple observation windows using
#' (start, stop] encoding.
#'
#' @param start A vector of start times for the observation windows.
#' @param stop A vector of stop times for the observation windows.
#' @param event A vector of 0/1 event indicators (1=event).
#' @param ltrunc A vector of left truncation times.
#' @param rtrunc A vector of right truncation times.
#' @param id A vector of subject ids.
#' @param x Any R object.
#' @param i,j Row and columns index vectors. The result is a `Survdt2` object if
#'   `j` is missing, otherwise `x` is treated as a matrix.
#' @param ... Additional arguments passed to other methods.
#'
#' @details
#' The \eqn{i}th subject's survival data consists of the left and right truncation
#' times \eqn{L_i} and \eqn{R_i} as well as a series of (start, stop] observation
#' windows \eqn{(t_{i0}}, \eqn{t_{i1}]}, \eqn{(t_{i1}}, \eqn{t_{i2}]},
#' \eqn{\ldots}, \eqn{(t_{i,j-1}}, \eqn{t_{ij}}\eqn{]} for some \eqn{j}.
#' Furthermore, each of these observation windows has an associated event indicator
#' that is equal to one if the event time is equal to the stop time, and zero
#' otherwise.
#' Each subject should have an observation window with an event, and multiple
#' events per subject are not allowed.
#' Attempting to run [Survdt2()] with inputs that do not follow this specification
#' will result in an error.
#'
#' This data structure is used for Cox regression with time-varying covariates,
#' since the subject's covariate values can vary across observation windows.
#'
#'
#' @returns
#' * [Survdt2()] returns an object of class `Survdt2`, with columns `start`,
#'   `stop`, `event`, `ltrunc`, `rtrunc`, and `id`.
#'   Indexing a `Survdt2` object by rows will return another `Survdt2` object,
#'   otherwise it is treated as a matrix.
#'   The length of a `Survdt2` object is the total number of observations across
#'   all subjects, and `is.na()` returns a logical index vector for the non-complete
#'   rows.
#'   Calling `as.data.frame` on a `Survdt2` object will return a data frame
#'   stripped of the `Survdt2` class attribute.
#' * [is.Survdt2()] returns `TRUE` if `x` inherits from the class `Survdt2` and
#'   `FALSE` otherwise.
#' * [survdt2to1()] takes a `Survdt2` object as input and returns the result from
#'   calling [Survdt()] on collapsed data with one row per `id`.
#'   The subjects in the collapsed data are sorted in the order they first appear
#'   in the `Survdt2` object.
#'   If all event indicators for a subject are zero and/or `NA`, the event time
#'   in the collapsed data is set to `NA`.
#'   Lastly, the `id` values are preserved within the row names of the `Survdt`
#'   object.
#' * The plot method for `Survdt2` first collapses the data into `Survdt`
#'   format using [survdt2to1()] and then calls [plot.Survdt()].
#'
#' @export
#'
#' @seealso
#' * [coxph_indtrunc()]: inverse probability weighted Cox regression with
#'   time-varying covariates.
#' * [Survdt()]: the more basic survival object with one row per subject.
#'
#' @examples
#' # first create (start, stop] data frame;
#' # could then apply time transformations
#' # to covariates
#' splitdat <- time_split(aids, cut = c(20, 40),
#'                        stop_name = "incu",
#'                        id_name = "id",
#'                        origin = -1)
#'
#' y1 <- with(splitdat,
#'            Survdt2(start, incu, event, ltrunc, rtrunc, id))
#' plot(y1)
#'
#' y2 <- with(aids,
#'            Survdt(incu, ltrunc, rtrunc))
#' max(abs(unclass(survdt2to1(y1)) - unclass(y2)))
#'
#'
#' coxph_indtrunc(Survdt2(start, incu, event, ltrunc, rtrunc, id) ~ age_gp,
#'                data = splitdat)
#'
Survdt2 <- function(start, stop, event, ltrunc, rtrunc, id){
  # check input
  if (inherits(start, "difftime")) start <- as.numeric(start)
  if (inherits(stop, "difftime")) stop <- as.numeric(stop)
  if (inherits(ltrunc, "difftime")) ltrunc <- as.numeric(ltrunc)
  if (inherits(rtrunc, "difftime")) rtrunc <- as.numeric(rtrunc)
  if (is.logical(event)) event <- as.numeric(event)

  stopifnot(is.numeric(start), is.vector(start), !any(is.infinite(start)),
            is.numeric(stop), is.vector(stop), !any(is.infinite(stop)),
            is.numeric(event), is.vector(event),
            is.numeric(ltrunc), is.vector(ltrunc), !any(is.infinite(ltrunc)),
            is.numeric(rtrunc), is.vector(rtrunc), !any(is.infinite(rtrunc)),
            is.factor(id) || is.vector(id), !is.list(id))
  if (!is.factor(id)) id <- as.factor(id)

  stopifnot(length(start) > 0, length(start) == length(stop),
            length(start) == length(event), length(start) == length(ltrunc),
            length(start) == length(rtrunc), length(start) == length(id),
            !any(is.na(id)), all(is.na(event) | (event %in% c(0, 1))))

  if (any(stats::aggregate(event ~ id,
                           FUN = function(z) {sum(is.na(z) | (z == 1)) < 1})[, 2]))
    stop("cannot have an id with all event=0")
  if (any(stats::aggregate(ltrunc ~ id,
                           FUN = function(z) {length(unique(z)) > 1})[, 2]))
    stop("ltrunc must be constant within id")
  if (any(stats::aggregate(rtrunc ~ id,
                           FUN = function(z) {length(unique(z)) > 1})[, 2]))
    stop("rtrunc must be constant within id")

  na_ltrunc <- is.na(ltrunc)
  na_rtrunc <- is.na(rtrunc)
  na_trunc <- na_ltrunc | na_rtrunc
  na_start <- is.na(start)
  na_stop <- is.na(stop)
  event_indx <- !is.na(event) & (event == 1)
  if (any(event_indx) && any(duplicated(id[event_indx])))
    stop("cannot have multiple events per id")
  if (any(!na_start & !na_stop) && any((start >= stop)[!na_start & !na_stop]))
    stop("must have start<stop")
  if (any(!na_trunc) && any((ltrunc >= rtrunc)[!na_trunc]))
    stop("ltrunc must be <rtrunc")
  if (any(event_indx) && any(!na_stop[event_indx] & !na_ltrunc[event_indx]) &&
      any((ltrunc[event_indx] >= stop[event_indx])[!na_stop[event_indx] &
                                                   !na_ltrunc[event_indx]]))
    stop("each event time must be >ltrunc")
  if (any(event_indx) && any(!na_stop[event_indx] & !na_rtrunc[event_indx]) &&
      any((rtrunc[event_indx] < stop[event_indx])[!na_stop[event_indx] &
                                                  !na_rtrunc[event_indx]]))
    stop("each event time must be <=rtrunc")

  id_levels <- levels(id)
  id <- as.numeric(id)
  x <- cbind(start = start, stop = stop, event = event, ltrunc = ltrunc,
             rtrunc = rtrunc, id = id)
  attr(x, "id_levels") <- id_levels

  complete_id_indx <- sapply(unique(id),
                             FUN = function(z) {!any(is.na(x[id == z, ]))})
  if (any(complete_id_indx)){
    tmpx <- x[complete_id_indx, ]
    tmpx <- tmpx[order(tmpx[, "id", drop = TRUE], tmpx[, "start", drop = TRUE]), ]
    if (any((tmpx[-nrow(tmpx), "stop", drop = TRUE] !=
             tmpx[-1, "start", drop = TRUE]) &
            (tmpx[-nrow(tmpx), "id", drop = TRUE] ==
             tmpx[-1, "id", drop = TRUE])))
      stop("(start, stop] windows must be adjacent without overlap or gaps")
  }

  class(x) <- "Survdt2"
  x
}

#' @rdname Survdt2
#' @export
survdt2to1 <- function(x){
  stopifnot(is.Survdt2(x))
  x <- x[order(factor(x[, "id", drop = TRUE],
                      levels = unique(x[, "id", drop = TRUE])),
               x[, "stop", drop = TRUE],
               na.last = FALSE), ]

  id_indx <- which(!duplicated(x[, "id", drop = TRUE], fromLast = TRUE))
  event_indx <- which(!is.na(x[, "event", drop = TRUE]) &
                        (x[, "event", drop = TRUE] == 1))

  unique_ids <- x[id_indx, "id", drop = TRUE]
  id_has_event <- unique_ids %in% x[event_indx, "id", drop = TRUE]
  id_indx[id_has_event] <- event_indx
  event_times <- ifelse(id_has_event, x[id_indx, "stop", drop = TRUE], NA)
  names(event_times) <- as.character(x[id_indx, "id", drop = TRUE])
  Survdt(event_times,
         x[id_indx, "ltrunc", drop = TRUE],
         x[id_indx, "rtrunc", drop = TRUE])
}

#' @rdname Survdt2
#' @export
`[.Survdt2` <- function(x, i, j, ...){
  if (missing(j) && !missing(i)){
    x <- unclass(x)[i, , drop = FALSE]
    class(x) <- "Survdt2"
    x
  }else{
    class(x) <- "matrix"
    NextMethod("[")
  }
}

#' @rdname Survdt2
#' @export
is.na.Survdt2 <- function(x){
  as.vector(rowSums(is.na(unclass(x))) > 0)
}

#' @rdname Survdt2
#' @export
is.Survdt2 <- function(x){
  inherits(x, "Survdt2")
}

#' @rdname Survdt2
#' @export
length.Survdt2 <- function(x){
  nrow(x)
}

#' @rdname Survdt2
#' @export
as.data.frame.Survdt2 <- function(x, ...){
  as.data.frame(unclass(x), ...)
}

#' @rdname Survdt2
#' @export
plot.Survdt2 <- function(x, ...){
  plot(survdt2to1(x), ...)
}

#' Compute the nonparametric MLE under double truncation
#'
#' @description
#' Computes the nonparametric maximum likelihood estimator (NPMLE) of the
#' pre-truncation event time and truncation time distributions using an EM
#' algorithm.
#'
#' @param y A `Survdt` object or expression. Observations with missing values
#'   are discarded.
#'   See [Survdt()].
#' @param data An optional data frame used to evaluate `y`.
#' @param se Whether to compute standard errors for each estimate
#'   (default `TRUE`).
#' @param cumhaz Whether to also compute the NPMLE of the event time cumulative
#'   hazard (default `FALSE`).
#' @param ainv Whether to return the NPMLE of the normalized inverse selection
#'   probabilities at the observed event times (default `TRUE`).
#' @param infl Whether to return the influence values for each estimate
#'   (default `FALSE`).
#' @param max_iter The maximum number of iterations for the EM algorithm.
#' @param error Tolerance for determining numerical convergence, defined as the
#'   relative change in the log likelihood being less than `error` or the absolute
#'   change being less than sqrt(`error`) at any iteration.
#' @param init Optional event time cdf point masses to initialize the EM
#'   algorithm.
#' @param convergence_warning Whether to issue a warning when numerical
#'   convergence is not achieved by `max_iter` iterations (default `TRUE`).
#' @param check_identifiability Whether to check and return an error if the
#'   NPMLE is not identifiable (default `TRUE`, see below).
#' @param x Any R object.
#' @param ... Additional arguments passed to or from other methods.
#'
#' @details
#' Given a doubly truncated sample \eqn{(T_1,L_1,R_1)}, \eqn{(T_2,L_2,R_2)},
#' \eqn{\ldots}, \eqn{(T_n,L_n,R_n)}, the NPMLE of the (pre-truncation) event
#' time cdf is a right-continuous step function
#' \deqn{\widehat{F}(t) = \sum_{i=1}^n 1(T_i\leq t) \widehat{f}_i}
#' with point masses at the observed event times.
#' Assuming (quasi-)independent truncation, these point masses maximize the
#' likelihood function
#' \deqn{L(f_1, \ldots, f_n) = \prod_{i=1}^n \frac{f_i}{\sum_j 1(L_i<T_j \leq R_i) f_j}.}
#' Efron and Petrosian (1999) provided a simple EM algorithm to compute the NPMLE.
#'
#' The point masses of the NPMLE can be expressed as
#' \eqn{\widehat{f}_i = 1/[\widehat{a}(T_i)n]},
#' where \eqn{a(t) = P(L<T \leq R|T=t)/P(L<T \leq R)} is the normalized
#' probability of observing time \eqn{t} within a truncation interval.
#' These inverse selection probabilities are used to correct for double truncation
#' bias in [coxph_indtrunc()].
#'
#' Once \eqn{\widehat{F}(t)} is obtained, other quantities such as the survival
#' function, cumulative hazard, and bivariate truncation time cdf point masses
#' are straightforward to compute.
#'
#' @section Identifiability:
#' Xiao and Hudgens (2019) provided a necessary and sufficient condition for the
#' likelihood function \eqn{L(f_1, \ldots, f_n)} to have a unique maximizer.
#' By default, this condition is checked by [npmle()] before estimation.
#'
#' @section Standard errors:
#' By default, [npmle()] computes variance estimates for the NPMLE
#' using the closed-form standard errors derived by Vazquez and Xie (2025).
#' This is generally more computationally efficient than the resampling methods
#' used by other packages.
#' If fit with `infl`=`TRUE`, the `npmle` object may be supplied as an input to
#' other [survdt] functions, which will then not need to re-compute the influence
#' values.
#'
#' @returns
#' [npmle()] returns an object of class `npmle`, i.e. a list with the following
#' components:
#' \item{n}{The complete case sample size.}
#' \item{n_miss}{The number of discarded observations (with missing values).}
#' \item{cdf}{A matrix containing the estimated event time cdf evaluated at
#'   ordered unique event times, with columns `time`, `cdf`, and `se`.}
#' \item{cumhaz}{A matrix containing the estimated event time cumulative hazard
#'   evaluated at ordered unique event times, with columns `time`, `cumhaz`, and
#'   `se`.
#'   Set to `NULL` if `cumhaz`=`FALSE`.}
#' \item{ainv}{A matrix containing the estimated normalized inverse selection
#'   probabilities for each event time (not sorted and including duplicates),
#'   with columns `time`, `ainv`, and `se`.
#'   Set to `NULL` if `ainv`=`FALSE`.}
#' \item{infl_cdf}{A matrix containing the estimated influence function for the
#'   cdf, with `[i,j]`th element corresponding to the `i`th ordered unique event
#'   time and `j`th unsorted individual.
#'   Set to `NULL` if `infl`=`FALSE`.}
#' \item{infl_cumhaz}{A matrix containing the estimated influence function for
#'   the cumulative hazard, with `[i,j]`th element corresponding to the `i`th
#'   ordered unique event time and `j`th unsorted individual.
#'   Set to `NULL` if `cumhaz=FALSE` or `infl`=`FALSE`.}
#' \item{infl_ainv}{A matrix containing the estimated influence function for the
#'   normalized inverse selection probabilities, with `[i,j]`th element
#'   corresponding to the `i`th event time (including ties) and `j`th individual,
#'   both unsorted.
#'   Set to `NULL` if `ainv=FALSE` or `infl`=`FALSE`.}
#' \item{masses}{A data frame containing the point masses for the event time
#'   (`f`) and truncation time (`g`) cdfs and the estimated selection
#'   probabilities for each event time (`pt`) and truncation interval (`plr`),
#'   with one row per observation.
#'   Not sorted.}
#' \item{converged}{Whether the EM algorithm converged before `max_iter` was
#'   reached.}
#' \item{n_iter}{The total number of EM algorithm iterations that ran.}
#' \item{overall_sampling_prob}{A named vector with the estimated overall
#'   probability of non-truncation in `prob` and its standard error in `se`.}
#'
#' [is.npmle()] returns `TRUE` if `x` inherits from the class `npmle` and
#' `FALSE` otherwise. The `print` method for `npmle` objects displays some sample
#' information and the first few steps of the estimated cdf.
#'
#' @export
#'
#' @seealso
#' * [plot.npmle()]: compute/plot estimates and pointwise confidence
#'   intervals for the event time survival function and cumulative hazard,
#'   normalized inverse selection probability weights, and selection probability
#'   as a function of time.
#' * [test_ignorability_indtrunc()]: test for ignorable sampling bias using the
#'   NPMLE.
#' * [test_samplediff_indtrunc()]: test for differences in the event time
#'   distributions across groups using the NPMLE.
#' * [test_quasiindep_ctau()]: test for violations of the quasi-independent
#'   truncation assumption.
#' * `vignette("nonparametric-analysis", package = "survdt")`: an example of
#'   various nonparametric analyses applied to the [aids] data, if vignettes
#'   were built when installing [survdt].
#'
#' @examples
#' npmle(Survdt(incu, ltrunc, rtrunc),
#'       data = aids)
#'
#'
#' @references
#' Efron, B., & Petrosian, V. (1999). Nonparametric methods for doubly truncated
#' data. *Journal of the American Statistical Association*, **94**(447), 824--834.
#'
#' de Uña-Álvarez, J., & Van Keilegom, I. (2021). Efron–Petrosian integrals for
#' doubly truncated data with covariates: An asymptotic analysis. *Bernoulli*,
#' **27**(1), 249--273.
#'
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
#'
#' Xiao, J., & Hudgens, M. G. (2019). On nonparametric maximum likelihood
#' estimation with double truncation. *Biometrika*, **106**(4), 989--996.
npmle <- function(y, data = NULL, se = TRUE, cumhaz = FALSE, ainv = TRUE,
                  infl = FALSE, max_iter = 1000, error = 1e-9, init = NULL,
                  convergence_warning = TRUE, check_identifiability = TRUE){
  # check input
  fn_call <- match.call()
  if (missing(y)) stop('\n  argument "y" is missing, with no default')
  if (!is.null(data)){
    if (is.matrix(data)) data <- as.data.frame(data)
    stopifnot(is.data.frame(data))
    y <- eval(fn_call[["y"]], envir = data, enclos = parent.frame())
  }
  if (is.Survdt2(y)) stop("Input y should be a Survdt object, not Survdt2.",
                          " See ?survdt::survdt2to1")
  stopifnot(is.Survdt(y),
            is.logical(se), is.vector(se), length(se) == 1, all(!is.na(se)),
            is.logical(cumhaz), is.vector(cumhaz), length(cumhaz) == 1,
            all(!is.na(cumhaz)),
            is.logical(ainv), is.vector(ainv),
            length(ainv) == 1, all(!is.na(ainv)),
            is.logical(infl), is.vector(infl),
            length(infl) == 1, all(!is.na(infl)),
            is.numeric(max_iter), is.vector(max_iter), length(max_iter) == 1,
            all(!is.na(max_iter)), max_iter >= 0,
            is.numeric(error), is.vector(error), length(error) == 1,
            all(!is.na(error)), error > 0,
            is.logical(convergence_warning), is.vector(convergence_warning),
            length(convergence_warning) == 1, all(!is.na(convergence_warning)),
            is.logical(check_identifiability), is.vector(check_identifiability),
            length(check_identifiability) == 1,
            all(!is.na(check_identifiability)))
  if (is.null(init)) init <- rep(1 / length(y), length(y))
  stopifnot(is.numeric(init), is.vector(init), length(init) == length(y),
            all(init > 0), all(init < 1))

  # missing values
  n_miss <- sum(is.na(y))
  init <- init[!is.na(y)]
  if (length(init) == 0) stop("No non-missing observations in y")
  if (any(is.na(init))) stop("Missing values in init")
  y <- y[!is.na(y), ]
  init <- init / sum(init)

  n <- length(y)
  sort_indx <- order(y[, "time", drop = TRUE])
  unsort_indx <- order(sort_indx)
  y <- y[sort_indx, ]
  init <- init[sort_indx] / sum(init)
  J <- apply(y[, c("ltrunc", "rtrunc")], MARGIN = 1,
             FUN = function(lr) (lr[1] < y[, "time", drop = TRUE]) &
               (y[, "time", drop = TRUE] <= lr[2]))

  # check connectivity
  if (check_identifiability == TRUE){
    gph <- igraph::graph(unlist(
      sapply(seq_len(n), FUN = function(i) c(rbind(i, which(J[, i]))))
    ))
    if (!igraph::is.connected(gph, "strong")) stop("NPMLE is not identifiable")
  }

  # EM algorithm
  f <- init
  converged <- FALSE
  inv_plr <- 1 / c(t(f) %*% J)
  loglike <- sum(log(f * inv_plr))
  for (i in seq_len(floor(max_iter))){
    f <- 1 / c(J %*% inv_plr)
    f <- f / sum(f)
    inv_plr <- 1 / c(t(f) %*% J)
    loglike_next <- sum(log(f * inv_plr))

    if ((abs(loglike_next - loglike) < sqrt(error)) ||
        (abs((loglike_next - loglike) / loglike) < error)){
      loglike <- loglike_next
      converged <- TRUE
      break
    }
    loglike <- loglike_next
  }
  g <- inv_plr / sum(inv_plr)
  pt <- c(J %*% g)

  if (!converged && convergence_warning)
    warning("EM algorithm reached max_iter before convergence")

  # output
  masses <- data.frame(time = y[, "time", drop = TRUE],
                       ltrunc = y[, "ltrunc", drop = TRUE],
                       rtrunc = y[, "rtrunc", drop = TRUE],
                       f = f, g = g, pt = pt, plr = c(1 / inv_plr))
  overall_sampling_prob <- c(prob = 1 / mean(1 / masses$plr), se = NA)
  jump_indx <- !duplicated(masses$time, fromLast = TRUE)
  npcdf <- cbind(time = masses$time, cdf = pmin(cumsum(masses$f), 1),
                 se = NA)[jump_indx, ]
  npcumhaz <- NULL
  npainv <- NULL
  infl_cdf <- NULL
  infl_cumhaz <- NULL
  infl_ainv <- NULL

  if ((se == TRUE) || (infl == TRUE)){
    stderrs <- npmle_infl(masses = masses, J = J, ainv = ainv, cumhaz = cumhaz,
                          msel_prob = TRUE, already_sorted = TRUE)
    npcdf[, "se"] <- sqrt(stderrs$var_F)
    if (infl == TRUE) infl_cdf <- stderrs$L_F[, unsort_indx]
    overall_sampling_prob["se"] <- sqrt(stderrs$infl_msel_prob$var)

    if (cumhaz == TRUE){
      npcumhaz <- cbind(time = stderrs$cumhaz[, 1], cumhaz = stderrs$cumhaz[, 2],
                        se = sqrt(stderrs$var_cumhaz))
      if (infl == TRUE) infl_cumhaz <- stderrs$L_cumhaz[, unsort_indx]
    }

    if (ainv == TRUE){
      npainv <- cbind(time = masses$time, ainv = 1 / stderrs$a,
                      se = sqrt(stderrs$var_ainv))[unsort_indx, ]
      if (infl == TRUE) {
        infl_ainv <- stderrs$L_ainv[unsort_indx, ]
        infl_ainv <- infl_ainv[, unsort_indx]
      }
    }
  }else{
    if (cumhaz == TRUE){
      lcont_survival <- 1 - c(0, npcdf[-nrow(npcdf), 2])
      dF <- diff(c(0, npcdf[, 2]))
      npcumhaz <- cbind(time = npcdf[, 1], cumhaz = cumsum(dF / lcont_survival),
                        se = NA)
    }
    if (ainv == TRUE){
      npainv <- cbind(time = masses$time, ainv = n * masses$f,
                      se = NA)[unsort_indx, ]
    }
  }

  res <- list(n = n, n_miss = n_miss, cdf = npcdf, cumhaz = npcumhaz,
              ainv = npainv, infl_cdf = infl_cdf, infl_cumhaz = infl_cumhaz,
              infl_ainv = infl_ainv, masses = masses[unsort_indx, ],
              converged = converged, n_iter = i,
              overall_sampling_prob = overall_sampling_prob)
  class(res) <- "npmle"
  res
}

#' @rdname npmle
#' @export
is.npmle <- function(x){
  inherits(x, "npmle")
}


#' Plot estimates from an NPMLE fit
#'
#' @description
#' Given a fitted NPMLE from [npmle()], plot various quantities related to the
#' event time or truncation time distributions along with pointwise confidence
#' intervals.
#' The estimates and standard errors are computed by [plot.npmle()] if they
#' were not included in the `npmle` fit.
#'
#' @param x An `npmle` object. See [npmle()].
#' @param ci Whether to plot pointwise confidence intervals (default `TRUE`).
#' @param level Confidence level for the pointwise confidence intervals (default
#'   0.95).
#' @param ctype Type of confidence interval.
#'   * `"plain"`: linear confidence intervals from a normal approximation.
#'   * `"log"`: back-transformed confidence intervals from the natural log scale.
#'     Guaranteed to be non-negative.
#'   * `"log-log"` (the default): for the survival function and selection
#'     probability, back-transformed confidence intervals from the (natural) log
#'     minus log scale, which are guaranteed to be between zero and one.
#'     For the normalized inverse probability weights and cumulative hazard,
#'     the same confidence intervals as `"log"` are used.
#' @param target Which NPMLE quantity to plot.
#'   * `"survival"` (the default): the event time survival function.
#'   * `"cumhaz"`: the event time cumulative hazard function.
#'   * `"ainv"`: the normalized inverse probability weights at each observed
#'     event time. See [npmle()].
#'   * `"sel_prob"`: the selection (non-truncation) probability as a function of
#'     time.
#' @param ... Additional arguments passed to other methods.
#'
#' @returns
#' After displaying the plot, a list with two elements is returned
#' invisibly.
#' The first element `ests` is a data frame with the columns `time`,
#' `est`, `lower`, and `upper`, containing the point estimates and their computed
#' confidence intervals.
#' The second element `plot` is the plotted object returned by [ggplot2::ggplot()].
#'
#' @export
#'
#' @seealso [npmle()]
#'
#' @examples
#' fit <- npmle(Survdt(incu, ltrunc, rtrunc),
#'              data = aids)
#' plot(fit)
#'
#' plot(fit, target = "cumhaz")
plot.npmle <- function(x, ci = TRUE, level = 0.95,
                       ctype = c("log-log", "log", "plain"),
                       target = c("survival", "cumhaz", "ainv", "sel_prob"),
                       ...){
  ctype <- match.arg(ctype)
  target <- match.arg(target)
  stopifnot(is.npmle(x),
            is.logical(ci), is.vector(ci), all(!is.na(ci)),
            is.numeric(level), is.vector(level), all(!is.na(level)))
  stopifnot(length(ci) == 1, length(level) == 1)
  stopifnot(level > 0, level < 1)
  q <- stats::qnorm(1-(1-level)/2)

  ci_transform <- ifelse(ctype == "plain", "linear", ctype)
  if (target == "cumhaz"){
    ylab <- "Cumulative hazard"
    linedir <- "hv"
    if ((ci == FALSE) && is.null(x[["cumhaz"]])){
      x <- npmle(with(x[["masses"]],
                      Survdt(time = time, ltrunc = ltrunc, rtrunc = rtrunc)),
                 se = ci, cumhaz = TRUE, ainv = FALSE, infl = FALSE,
                 max_iter = 0, init = x[["masses"]]$f,
                 convergence_warning = FALSE, check_identifiability = FALSE)
    }
    if ((ci == TRUE) && (is.null(x[["cumhaz"]]) ||
                         any(is.na(x[["cumhaz"]][, "se"])))){
      stderrs <- npmle_infl(x[["masses"]][order(x[["masses"]]$time), ,
                                          drop = FALSE],
                            cumhaz = TRUE, L_F = x$infl_cdf,
                            already_sorted = TRUE)
      x$cumhaz <- cbind(time = stderrs$cumhaz[, 1], cumhaz = stderrs$cumhaz[, 2],
                        se = sqrt(stderrs$var_cumhaz))
    }

    ests <- data.frame(time = x[["cumhaz"]][, "time", drop = TRUE],
                       est = x[["cumhaz"]][, "cumhaz", drop = TRUE],
                       se = x[["cumhaz"]][, "se", drop = TRUE],
                       lower = rep(NA, nrow(x[["cumhaz"]])),
                       upper = rep(NA, nrow(x[["cumhaz"]])))
    ci_transform <- ifelse(ctype == "log-log", "log", ci_transform)
  }else if (target == "survival"){
    ylab <- "Survival function"
    linedir <- "hv"
    if (ci && any(is.na(x[["cdf"]][, "se", drop = TRUE]))){
      stderrs <- npmle_infl(x[["masses"]][order(x[["masses"]]$time), ,
                                          drop = FALSE],
                            L_F = x[["infl_cdf"]], already_sorted = TRUE)
      x$cdf[, "se"] <- sqrt(stderrs$var_F)
    }

    ests <- data.frame(time = x[["cdf"]][, "time", drop = TRUE],
                       est = 1 - x[["cdf"]][, "cdf", drop = TRUE],
                       se = x[["cdf"]][, "se", drop = TRUE],
                       lower = rep(NA, nrow(x[["cdf"]])),
                       upper = rep(NA, nrow(x[["cdf"]])))
  }else if (target == "ainv"){
    ylab <- "Normalized IPW"
    if ((ci == FALSE) && is.null(x[["ainv"]])){
      x <- npmle(with(x[["masses"]],
                      Survdt(time = time, ltrunc = ltrunc, rtrunc = rtrunc)),
                 se = ci, cumhaz = FALSE, ainv = TRUE, infl = FALSE,
                 max_iter = 0, init = x[["masses"]]$f,
                 convergence_warning = FALSE, check_identifiability = FALSE)
    }
    if ((ci == TRUE) &&
        (is.null(x[["ainv"]]) || any(is.na(x[["ainv"]][, "se"])))){
      sort_indx <- order(x[["masses"]]$time)
      unsort_indx <- order(sort_indx)
      stderrs <- npmle_infl(x[["masses"]][sort_indx, , drop = FALSE], ainv = TRUE,
                            L_F = x[["infl_cdf"]], already_sorted = TRUE)

      x$ainv <- cbind(time = x[["masses"]]$time, ainv = 1 / stderrs$a[unsort_indx],
                      se = sqrt(stderrs$var_ainv[unsort_indx]))
    }

    ests <- data.frame(time = x[["ainv"]][, "time", drop = TRUE],
                       est = x[["ainv"]][, "ainv", drop = TRUE],
                       se = x[["ainv"]][, "se", drop = TRUE],
                       lower = rep(NA, nrow(x[["ainv"]])),
                       upper = rep(NA, nrow(x[["ainv"]])))
    ci_transform <- ifelse(ctype == "log-log", "log", ci_transform)
  }else if (target == "sel_prob"){
    ylab <- "Selection probability"
    linedir <- "vh"
    if (ci == TRUE){
      stderrs <- npmle_infl(x[["masses"]][order(x[["masses"]]$time), ,
                                          drop = FALSE],
                            infl_pt = TRUE, L_F = x[["infl_cdf"]],
                            already_sorted = TRUE)
      ests <- data.frame(time = stderrs$infl_pt$time,
                         est = c(0, stderrs$infl_pt$pt_btwn),
                         se = c(NA, sqrt(stderrs$infl_pt$var_pt_btwn)),
                         lower = rep(NA, length(stderrs$infl_pt$time)),
                         upper = rep(NA, length(stderrs$infl_pt$time)))

    }else{
      lr_times <- unique(sort(c(x[["masses"]]$ltrunc, x[["masses"]]$rtrunc)))
      nlr <- length(lr_times)
      ests <- data.frame(time = lr_times,
                         est = sapply(lr_times, FUN = function(ti) {
                           sum(x[["masses"]]$g[(x[["masses"]]$ltrunc < ti) &
                                                 (ti <= x[["masses"]]$rtrunc)])
                         }),
                         se = rep(NA, nlr),
                         lower = rep(NA, nlr),
                         upper = rep(NA, nlr))
    }
  }


  if (ci == TRUE){
    if (ci_transform == "linear"){
      ests$lower <- ests$est - q * ests$se
      ests$upper <- ests$est + q * ests$se
      if (ests$est[1] <= 0){
        ests$upper[1] <- ests$upper[2]
        ests$lower[1] <- 0
      }
    }else if (ci_transform == "log"){
      ests$lower <- ests$est * exp(-q * ests$se / abs(ests$est))
      ests$upper <- ests$est * exp(q * ests$se / abs(ests$est))
      if (ests$est[nrow(ests)] <= 0){
        ests$upper[nrow(ests)] <- ests$upper[nrow(ests) - 1]
        ests$lower[nrow(ests)] <- 0
      }
      if (ests$est[1] <= 0){
        ests$upper[1] <- ests$upper[2]
        ests$lower[1] <- 0
      }
    }else if (ci_transform == "log-log"){
      ests$upper <- ests$est^exp(-q * ests$se /
                                   abs(ests$est * log(abs(ests$est))))
      ests$lower <- ests$est^exp(q * ests$se /
                                   abs(ests$est * log(abs(ests$est))))
      if (ests$est[nrow(ests)] <= 0){
        ests$upper[nrow(ests)] <- ests$upper[nrow(ests) - 1]
        ests$lower[nrow(ests)] <- 0
      }
      if (ests$est[1] <= 0){
        ests$upper[1] <- ests$upper[2]
        ests$lower[1] <- 0
      }
    }
  }

  pltfn_lower <- function(x) ggplot2::geom_line(x, linetype = "dashed")
  pltfn_upper <- pltfn_lower
  pltfn_est <- ggplot2::geom_point
  if (target != "ainv"){
    if (target %in% c("survival", "cumhaz")){
      ests <- rbind(ests[1, , drop = FALSE], ests)
      ests$est[1] <- ifelse(target == "survival", 1, 0)
      ests$se[1] <- NA
      ests$lower[1] <- ifelse(target == "survival",
                              ests$lower[1],
                              min(0, ests$lower[1]))
      ests$upper[1] <- ifelse(target == "survival",
                              max(1, ests$upper[1]),
                              ests$upper[1])
    }

    pltfn_est <- function(x) ggplot2::geom_step(x, direction = linedir)
    pltfn_lower <- function(x) ggplot2::geom_step(x, linetype = "dashed",
                                                  direction = linedir)
    pltfn_upper <- pltfn_lower
  }

  plt <- ggplot2::ggplot(ests)
  if (ci == TRUE){
    plt <- with(ests,
                plt +
                  pltfn_lower(ggplot2::aes(x = time, y = lower)) +
                  pltfn_upper(ggplot2::aes(x = time, y = upper)))
  }

  plt <- with(ests,
              plt +
                pltfn_est(ggplot2::aes(x = time, y = est)) +
                ggplot2::labs(x = "Time", y = ylab) +
                ggplot2::coord_cartesian(xlim = c(min(x[["masses"]]$time) / 1.01,
                                                  max(x[["masses"]]$time) * 1.01),
                                         ylim = c(0, max(est) * 1.01),
                                         expand = FALSE) +
                ggplot2::theme_classic())

  print(plt)
  invisible(list(ests = ests, plot = plt))
}



#' @rdname npmle
#' @export
print.npmle <- function(x, ...){
  cat(paste0("\nNPMLE fit based on ", x[["n"]], " complete samples (",
             x[["n_miss"]], " dropped due to missingness)\n",
             "with ", nrow(x[["cdf"]]), " unique event times:\n\n"))
  print(x[["cdf"]][1:min(nrow(x[["cdf"]]), 5), , drop = FALSE])
  cat(ifelse(nrow(x[["cdf"]]) <= 5, " \n\n", "...\n\n"))
  invisible(x)
}

#' Test for non-ignorable sampling bias
#'
#' @description
#' Implements two possible nonparametric tests for non-ignorable sampling bias
#' based on the NPMLE:
#' 1. check for non-constant selection probabilities across the event time range;
#' 2. check for a difference between the estimated event time cumulative
#'    distribution functions adjusted and not adjusted for double truncation.
#'
#' Typically used to determine if a doubly truncated sample must be analyzed
#' with methods that account for truncation bias.
#'
#' @param y A `Survdt` object or expression, or an `npmle` object. Observations
#'   with missing values are discarded.
#'   See [Survdt()] and [npmle()].
#' @param data An optional data frame used to evaluate `y`.
#' @param test_type The test statistic to use: `"sel_prob"` (default) or `"cdf"`.
#'   See below.
#' @param level The confidence level for the test (default 0.95).
#' @param B The number of simulations from the estimated null distribution used
#'   to compute the p-value (default 2000).
#' @param restr_times An optional numerical vector of length two giving a time
#'   window to restrict the test statistic.
#'   Values of `NA` are replaced with the minimum and maximum observed event
#'   time respectively, i.e. no restriction.
#'   The default setting is no restriction.
#' @param wt_fn An optional function that takes a numerical vector of times as
#'   input and returns a vector of weights to apply in the test statistic.
#'   If unspecified, all times have equal weight of 1.
#' @param x Any R object.
#' @param ... Additional arguments passed to other methods.
#'
#' @details
#' The sampling bias from double truncation is called ignorable when the
#' selection (non-truncation) probability is constant over all possible event
#' times.
#' Since all event times have an equal probability of being included in the sample,
#' the sampling mechanism does not alter their distribution, so standard analysis
#' methods can be applied to the data.
#'
#' Define the normalized selection probability at any time \eqn{t} by
#' \eqn{a(t) = P(L<T \leq R|T=t)/P(L<T \leq R)}.
#' Its estimate \eqn{\widehat{a}(t)} is a left-continuous step function with
#' jumps at the observed truncation times.
#' By default, [test_ignorability_indtrunc()] tests for non-ignorable sampling
#' bias through the test statistic
#' \deqn{\sup_{\min_i T_i \leq t \leq \max_i T_i} |\widehat{a}(t) - 1|}
#' with the p-value computed by simulating from the estimated asymptotic null
#' distribution.
#' Another option is to compare two estimated event time cdfs: the NPMLE
#' \eqn{\widehat{F}(t)} which adjusts for double truncation and the empirical
#' cdf \eqn{\widehat{F}^*(t)} which does not adjust for double truncation.
#' The test statistic would then be
#' \deqn{\sup_t |\widehat{F}(t) - \widehat{F}^*(t)|.}
#'
#' @section Restricted time window and weighted tests:
#' When the NPMLE has high variance at extreme event times, the test may suffer
#' from reduced power.
#' One possible remedy is to restrict the time range of the test to some interval
#' \eqn{[\tau_1, \tau_2]} contained in \eqn{[\min_i T_i, \max_i T_i]}.
#' Another option is to down-weight event times with large variance through a
#' weight function \eqn{w(t)}.
#' The stabilized test statistic will have the form
#' \deqn{\sup_{\tau_1 \leq t \leq \tau_2} |w(t)(\widehat{h}(t) - h(t))|}
#' where \eqn{\widehat{h}(t)} is the NPMLE without assuming the null hypothesis
#' holds and \eqn{h(t)} is the reference value under the null.
#' See `vignette("nonparametric-analysis", package = "survdt")` for an illustration
#' with the [aids] data assuming vignettes were built when installing [survdt].
#'
#' @section No effective truncation:
#' A doubly truncated sample has no effective truncation when
#' \eqn{\max_i L_i < \min_i T_i} and \eqn{\max_i T_i \leq \min_i R_i}.
#' The estimated selection probability is equal to one across all event times
#' in this case, and its standard errors may not be reliable.
#' Typically one would conclude that the truncation mechanism is ignorable in
#' these types of samples, so standard methods can be applied for data analysis.
#'
#' @returns
#' [test_ignorability_indtrunc()] returns an object of class
#' `ignorability_indtrunc_test`, i.e. a list with the following components:
#' \item{n}{The complete case sample size.}
#' \item{n_unique}{The number of unique event times among complete cases.}
#' \item{n_miss}{The number of discarded observations (with missing values).}
#' \item{teststat}{The value of the observed test statistic.}
#' \item{pval}{The p-value for the test.}
#' \item{critval}{The critical value for the test at the user-specified level.}
#' \item{testresult}{Whether the test resulted in rejecting the null hypothesis
#'   of ignorable sampling bias.}
#' \item{sims}{A vector of the `B` simulations from the estimated null
#'   distribution of the test statistic.}
#' \item{test_type}{The matched `test_type` value.}
#' \item{level}{The user-supplied `level`.}
#' \item{B}{The number of simulations specified by the user.}
#' \item{timerange}{A vector with the minimum and maximum times used in the
#'   test.}
#' \item{wt_name}{A string giving the name of the user-supplied `wt_fn`, if any.}
#' \item{eventrange}{A vector with the unrestricted range of observed event times.}
#' \item{ests}{A data frame of estimates to be used by
#'   [plot.ignorability_indtrunc_test()].}
#'
#' The `print` method for `ignorability_indtrunc_test` objects displays a summary
#' of the test results, while the `plot` method returns a visualization comparing
#' the NPMLE and its reference value under the null hypothesis using a uniform
#' confidence band, generated through [ggplot2::ggplot()].
#'
#' @export
#'
#' @seealso
#' * [npmle()]: the nonparametric maximum likelihood estimator which forms the
#'   basis for these tests.
#' * `vignette("nonparametric-analysis", package = "survdt")`: an illustration
#'   of ignorability tests applied to the [aids] data, if vignettes were built
#'   when installing [survdt].
#'
#' @examples
#' igtest <- test_ignorability_indtrunc(Survdt(incu, ltrunc, rtrunc),
#'                                      data = aids)
#' igtest
#' plot(igtest)
#'
#' @references
#' de Uña‐Álvarez, J. (2023). Testing for an ignorable sampling bias under random
#' double truncation. *Statistics in Medicine*, **42**(20), 3732--3744.
#'
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
test_ignorability_indtrunc <- function(y, data = NULL,
                                       test_type = c("sel_prob", "cdf"),
                                       level = 0.95, B = 2000,
                                       restr_times = NULL, wt_fn = NULL){
  # check input
  fn_call <- match.call()
  if (missing(y)) stop('\n  argument "y" is missing, with no default')
  if (!is.null(data)){
    if (is.matrix(data)) data <- as.data.frame(data)
    stopifnot(is.data.frame(data))
    y <- eval(fn_call[["y"]], envir = data, enclos = parent.frame())
  }
  if (is.Survdt2(y)) stop("Input y should be a Survdt object, not Survdt2.",
                          " See ?survdt::survdt2to1")
  stopifnot(is.npmle(y) || is.Survdt(y))
  test_type <- match.arg(test_type)
  if (is.null(restr_times)) restr_times <- c(NA_real_, NA_real_)
  if (is.null(wt_fn)) {
    wt_name <- "none"
    wt_fn <- function(x) rep(1, length(x))
  }else{
    wt_name <- format(substitute(wt_fn))
  }
  stopifnot(is.character(test_type), is.vector(test_type), all(!is.na(test_type)),
            is.numeric(level), is.vector(level), all(!is.na(level)),
            is.numeric(B), is.vector(B), all(!is.na(B)),
            is.numeric(restr_times), is.vector(restr_times), is.function(wt_fn))
  stopifnot(length(test_type) == 1, length(level) == 1, length(B) == 1,
            length(restr_times) == 2)
  stopifnot(0 < level, level < 1, B > 1)
  B <- floor(B)

  # npmle
  if (is.Survdt(y)){
    y <- npmle(y, se = TRUE, cumhaz = FALSE, ainv = FALSE, infl = TRUE)
    if (!y[["converged"]]) stop("Fitted npmle did not converge")

  }
  sort_indx <- order(y[["masses"]]$time)
  unsort_indx <- order(sort_indx)
  n <- y[["n"]]
  n_unique <- nrow(y[["cdf"]])
  n_miss <- y[["n_miss"]]
  eventrange <- range(y[["masses"]]$time)
  time_lims <- eventrange
  if (!is.na(restr_times[1]) && (restr_times[1] >= time_lims[2]))
    stop("restr_times[1] must be less than the maximum observed event time")
  if (!is.na(restr_times[2]) && (restr_times[2] <= time_lims[1]))
    stop("restr_times[2] must be greater than the minimum observed event time")
  if (all(!is.na(restr_times)) && (restr_times[1] >= restr_times[2]))
    stop("Must have restr_times[1]<restr_times[2]")
  if (all((y[["masses"]]$ltrunc < time_lims[1]) &
          (time_lims[2] <= y[["masses"]]$rtrunc)))
    warning(paste0("All truncation intervals cover the entire event time range,",
                   " indicating no effective truncation. ",
                   "Test results may be unreliable."))
  if (n_unique == 1)
    stop("Ignorability is not testable with one unique event time.")

  test_window <- ifelse(is.na(restr_times), time_lims,
                        c(max(restr_times[1], time_lims[1]),
                          min(restr_times[2], time_lims[2])))
  # test statistic and null distribution
  if (test_type == "cdf"){
    if (is.null(y[["infl_cdf"]]))
      y$infl_cdf <- npmle_infl(y[["masses"]][sort_indx, , drop = FALSE],
                               already_sorted = TRUE)$L_F[, unsort_indx]
    wts <- wt_fn(y[["cdf"]][, "time", drop = TRUE])
    if (!is.numeric(wts) || !is.vector(wts))
      stop("wt_fn must return a numeric vector")
    if (length(wts) != nrow(y[["cdf"]]))
      stop("wt_fn output length does not match input")
    bad_times <- y[["cdf"]][is.na(wts) | is.infinite(wts), "time", drop = TRUE]
    if (length(bad_times) > 5) bad_times <- c(bad_times[1:5], "...")
    if (length(bad_times) > 0)
      stop(paste0("wt_fn produced infinite or missing values at time ",
                  paste0(bad_times, collapse = ", ")))

    sorted_times <- y[["masses"]]$time[sort_indx]
    unique_indx <- !duplicated(sorted_times, fromLast = TRUE)
    e_cdf <- cumsum(rep(1, n))[unique_indx] / n
    endpoints <- findInterval(test_window, sorted_times[unique_indx])
    time_indx <- endpoints[1]:endpoints[2]
    window_times <- (sorted_times[unique_indx])[time_indx]
    test_wts <- wts[time_indx]
    teststat <- max(abs(test_wts * (y[["cdf"]][time_indx, "cdf", drop = TRUE] -
                                      e_cdf[time_indx])))
    L_nulldist <- y[["infl_cdf"]][time_indx, , drop = FALSE] -
      sapply(y[["masses"]]$time, FUN = function(xi) {
        (xi <= window_times) - e_cdf[time_indx]
      })
    ests <- data.frame(time = y[["cdf"]][, "time", drop = TRUE],
                       npmle_cdf = y[["cdf"]][, "cdf", drop = TRUE],
                       empirical_cdf = e_cdf,
                       out_of_window = rep(TRUE, length(e_cdf)),
                       wt = wts)
    ests$out_of_window[time_indx] <- FALSE
  }else if (test_type == "sel_prob"){
    if (is.null(y[["infl_cdf"]])){
      infl_vals <- npmle_infl(y[["masses"]][sort_indx, , drop = FALSE],
                              infl_at = TRUE, already_sorted = TRUE)
    }else{
      infl_vals <- npmle_infl(y[["masses"]][sort_indx, , drop = FALSE],
                              infl_at = TRUE, L_F = y[["infl_cdf"]][, sort_indx],
                              already_sorted = TRUE)
    }
    wts <- wt_fn(infl_vals$infl_at$time)
    if (!is.numeric(wts) || !is.vector(wts))
      stop("wt_fn must return a numeric vector")
    if (length(wts) != length(infl_vals$infl_at$time))
      stop("wt_fn output length does not match input")
    bad_times <- infl_vals$infl_at$time[is.na(wts) | is.infinite(wts)]
    if (length(bad_times) > 5) bad_times <- c(bad_times[1:5], "...")
    if (length(bad_times) > 0)
      stop(paste0("wt_fn produced infinite or missing values at time ",
                  paste0(bad_times, collapse = ", ")))

    endpoints <- pmin(findInterval(test_window, infl_vals$infl_at$time,
                                   left.open = TRUE) + 1,
                      length(infl_vals$infl_at$time))
    lrindx <- seq(endpoints[1], endpoints[2])
    test_wts <- wts[lrindx]
    teststat <- max(abs(test_wts * (infl_vals$infl_at$at_btwn[lrindx - 1] - 1)))
    L_nulldist <- infl_vals$infl_at$L_at_btwn[lrindx - 1, , drop = FALSE]
    ests <- data.frame(time = infl_vals$infl_at$time,
                       at = c(0, infl_vals$infl_at$at_btwn),
                       overall_sampling_prob =
                         rep(y[["overall_sampling_prob"]]["prob"],
                             length(infl_vals$infl_at$time)),
                       out_of_window = rep(TRUE, length(infl_vals$infl_at$time)),
                       wt = wts)
    ests$out_of_window[lrindx] <- FALSE
  }

  sims <- replicate(B, {
    wt <- stats::rnorm(n, 1, 1)
    max(abs(test_wts * c(L_nulldist %*% wt / n)))
  })
  critval <- stats::quantile(sims, level)

  res <- list(n = n, n_unique = n_unique, n_miss = n_miss, teststat = teststat,
              pval = mean(sims >= teststat), critval = critval,
              testresult = teststat > critval, sims = sims,
              test_type = test_type, level = level, B = B,
              timerange = test_window, wt_name = wt_name, eventrange = eventrange,
              ests = ests)
  class(res) <- "ignorability_indtrunc_test"
  res
}

#' @rdname test_ignorability_indtrunc
#' @export
plot.ignorability_indtrunc_test <- function(x, level, ...){
  if (missing(level)) level <- x[["level"]]
  stopifnot(is.numeric(level), is.vector(level), all(!is.na(level)))
  stopifnot(length(level) == 1, all(level > 0), all(level < 1))
  q <- stats::quantile(x[["sims"]], level)

  ests <- NULL
  if (x[["test_type"]] == "cdf"){
    ests <- data.frame(time = x[["ests"]]$time,
                       npmle = x[["ests"]]$npmle_cdf,
                       reference = x[["ests"]]$empirical_cdf,
                       out_of_window = x[["ests"]]$out_of_window,
                       wt = x[["ests"]]$wt)
    ests$lower <- pmax(ests$reference - (q / abs(ests$wt)), 0)
    ests$upper <- pmin(ests$reference + (q / abs(ests$wt)), 1)
  }else if (x[["test_type"]] == "sel_prob"){
    msel_prob <- x[["ests"]]$overall_sampling_prob[1]
    ests <- data.frame(time = x[["ests"]]$time,
                       npmle = msel_prob * x[["ests"]]$at,
                       reference = rep(msel_prob, nrow(x[["ests"]])),
                       out_of_window = x[["ests"]]$out_of_window,
                       wt = x[["ests"]]$wt)
    ests$lower <- pmax(ests$reference * (1 - (q / abs(ests$wt))), 0)
    ests$upper <- pmin(ests$reference * (1 + (q / abs(ests$wt))), 1)
  }
  ests$lower[ests$out_of_window] <- 0
  ests$upper[ests$out_of_window] <- 1
  stairdir <- ifelse(x[["test_type"]] == "cdf", "hv", "vh")

  with(ests,
       ggplot2::ggplot(ests) +
         ggplot2::geom_step(ggplot2::aes(x = time, y = lower),
                            linetype = "dashed", direction = stairdir) +
         ggplot2::geom_step(ggplot2::aes(x = time, y = upper),
                            linetype = "dashed", direction = stairdir) +
         ggplot2::geom_step(ggplot2::
                              aes(x = time, y = reference,
                                  color = "assuming ignorable sampling bias"),
                            direction = stairdir) +
         ggplot2::geom_step(ggplot2::aes(x = time, y = npmle, color = "NPMLE"),
                            direction = stairdir) +
         ggplot2::scale_color_manual(breaks =
                                       c("NPMLE",
                                         "assuming ignorable sampling bias"),
                                     values = c("maroon", "black"),
                                     guide =
                                       ggplot2::guide_legend(override.aes =
                                                               list(lwd = 2))) +
         ggplot2::labs(x = "Time",
                       y = ifelse(x[["test_type"]] == "cdf",
                                  "Cumulative distribution function",
                                  "Selection probability"),
                       color = "") +
         ggplot2::coord_cartesian(ylim = c(0, 1.01), xlim = x[["timerange"]],
                                  expand = FALSE) +
         ggplot2::theme_classic() +
         ggplot2::theme(legend.position = "bottom"))
}

#' @rdname test_ignorability_indtrunc
#' @export
print.ignorability_indtrunc_test <- function(x, ...){
  pval <- format(x[["pval"]], digits = 2, nsmall = 1)
  pval <- ifelse(pval == "0.0",
                 paste0("<", 1 / x[["B"]]),
                 paste0("=", pval))
  cat(paste0("\n",
             "Ignorable sampling bias test under quasi-independent truncation.",
                "\n",
                "\n",
             "  NPMLE fit based on ", x[["n"]], " complete samples (",
                x[["n_miss"]], " dropped due to missingness)",
                "\n",
             "    with ", x[["n_unique"]], " unique event times.",
                "\n",
             "  Min event time: ", x[["eventrange"]][1],
                "\n",
             "  Max event time: ", x[["eventrange"]][2],
                "\n",
                "\n",
             "  Supremum test of type ", '"', x[["test_type"]], '".',
                "\n",
             "  Test statistic: ",
                  format(x[["teststat"]], digits = 2, nsmall = 1),
                  "\n",
             "    Time range:   [", x[["timerange"]][1], ", ",
                  x[["timerange"]][2], "]",
                  "\n",
             "    Time weights: ", x[["wt_name"]],
                  "\n",
             "  Critical value: ",
                  format(x[["critval"]], digits = 2, nsmall = 1),
                " at level ", x[["level"]],
                "\n",
             "  Significant evidence ", ifelse(x[["testresult"]], "", "not "),
                "found against ignorability (p", pval, ").",
                "\n",
             "  Results based on ", x[["B"]], " simulations from the estimated ",
                "null distribution.",
                "\n",
                "\n"))
  invisible(x)
}

#' Test for dependence between truncation times and the event time
#'
#' @description
#' Computes the conditional Kendall's tau rank correlation between the event
#' time and each of the two truncation times, accounting for double truncation.
#' Then tests whether both correlations are equal to zero.
#'
#' Typically used to determine if nonparametric analysis can be done using the
#' NPMLE, which requires quasi-independence (see [npmle()]).
#'
#' @param y A `Survdt` object or expression. Observations with missing values are
#'   discarded. See [Survdt()].
#' @param data An optional data frame used to evaluate `y`.
#' @param singular_thresh The numerical threshold for determining whether the
#'   variance matrix for the correlations is singular (default `1e-9`).
#'   This can occur when the truncation times are a constant distance from each
#'   other.
#' @inheritParams test_ignorability_indtrunc
#'
#' @details
#' The conditional Kendall's tau rank correlation between the left truncation
#' time and event time is estimated by the average of
#' \deqn{\text{sign}\{(T_i - T_j)(L_i - L_j)\}}
#' across all comparable pairs of subjects \eqn{(i,j)}.
#' A pair is called comparable if \eqn{L_j<T_i \leq R_j} and \eqn{L_i<T_j \leq R_i},
#' which ensures that the order of the pair's times is not influenced by truncation
#' under quasi-independence.
#' Similarly, the conditional Kendall's tau rank correlation between the right
#' truncation time and event time is estimated by the average of
#' \deqn{\text{sign}\{(T_i - T_j)(R_i - R_j)\}}
#' across the same comparable pairs of subjects \eqn{(i,j)}.
#'
#' Under quasi-independent truncation, both conditional Kendall's tau rank
#' correlations should converge to zero in large samples.
#'
#' @returns
#' [test_quasiindep_ctau()] returns an object of class `quasiindep_ctau_test`,
#' i.e. a list with the following components:
#' \item{tau}{A numerical vector of length two containing the estimated
#'   conditional Kendall's tau rank correlation (1) between the event time and
#'   left truncation time, and (2) between the event time and right truncation
#'   time.}
#' \item{teststat}{The chi-squared test statistic for testing the null hypothesis
#'   that both correlations are zero.}
#' \item{df}{The degrees of freedom for `teststat`.}
#' \item{pval}{The p-value for testing that both correlations are zero.}
#' \item{var_matrix}{The estimated variance matrix for the two conditional
#'   Kendall's tau correlation estimates.}
#' \item{singular}{Whether `var_matrix` was determined to be singular.}
#' \item{prob_comp}{The fraction of subject pairs that are comparable.}
#'
#' The `print` method for `quasiindep_ctau_test` objects displays a summary of
#' the test results.
#'
#' @export
#'
#' @seealso
#' * [test_quasiindep_covariates()]: a test for dependent truncation when
#'   covariates are involved, e.g. when aiming to analyze data with
#'   [coxph_indtrunc()].
#'
#' @examples
#' test_quasiindep_ctau(Survdt(incu, ltrunc, rtrunc), aids)
#'
#' @references
#' Martin, E. C., & Betensky, R. A. (2005). Testing quasi-independence of failure
#' and truncation times via conditional Kendall's tau. *Journal of the American
#' Statistical Association*, **100**(470), 484--492.
test_quasiindep_ctau <- function(y, data = NULL, singular_thresh = 1e-9){
  # check input
  fn_call <- match.call()
  if (missing(y)) stop('\n  argument "y" is missing, with no default')
  if (!is.null(data)){
    if (is.matrix(data)) data <- as.data.frame(data)
    stopifnot(is.data.frame(data))
    y <- eval(fn_call[["y"]], envir = data, enclos = parent.frame())
  }
  if (is.Survdt2(y)) stop("Input y should be a Survdt object, not Survdt2.",
                          " See ?survdt::survdt2to1")
  stopifnot(is.Survdt(y),
            is.numeric(singular_thresh), is.vector(singular_thresh))
  stopifnot(length(singular_thresh) == 1, !any(is.na(singular_thresh)))
  stopifnot(singular_thresh > 0)

  # missing data
  n_init <- length(y)
  complete_cases <- !is.na(y)
  if (sum(complete_cases) == 0)
    stop("All subjects have missing data")
  y <- y[complete_cases, ]
  n <- length(y)
  n_miss <- n_init - n

  # components for tau and variance matrix
  comparable <- sapply(seq_len(n), FUN = function(i) {
    (y[, "ltrunc", drop = TRUE] < y[i, "time", drop = TRUE]) &
      (y[i, "time", drop = TRUE] <= y[, "rtrunc", drop = TRUE]) &
      (y[i, "ltrunc", drop = TRUE] < y[, "time", drop = TRUE]) &
      (y[, "time", drop = TRUE] <= y[i, "rtrunc", drop = TRUE])
  })
  ests <- rowSums(sapply(seq_len(n), FUN = function(i){
    if (any(comparable[-i, i])){
      comp_indx <- which(comparable[, i])
      comp_indx <- comp_indx[comp_indx != i]

      n_pairs <- 0
      new_pairs <- c(0, 0)
      if (any(comp_indx > i)){
        pair_indx <- comp_indx[comp_indx > i]
        n_pairs <- length(pair_indx)
        new_pairs <- c(sum(sign((y[i, "time", drop = TRUE] -
                                   y[pair_indx, "time", drop = TRUE]) *
                                  (y[i, "ltrunc", drop = TRUE] -
                                     y[pair_indx, "ltrunc", drop = TRUE]))),
                       sum(sign((y[i, "time", drop = TRUE] -
                                   y[pair_indx, "time", drop = TRUE]) *
                                  (y[i, "rtrunc", drop = TRUE] -
                                     y[pair_indx, "rtrunc", drop = TRUE]))))
      }

      new_triples <- c(0, 0, 0)
      if (length(comp_indx) > 1){
        ltsum <- sum(sign((y[i, "time", drop = TRUE] -
                             y[comp_indx, "time", drop = TRUE]) *
                            (y[i, "ltrunc", drop = TRUE] -
                               y[comp_indx, "ltrunc", drop = TRUE])))
        rtsum <- sum(sign((y[i, "time", drop = TRUE] -
                             y[comp_indx, "time", drop = TRUE]) *
                            (y[i, "rtrunc", drop = TRUE] -
                               y[comp_indx, "rtrunc", drop = TRUE])))

        new_triples <- c(ltsum^2, ltsum * rtsum, rtsum^2)
      }
      c(new_pairs, new_triples, n_pairs)
    }else{
      c(0, 0, 0, 0, 0, 0)
    }
  }))

  # results
  tau <- ests[1:2] / ests[6]
  prob_comp <- ests[6] / (n * (n - 1) / 2)
  n3 <- n * (n-1)^2
  var_matrix <- matrix(c((ests[3] / n3) - (prob_comp * tau[1])^2,
                         (ests[4] / n3) - (tau[1] * tau[2] * prob_comp^2),
                         (ests[4] / n3) - (tau[1] * tau[2] * prob_comp^2),
                         (ests[5] / n3) - (prob_comp * tau[2])^2), nrow = 2)
  var_matrix <- var_matrix * 4 / (n * prob_comp^2)
  singular <- (var_matrix[1, 1] * var_matrix[2, 2] - var_matrix[1, 2]^2) <
    singular_thresh
  if (singular){
    teststat <- ifelse(var_matrix[1, 1] > 0, tau[1]^2 / (var_matrix[1, 1]), NA)
    df <- 1
    pval <- stats::pchisq(teststat, df, lower.tail = FALSE)
  }else{
    teststat <- t(tau) %*% solve(var_matrix) %*% tau
    df <- 2
    pval <- stats::pchisq(teststat, df, lower.tail = FALSE)
  }

  res <- list(tau = tau, teststat = teststat,
              df = ifelse(is.na(teststat), NA, df), pval = pval,
              var_matrix = var_matrix, singular = singular,
              prob_comp = prob_comp)
  class(res) <- "quasiindep_ctau_test"
  return(res)
}

#' @rdname test_quasiindep_ctau
#' @export
print.quasiindep_ctau_test <- function(x, ...){
  pval <- format(x[["pval"]], digits = 2, nsmall = 1)
  cat(paste0("\n",
             "Quasi-independent truncation test by conditional rank correlations.",
             "\n",
             "\n",
             "  Conditional Kendall's tau between event time and...",
                  "\n",
             "    left truncation time: ",
                     format(x[["tau"]][1], digits = 2, nsmall = 1),
                  "\n",
             "    right truncation time: ",
                     format(x[["tau"]][2], digits = 2, nsmall = 1),
             "\n",
             "  Test statistic: chisq=",
                   format(x[["teststat"]], digits = 2, nsmall = 1),
                ", df=", x[["df"]],
             "\n",
             "  p-value: ", pval,
             "\n",
             "  Proportion of subject pairs comparable was ",
                format(x[["prob_comp"]], digits = 2, nsmall = 1), ".",
             "\n",
             "\n"))
  invisible(x)
}


#' Test for survival differences across groups
#'
#' @description
#' Tests for any difference in event time distribution by comparing
#' the stratified and unstratified estimates obtained using the NPMLE.
#'
#' @param formula A formula with a `Survdt` response and stratification variables
#'   on the right hand side.
#'   Note that subtraction will not remove stratification variables here.
#'   Subjects with missing values are discarded.
#'   See [Survdt()].
#' @param data An optional data frame used to evaluate `formula`.
#' @param test_type The test statistic to use: `"survival"` (default) or
#'   `"cumhaz"`.
#'   See below.
#' @param max_iter The maximum number of iterations to use within each call to
#' [npmle()]. If missing then the default setting of [npmle()] is used.
#' @param simple_labels Whether to drop the factor names from the stratum labels
#' (Default `FALSE`).
#' @inheritParams test_ignorability_indtrunc
#'
#' @details
#' Let \eqn{\widehat{S}(t|s)} be the NPMLE for the survival function in stratum
#' \eqn{s}.
#' By default, [test_samplediff_indtrunc()] tests for event time distribution
#' differences through the test statistic
#' \deqn{\max_s\sup_{\min_i T_i \leq t \leq \max_i T_i} |\widehat{S}(t|s) - \widehat{S}(t)|}
#' where \eqn{\widehat{S}(t)} is the unstratified NPMLE.
#' Another option is to compare the stratified cumulative hazards
#' \eqn{\widehat{\Lambda}(t|s)} with the unstratified NPMLE \eqn{\widehat{\Lambda}(t)}
#' through the test statistic
#' \deqn{\max_s\sup_{\min_i T_i \leq t \leq \max_i T_i}
#'       |\widehat{\Lambda}(t|s) - \widehat{\Lambda}(t)|.}
#' The p-value in all cases is computed by simulating from the estimated asymptotic
#' null distribution.
#'
#' @inheritSection test_ignorability_indtrunc Restricted time window and weighted tests
#'
#' @returns
#' [test_samplediff_indtrunc()] returns an object of class
#' `samplediff_indtrunc_test`, i.e. a list with the following components:
#' \item{n}{The complete case sample size.}
#' \item{n_unique}{The number of unique event times among complete cases.}
#' \item{n_miss}{The number of discarded observations (with missing values).}
#' \item{teststat}{The value of the observed test statistic.}
#' \item{pval}{The p-value for the test.}
#' \item{critval}{The critical value for the test at the user-specified level.}
#' \item{testresult}{Whether the test results in rejecting the null
#'   hypothesis of no differences across strata.}
#' \item{sims}{A vector of the `B` simulations from the estimated null
#'   distribution of the test statistic.}
#' \item{test_type}{The matched `test_type` value.}
#' \item{level}{The user-supplied `level`.}
#' \item{B}{The number of simulations specified by the user.}
#' \item{timerange}{A vector with the minimum and maximum times used in the test.}
#' \item{wt_name}{A string giving the name of the user-supplied `wt_fn`, if any.}
#' \item{eventrange}{A vector with the unrestricted range of observed event times.}
#' \item{strata_ests}{A data frame of estimates to be used by
#'   [plot.samplediff_indtrunc_test()].}
#' \item{strata_summary}{Some summary statistics for each stratum.}
#'
#' The `print` method for `samplediff_indtrunc_test` objects displays a summary
#' of the test results, while the `plot` method returns a visualization comparing
#' the stratified and unstratified NPMLE using a uniform confidence band,
#' generated through [ggplot2::ggplot()].
#'
#' @export
#'
#' @seealso
#' * [npmle()]: the nonparametric maximum likelihood estimator which forms the
#'   basis for these tests.
#' * `vignette("nonparametric-analysis", package = "survdt")`: an illustration
#'   of group tests applied to the [aids] data, if vignettes were built when
#'   installing [survdt].
#'
#' @examples
#' difftest <- test_samplediff_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
#'                                      data = aids,
#'                                      restr_times = c(NA, 70))
#' difftest
#' plot(difftest)
#'
#' @references
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
test_samplediff_indtrunc <- function(formula, data = NULL,
                                     test_type = c("survival", "cumhaz"),
                                     level = 0.95, B = 2000,
                                     restr_times = NULL, wt_fn = NULL,
                                     max_iter = NULL, simple_labels = FALSE){
  # check input
  if (missing(formula))
    stop('\n  argument "formula" is missing, with no default')
  stopifnot(inherits(formula, "formula"))
  if (length(formula) < 3) stop("The formula is missing a LHS or a RHS")
  if (!is.null(data) && is.matrix(data)) data <- as.data.frame(data)
  stopifnot(is.null(data) || is.data.frame(data))

  mf <- stats::model.frame(stats::update(formula, ~.-1), data = data,
                           na.action = stats::na.pass)
  if (attr(attr(mf, "terms"), "response") == 0)
    stop("formula is missing a response")
  y <- mf[[attr(attr(mf, "terms"), "response")]]
  if (length(mf) < 2) stop("formula is missing stratafiction variables on RHS")
  strata_vars <- mf[-attr(attr(mf, "terms"), "response")]
  complete_cases <- rowSums(is.na(mf)) == 0
  factor_names <- c(apply(strata_vars[1, , drop = FALSE],
                          MARGIN = 1, FUN = names))
  stopifnot(is.logical(simple_labels), is.vector(simple_labels),
            length(simple_labels) == 1, all(!is.na(simple_labels)))
  if (simple_labels == TRUE) factor_names <- rep("", length(factor_names))
  s <- apply(strata_vars, MARGIN = 1, FUN = function(x) {
    paste0(factor_names, x, sep = "", collapse = ", ")
  })

  if (is.Survdt2(y))
    stop(paste0("Response should be a Survdt object with one observation",
                " per subject (not Survdt2). See ?survdt::survdt2to1"))
  test_type <- match.arg(test_type)
  if (is.null(restr_times)) restr_times <- c(NA_real_, NA_real_)
  if (is.null(wt_fn)) {
    wt_name <- "none"
    wt_fn <- function(x) rep(1, length(x))
  }else{
    wt_name <- format(substitute(wt_fn))
  }
  if (is.null(max_iter)) max_iter <- formals(npmle)[["max_iter"]]
  stopifnot(is.Survdt(y),
            is.character(test_type), is.vector(test_type), all(!is.na(test_type)),
            is.numeric(level), is.vector(level), !any(is.na(level)),
            is.numeric(B), is.vector(B), !any(is.na(B)),
            is.numeric(restr_times), is.vector(restr_times), is.function(wt_fn),
            is.numeric(max_iter), is.vector(max_iter), !any(is.na(max_iter)))
  stopifnot(length(y) == length(s), length(test_type) == 1, length(level) == 1,
            all(level > 0), all(level < 1), length(B) == 1, all(B > 1),
            length(restr_times) == 2, length(max_iter) == 1, all(max_iter > 1))
  B <- floor(B)
  max_iter <- floor(max_iter)

  # missing data
  n_init <- length(y)
  if (sum(complete_cases) == 0) stop("No observations without missing data")
  if (length(unique(s)) != length(unique(s[complete_cases])))
    warning("Dropping strata with no complete cases")
  y <- y[complete_cases, ]
  n <- length(y)
  s <- s[complete_cases]
  s <- factor(s, levels = unique(s[order(c(strata_vars[complete_cases, 1,
                                                       drop = TRUE])[1:n])]))
  sort_indx <- order(y[, "time", drop = TRUE])
  y <- y[sort_indx, ]
  s <- s[sort_indx]
  strata_vals <- unique(s)
  n_strata <- length(strata_vals)
  n_miss <- n_init - n
  if (n_strata < 2) stop("Only one stratum in complete cases")

  eventrange <- range(y[, "time", drop = TRUE])
  time_lims <- eventrange
  if (!is.na(restr_times[1]) && (restr_times[1] >= time_lims[2]))
    stop("restr_times[1] must be less than the maximum observed event time")
  if (!is.na(restr_times[2]) && (restr_times[2] <= time_lims[1]))
    stop("restr_times[2] must be greater than the minimum observed event time")
  if (all(!is.na(restr_times)) && (restr_times[1] >= restr_times[2]))
    stop("Must have restr_times[1]<restr_times[2]")
  n_unique_strata <- stats::aggregate(y[, "time", drop = TRUE] ~ s,
                                      FUN = function(x) length(unique(x)))
  small_strata <- n_unique_strata[n_unique_strata[, 2, drop = TRUE] < 2, 1,
                                  drop = TRUE]
  if (length(n_unique_strata) > 5)
    n_unique_strata <- c(as.character(n_unique_strata[1:5]), "...")
  if (length(small_strata) > 0)
    stop(paste0("Less than two unique event times found in stratum: ",
                paste0(small_strata, collapse = "; ")))
  strata_summary <- data.frame(strata = n_unique_strata[, 1, drop = TRUE],
                               strata_size =
                                 sapply(n_unique_strata[, 1, drop = TRUE],
                                        FUN = function(x) sum(s == x)),
                               unique_event_times =
                                 n_unique_strata[, 2, drop = TRUE])

  test_window <- ifelse(is.na(restr_times), time_lims,
                        c(max(restr_times[1], time_lims[1]),
                          min(restr_times[2], time_lims[2])))

  # unstratified npmle
  pooled_npmle <- npmle(y, se = TRUE, cumhaz = test_type == "cumhaz",
                        infl = TRUE, max_iter = max_iter)
  if (pooled_npmle[["converged"]] == FALSE)
    stop("Unstratified npmle() did not converge")
  n_unique <- nrow(pooled_npmle[["cdf"]])
  wts <- wt_fn(pooled_npmle[["cdf"]][, "time", drop = TRUE])
  if (!is.numeric(wts) || !is.vector(wts))
    stop("wt_fn must return a numeric vector")
  bad_times <- pooled_npmle[["cdf"]][is.na(wts) | is.infinite(wts), "time",
                                     drop = TRUE]
  if (length(bad_times) > 5) bad_times <- c(bad_times[1:5], "...")
  if (length(bad_times) > 0)
    stop(paste0("wt_fn produced infinite or missing values at time ",
                paste0(bad_times, collapse = ", ")))

  endpoints_test <- findInterval(test_window,
                                 pooled_npmle[["cdf"]][, "time", drop = TRUE])
  test_indx <- endpoints_test[1]:endpoints_test[2]
  n_time_total <- nrow(pooled_npmle[["cdf"]])
  n_time_test <- length(test_indx)
  if (test_type == "survival"){
    est_unstrat <- 1 - pooled_npmle[["cdf"]][, "cdf", drop = TRUE]
    L_unstrat <- pooled_npmle[["infl_cdf"]][test_indx, , drop = FALSE]
  }else{
    est_unstrat <- pooled_npmle[["cumhaz"]][, "cumhaz", drop = TRUE]
    L_unstrat <- pooled_npmle[["infl_cumhaz"]][test_indx, , drop = FALSE]
  }

  # strata npmle and null distribution
  L_nulldist <- matrix(nrow = n_time_test * n_strata, ncol = n)
  test_wts <- rep(wts[test_indx], n_strata)
  new_out_of_window <- pooled_npmle[["cdf"]][, "cdf", drop = TRUE] <= 0
  strata_ests <- data.frame(strata = rep(strata_vals, each = n_time_total),
                            time = rep(pooled_npmle[["cdf"]][, "time",
                                                             drop = TRUE],
                                       n_strata),
                            unstrat_npmle = rep(est_unstrat, n_strata),
                            npmle = rep(NA, n_time_total * n_strata),
                            out_of_window = rep(!(seq_len(n_time_total) %in%
                                                    test_indx),
                                                n_strata),
                            wt = rep(wts, n_strata))
  for (i in seq_len(n_strata)){
    strata_indx <- s == strata_vals[i]
    strata_npmle <- npmle(y[strata_indx, ], se = TRUE,
                          cumhaz = test_type == "cumhaz", infl = TRUE,
                          max_iter = max_iter)
    if (strata_npmle[["converged"]] == FALSE)
      stop(paste0("Stratified npmle() did not converge in stratum ",
                  strata_vals[i]))

    rows_L <- n_time_test * (i-1) + seq_len(n_time_test)
    rows_ests <- n_time_total * (i-1) + seq_len(n_time_total)
    timeindx_s <- findInterval(strata_ests$time[rows_ests],
                               strata_npmle[["cdf"]][, "time", drop = TRUE]) + 1

    nsratio <- n / sum(strata_indx)
    if (test_type == "survival"){
      est_strat <- 1 -
        c(0, strata_npmle[["cdf"]][, "cdf", drop = TRUE], 1)[timeindx_s]
      first_Lrow <- strata_npmle[["infl_cdf"]][1, , drop = TRUE]
      last_Lrow <- strata_npmle[["infl_cdf"]][nrow(strata_npmle[["infl_cdf"]]), ,
                                              drop = TRUE]
      L_strat <- rbind(nsratio * first_Lrow,
                       nsratio * strata_npmle[["infl_cdf"]],
                       nsratio * last_Lrow)[timeindx_s[test_indx], , drop = FALSE]
    }else{
      est_strat <- c(0,
                     strata_npmle[["cumhaz"]][, "cumhaz", drop = TRUE],
                     strata_npmle[["cumhaz"]][nrow(strata_npmle[["cumhaz"]]),
                                              "cumhaz"])[timeindx_s]
      first_Lrow <- strata_npmle[["infl_cumhaz"]][1, , drop = TRUE]
      last_Lrow <-
        strata_npmle[["infl_cumhaz"]][nrow(strata_npmle[["infl_cumhaz"]]), ,
                                      drop = TRUE]
      L_strat <- rbind(nsratio * first_Lrow,
                       nsratio * strata_npmle[["infl_cumhaz"]],
                       nsratio * last_Lrow)[timeindx_s[test_indx], ,
                                            drop = FALSE]
    }
    strata_ests$npmle[rows_ests] <- est_strat
    L_nulldist[rows_L, ] <- -L_unstrat
    L_nulldist[rows_L, strata_indx] <- L_nulldist[rows_L, strata_indx,
                                                  drop = FALSE] +
      L_strat
  }
  if (any(new_out_of_window)){
    strata_ests$out_of_window <- strata_ests$out_of_window |
      rep(new_out_of_window, n_strata)
    L_nulldist[rep(new_out_of_window[test_indx], n_strata), ] <- 0

    testindx_range <- range(test_indx[!new_out_of_window[test_indx]])
    test_window[1] <- max(test_window[1],
                          pooled_npmle[["cdf"]][testindx_range[1], "time",
                                                drop = TRUE])
    if (testindx_range[2] < n_time_total)
      test_window[2] <- min(test_window[2],
                            pooled_npmle[["cdf"]][testindx_range[2] + 1, "time",
                                                  drop = TRUE])
  }

  # results
  teststat <-
    max(abs(strata_ests$wt[!strata_ests$out_of_window] *
              (strata_ests$npmle[!strata_ests$out_of_window] -
                 strata_ests$unstrat_npmle[!strata_ests$out_of_window])))
  sims <- replicate(B, {
    wt <- stats::rnorm(n, 1, 1)
    max(abs(test_wts * c(L_nulldist %*% wt / n)))
  })
  critval <- stats::quantile(sims, level)

  res <- list(n = n, n_unique = n_unique, n_miss = n_miss, teststat = teststat,
              pval = mean(sims >= teststat), critval = critval,
              testresult = teststat > critval, sims = sims, test_type = test_type,
              level = level, B = B, timerange = test_window, wt_name = wt_name,
              eventrange = eventrange, strata_ests = strata_ests,
              strata_summary = strata_summary)
  class(res) <- "samplediff_indtrunc_test"
  res
}

#' @rdname test_samplediff_indtrunc
#' @export
plot.samplediff_indtrunc_test <- function(x, level, ...){
  if (missing(level)) level <- x[["level"]]
  stopifnot(is.numeric(level), is.vector(level), all(!is.na(level)))
  stopifnot(length(level) == 1, all(level > 0), all(level < 1))
  q <- stats::quantile(x[["sims"]], level)
  ests <- x[["strata_ests"]]

  upper_bounds <- ests$unstrat_npmle[!ests$out_of_window] +
    (q / abs(ests$wt[!ests$out_of_window]))
  upper_cutoff <- ifelse(x[["test_type"]] == "survival", 1,
                         max(max(ests$npmle[!ests$out_of_window]),
                             max(ests$unstrat_npmle[!ests$out_of_window]),
                             max(upper_bounds[!is.infinite(upper_bounds)])) *
                           1.01)
  ests$lower <- ifelse(ests$out_of_window, 0,
                       pmax(0, ests$unstrat_npmle - (q / abs(ests$wt))))
  ests$upper <- ifelse(ests$out_of_window, upper_cutoff,
                       pmin(upper_cutoff, ests$unstrat_npmle + (q / abs(ests$wt))))

  ylab <-  ifelse(x[["test_type"]] == "survival",
                  "Survival function", "Cumulative hazard")
  with(ests,
       ggplot2::ggplot(ests) +
         ggplot2::geom_step(ggplot2::aes(x = time, y = lower),
                            linetype = "dashed", direction = "hv") +
         ggplot2::geom_step(ggplot2::aes(x = time, y = upper),
                            linetype = "dashed", direction = "hv") +
         ggplot2::geom_step(ggplot2::aes(x = time, y = npmle, group = strata,
                                         color = "stratified NPMLE"),
                            direction = "hv") +
         ggplot2::geom_step(ggplot2::aes(x = time, y = unstrat_npmle,
                                         color = "unstratified NPMLE"),
                            direction = "hv") +
         ggplot2::scale_color_manual(breaks = c("stratified NPMLE",
                                                "unstratified NPMLE"),
                                     values = c("maroon", "black"),
                                     guide =
                                       ggplot2::guide_legend(override.aes =
                                                               list(lwd = 2))) +
         ggplot2::labs(x = "Time", y = ylab, color = "") +
         ggplot2::coord_cartesian(ylim = c(0, upper_cutoff * 1.01),
                                  xlim = x[["timerange"]],
                                  expand = FALSE) +
         ggplot2::theme_classic() +
         ggplot2::theme(legend.position = "bottom"))
}

#' @rdname test_samplediff_indtrunc
#' @export
print.samplediff_indtrunc_test <- function(x, ...){
  pval <- format(x[["pval"]], digits = 2, nsmall = 1)
  pval <- ifelse(pval == "0.0",
                 paste0("<", 1 / x[["B"]]),
                 paste0("=", pval))
  cat(paste0("\n",
             "Test for group differences under quasi-independent truncation.",
             "\n",
             "\n",
             "  Unstratified NPMLE fit based on ", x[["n"]], " complete samples (",
             x[["n_miss"]], " dropped due to missingness)",
             "\n",
             "    with ", x[["n_unique"]], " unique event times.",
             "\n",
             "  Min event time: ", x[["eventrange"]][1],
             "\n",
             "  Max event time: ", x[["eventrange"]][2],
             "\n",
             "\n",
             "  Supremum test of type ", '"', x[["test_type"]], '".',
             "\n",
             "  Test statistic: ",
                  format(x[["teststat"]], digits = 2, nsmall = 1),
             "\n",
             "    Time range:   [", x[["timerange"]][1], ", ",
             x[["timerange"]][2], "]",
             "\n",
             "    Time weights: ", x[["wt_name"]],
             "\n",
             "  Critical value: ",
                  format(x[["critval"]], digits = 2, nsmall = 1),
                " at level ", x[["level"]],
             "\n",
             "  Significant evidence ", ifelse(x[["testresult"]], "", "not "),
             "found against equality across strata (p", pval, ").",
             "\n",
             "  Results based on ", x[["B"]], " simulations from the estimated ",
             "null distribution.",
             "\n",
             "\n",
             "Summary of the ", nrow(x[["strata_summary"]]), " groups compared:",
             "\n"))

  sort_indx <- order(x[["strata_summary"]][, "strata_size", drop = TRUE],
                     decreasing = TRUE)
  strata_summary <- x[["strata_summary"]][sort_indx,
                                          c("strata",
                                            "strata_size",
                                            "unique_event_times"),
                                          drop = FALSE]
  colnames(strata_summary) <- c("stratum", "size", "unique event times")
  if (nrow(x[["strata_summary"]]) > 10){
    print(strata_summary[1:10, , drop = FALSE],
          row.names = FALSE)
    cat("  ...")
    cat("\n\n")
  }else{
    print(strata_summary, row.names = FALSE)
    cat("\n")
  }
  invisible(x)
}


#' Test for dependence between truncation times and the event time or covariates
#'
#' @description
#' Implements a two-stage test for dependence between the truncation times and
#' the event time and covariates that first tests within covariate strata and then
#' across strata.
#'
#' Typically used to check the underlying assumptions for inverse probability
#' weighted regression with NPMLE weights, e.g. as used in [coxph_indtrunc()].
#'
#' @inheritParams test_samplediff_indtrunc
#'
#' @details
#' Implements a nonparametric test for dependent truncation to check if the
#' selection probability depends on either the event times or covariates.
#' The test has two stages:
#' 1. Test for dependence between event and truncation times stratified by
#'    covariate groups using conditional Kendall's tau
#'    (see [test_quasiindep_ctau()]).
#'    The aggregate p-value for this stage is computed from a chi-squared
#'    distribution.
#' 2. Test whether the selection probability depends on covariates by comparing
#'    the stratified and unstratified selection probabilities estimated using the
#'    NPMLE.
#'    Let \eqn{\widehat{\pi}(t|s)} be the estimate in stratum \eqn{s}, and
#'    \eqn{\widehat{\pi}(t)} be the unstratified estimate.
#'    Both are left-continuous step functions with jumps at observed truncation
#'    times.
#'    By default, [test_quasiindep_covariates()] uses the test statistic
#'    \deqn{\max_s\sup_{\min_i T_i\leq t \leq \max_i T_i}
#'          |\widehat{\pi}(t|s)-\widehat{\pi}(t)|}
#'    to compare selection probabilities across strata.
#'    The p-value for this stage is computed by simulating from the estimated
#'    asymptotic null distribution.
#'
#' @inheritSection test_ignorability_indtrunc Restricted time window and weighted tests
#'
#' @inheritSection test_ignorability_indtrunc No effective truncation
#'
#' @returns An object of class `quasiindep_strat_test`, i.e. a list with the following
#' components:
#' \item{n}{The complete case sample size.}
#' \item{n_unique}{The number of unique event times among complete cases.}
#' \item{n_miss}{The number of discarded observations (with missing values).}
#' \item{teststat}{The value of the observed test statistic for the across-strata
#'   test (stage 2).}
#' \item{pval}{The p-value for the across-strata test.}
#' \item{critval}{The critical value for the across-strata test at the
#'   user-specified level.}
#' \item{testresult}{Whether the across-strata test results in rejecting the null
#'   hypothesis of quasi-independence.}
#' \item{sims}{A vector of the `B` simulations from the estimated null distribution
#'   of the across-strata test statistic.}
#' \item{level}{The user-supplied `level`.}
#' \item{B}{The number of simulations specified by the user.}
#' \item{timerange}{A vector with the minimum and maximum times used in the
#'   across-strata test.}
#' \item{wt_name}{A string giving the name of the user-suppled `wt_fn`, if any.}
#' \item{eventrange}{A vector with the unrestricted range of observed event times.}
#' \item{strata_ests}{A data frame of estimates to be used by
#'   [plot.quasiindep_strat_test()].}
#' \item{strata_ctau}{A data frame of the within-strata test results (stage 1).
#'   Includes stratum-specific tests by conditional Kendall's tau as well as the
#'   aggregate p-value for testing within-strata quasi-independence in all strata
#'   simultaneously.}
#'
#' The `print` method for `quasiindep_strat_test` objects displays a summary of the
#' test results, while the `plot` method returns a visualization comparing the
#' stratified and unstratified NPMLE from stage 2 using a uniform confidence band,
#' generated through [ggplot2::ggplot()].
#'
#' @export
#'
#' @seealso
#' * [test_quasiindep_ctau()]: the conditional Kendall's tau test for quasi-dependence
#'   without covariates, which is the basis for stage 1.
#' * [npmle()]: the nonparametric maximum likelihood estimator which is the basis
#'   for stage 2.
#' * `vignette("cox-regression-quasiindep", package = "survdt")`: an illustration
#'   of this quasi-independence test applied to the [aids] data, if vignettes were
#'   built when installing [survdt].
#'
#' @examples
#' indtest <- test_quasiindep_covariates(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
#'                                       data = aids)
#' indtest
#' plot(indtest)
#'
#' @references
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted estimators
#' for doubly truncated Cox regression with closed-form standard errors.
#' *Lifetime Data Analysis*, **31**(2), 364–-393.
test_quasiindep_covariates <- function(formula, data = NULL, level = 0.95, B = 2000,
                                       restr_times = NULL, wt_fn = NULL,
                                       max_iter = NULL, simple_labels = FALSE){
  # check input
  if (missing(formula)) stop('\n  argument "formula" is missing, with no default')
  stopifnot(inherits(formula, "formula"))
  if (length(formula) < 3) stop("The formula is missing a LHS or a RHS")
  if (!is.null(data) && is.matrix(data)) data <- as.data.frame(data)
  stopifnot(is.null(data) || is.data.frame(data))

  mf <- stats::model.frame(stats::update(formula, ~.-1), data = data,
                           na.action = stats::na.pass)
  if (attr(attr(mf, "terms"), "response") == 0)
    stop("formula is missing a response")
  y <- mf[[attr(attr(mf, "terms"), "response")]]
  if (length(mf) < 2) stop("formula is missing stratafiction variables on RHS")
  strata_vars <- mf[-attr(attr(mf, "terms"), "response")]
  complete_cases <- rowSums(is.na(mf)) == 0
  factor_names <- c(apply(strata_vars[1, , drop = FALSE], MARGIN = 1, FUN = names))
  stopifnot(is.logical(simple_labels), is.vector(simple_labels),
            length(simple_labels) == 1, all(!is.na(simple_labels)))
  if (simple_labels == TRUE) factor_names <- rep("", length(factor_names))
  s <- apply(strata_vars, MARGIN = 1, FUN = function(x) {
    paste0(factor_names, x, sep = "", collapse = ", ")
  })

  if (is.Survdt2(y)) stop("Response should be a Survdt object with one observation",
                          " per subject (not Survdt2). See ?survdt::survdt2to1")
  if (is.null(restr_times)) restr_times <- c(NA_real_, NA_real_)
  if (is.null(wt_fn)) {
    wt_name <- "none"
    wt_fn <- function(x) rep(1, length(x))
  }else{
    wt_name <- format(substitute(wt_fn))
  }
  if (is.null(max_iter)) max_iter <- formals(npmle)[["max_iter"]]
  stopifnot(is.Survdt(y),
            is.numeric(level), is.vector(level), !any(is.na(level)),
            is.numeric(B), is.vector(B), !any(is.na(B)),
            is.numeric(restr_times), is.vector(restr_times), is.function(wt_fn),
            is.numeric(max_iter), is.vector(max_iter), !any(is.na(max_iter)))
  stopifnot(length(y) == length(s), length(level) == 1, all(level > 0),
            all(level < 1), length(B) == 1, all(B > 1), length(restr_times) == 2,
            length(max_iter) == 1, all(max_iter > 1))
  B <- floor(B)
  max_iter <- floor(max_iter)

  # missing data
  n_init <- length(y)
  if (sum(complete_cases) == 0) stop("No observations without missing data")
  if (length(unique(s)) != length(unique(s[complete_cases])))
    warning("Dropping strata with no complete cases")
  y <- y[complete_cases, ]
  n <- length(y)
  s <- s[complete_cases]
  s <- factor(s, levels = unique(s[order(c(strata_vars[complete_cases, 1,
                                                       drop = TRUE])[1:n])]))
  sort_indx <- order(y[, "time", drop = TRUE])
  y <- y[sort_indx, ]
  s <- s[sort_indx]
  strata_vals <- unique(s)
  n_strata <- length(strata_vals)
  n_miss <- n_init - n
  if (n_strata < 2) stop("Only one stratum in complete cases")

  eventrange <- range(y[, "time", drop = TRUE])
  time_lims <- eventrange
  if (!is.na(restr_times[1]) && (restr_times[1] >= time_lims[2]))
    stop("restr_times[1] must be less than the maximum observed event time")
  if (!is.na(restr_times[2]) && (restr_times[2] <= time_lims[1]))
    stop("restr_times[2] must be greater than the minimum observed event time")
  if (all(!is.na(restr_times)) && (restr_times[1] >= restr_times[2]))
    stop("Must have restr_times[1]<restr_times[2]")
  if (all((y[, "ltrunc", drop = TRUE] < time_lims[1]) &
          (time_lims[2] <= y[, "rtrunc", drop = TRUE])))
    warning(paste0("All truncation intervals cover the entire event time range, ",
                   "indicating no effective truncation. ",
                   "Test results may be unreliable."))
  if (length(unique(y[, "time", drop = TRUE])) == 1)
    stop("Ignorability is not testable with one unique event time.")
  test_window <- ifelse(is.na(restr_times), time_lims,
                        c(max(restr_times[1], time_lims[1]),
                          min(restr_times[2], time_lims[2])))

  # unstratified npmle
  pooled_npmle <- npmle(y, se = FALSE, cumhaz = FALSE, ainv = FALSE, infl = FALSE,
                        max_iter = max_iter)
  if (pooled_npmle[["converged"]] == FALSE) stop("Unstratified npmle() did not converge")
  n_unique <- nrow(pooled_npmle[["cdf"]])

  pooled_infl <- npmle_infl(pooled_npmle[["masses"]], infl_pt = TRUE,
                            already_sorted = TRUE)

  endpoints <- findInterval(time_lims, pooled_infl$infl_pt$time, left.open = TRUE)
  if (endpoints[2] >= length(pooled_infl$infl_pt$time))
    stop("Maximum event time lies past maximum right truncation time")
  lrindx <- seq(endpoints[1], endpoints[2])
  endpoints_test <- findInterval(test_window, pooled_infl$infl_pt$time,
                                 left.open = TRUE)
  test_indx <- (endpoints_test[1] <= lrindx) & (lrindx <= endpoints_test[2])
  n_lr_total <- length(lrindx)
  n_lr_test <- sum(test_indx)
  L_pi_unstrat <- pooled_infl$infl_pt$L_pt_btwn[lrindx[test_indx], , drop = FALSE]

  wts <- wt_fn(pooled_infl$infl_pt$time[lrindx + 1])
  if (!is.numeric(wts) || !is.vector(wts))
    stop("wt_fn must return a numeric vector")
  if (length(wts) != length(lrindx))
    stop("wt_fn output length does not match input")
  bad_times <- pooled_infl$infl_pt$time[lrindx[is.na(wts) | is.infinite(wts)] + 1]
  if (length(bad_times) > 5) bad_times <- c(bad_times[1:5], "...")
  if (length(bad_times) > 0)
    stop(paste0("wt_fn produced infinite or missing values at time ",
                paste0(bad_times, collapse = ", ")))

  # strata ctau, npmle, and null distribution
  L_nulldist <- matrix(nrow = n_lr_test * n_strata, ncol = n)
  test_wts <- rep(wts[test_indx], n_strata)
  new_out_of_window <- pooled_infl$infl_pt$pt_btwn[lrindx] <= 0
  strata_ests <- data.frame(strata = rep(strata_vals, each = n_lr_total),
                            time = rep(pooled_infl$infl_pt$time[lrindx], n_strata),
                            unstrat_pt = rep(pooled_infl$infl_pt$pt_btwn[lrindx],
                                             n_strata),
                            pt = rep(NA, n_lr_total * n_strata),
                            out_of_window = rep(!test_indx, n_strata),
                            wt = rep(wts, n_strata))
  strata_ctau <- data.frame(strata = strata_vals, strata_size = rep(NA, n_strata),
                            tau_left = rep(NA, n_strata),
                            tau_right = rep(NA, n_strata),
                            teststat = rep(NA, n_strata), df = rep(NA, n_strata),
                            pval = rep(NA, n_strata), var_left = rep(NA, n_strata),
                            var_right = rep(NA, n_strata), cov = rep(NA, n_strata))
  for (i in seq_len(n_strata)){
    strata_indx <- s == strata_vals[i]
    strata_npmle <- npmle(y[strata_indx, ], se = FALSE, cumhaz = FALSE,
                          ainv = FALSE, infl = FALSE, max_iter = max_iter)
    if (strata_npmle[["converged"]] == FALSE)
      stop(paste0("Stratified npmle() did not converge in stratum ", strata_vals[i]))

    strata_infl <- npmle_infl(strata_npmle[["masses"]], infl_pt = TRUE,
                              already_sorted = TRUE)

    strata_test <- test_quasiindep_ctau(y[strata_indx, ])
    strata_ctau$tau_left[i] <- strata_test[["tau"]][1]
    strata_ctau$tau_right[i] <- strata_test[["tau"]][2]
    strata_ctau$var_left[i] <- strata_test[["var_matrix"]][1, 1]
    strata_ctau$var_right[i] <- strata_test[["var_matrix"]][2, 2]
    strata_ctau$cov[i] <- strata_test[["var_matrix"]][1, 2]
    strata_ctau$teststat[i] <- strata_test[["teststat"]]
    strata_ctau$df[i] <- strata_test[["df"]]
    strata_ctau$pval[i] <- strata_test[["pval"]]
    strata_ctau$strata_size[i] <- sum(strata_indx)

    rows_L <- n_lr_test * (i-1) + seq_len(n_lr_test)
    rows_ests <- n_lr_total * (i-1) + seq_len(n_lr_total)
    lrindx_s <- findInterval(strata_ests$time[rows_ests],
                             strata_infl$infl_pt$time) + 1
    strata_ests$pt[rows_ests] <- c(0, strata_infl$infl_pt$pt_btwn, 0)[lrindx_s]
    nsratio <- n / sum(strata_indx)
    nrow_infl_s <- nrow(strata_infl$infl_pt$L_pt_btwn)
    L_nulldist[rows_L, ] <- -L_pi_unstrat
    L_nulldist[rows_L, strata_indx] <- L_nulldist[rows_L, strata_indx,
                                                  drop = FALSE] +
      rbind(nsratio * strata_infl$infl_pt$L_pt_btwn[1, , drop = FALSE],
            nsratio * strata_infl$infl_pt$L_pt_btwn,
            nsratio *
              strata_infl$infl_pt$L_pt_btwn[nrow_infl_s,
                                            ,
                                            drop = FALSE])[lrindx_s[test_indx],
                                                           ,
                                                           drop = FALSE]
  }
  if (any(new_out_of_window)){
    strata_ests$out_of_window <- strata_ests$out_of_window |
      rep(new_out_of_window, n_strata)
    L_nulldist[rep(new_out_of_window[test_indx], n_strata), ] <- 0
    lrindx_range <- range(lrindx[test_indx & !new_out_of_window])
    test_window[1] <- max(test_window[1],
                          pooled_infl$infl_pt$time[lrindx_range[1]])
    if (lrindx_range[2] < n_lr_total)
      test_window[2] <- min(test_window[2],
                            pooled_infl$infl_pt$time[lrindx_range[2] + 1])
  }

  # results
  teststat <- max(abs(strata_ests$wt[!strata_ests$out_of_window] *
                        (strata_ests$pt[!strata_ests$out_of_window] -
                           strata_ests$unstrat_pt[!strata_ests$out_of_window])))
  sims <- replicate(B, {
    wt <- stats::rnorm(n, 1, 1)
    max(abs(test_wts * c(L_nulldist %*% wt / n)))
  })
  critval <- stats::quantile(sims, level)
  strata_ctau$aggregate_teststat <- sum(strata_ctau$teststat)
  strata_ctau$aggregate_df <- sum(strata_ctau$df)
  strata_ctau$aggregate_pval <- stats::pchisq(strata_ctau$aggregate_teststat[1],
                                              strata_ctau$aggregate_df[1],
                                              lower.tail = FALSE)

  res <- list(n = n, n_unique = n_unique, n_miss = n_miss, teststat = teststat,
              pval = mean(sims >= teststat), critval = critval,
              testresult = teststat > critval, sims = sims, level = level, B = B,
              timerange = test_window, wt_name = wt_name, eventrange = eventrange,
              strata_ests = strata_ests, strata_ctau = strata_ctau)
  class(res) <- "quasiindep_strat_test"
  res
}


#' @rdname test_quasiindep_covariates
#' @export
plot.quasiindep_strat_test <- function(x, level, ...){
  if (missing(level)) level <- x[["level"]]
  stopifnot(is.numeric(level), is.vector(level), all(!is.na(level)))
  stopifnot(length(level) == 1, all(level > 0), all(level < 1))
  q <- stats::quantile(x[["sims"]], level)
  ests <- x[["strata_ests"]]
  ests$lower <- ifelse(ests$out_of_window, 0,
                       pmax(0, ests$unstrat_pt - (q / abs(ests$wt))))
  ests$upper <- ifelse(ests$out_of_window, 1,
                       pmin(1, ests$unstrat_pt + (q / abs(ests$wt))))

  with(ests,
       ggplot2::ggplot(ests) +
         ggplot2::geom_step(ggplot2::aes(x = time, y = lower),
                            linetype = "dashed", direction = "hv") +
         ggplot2::geom_step(ggplot2::aes(x = time, y = upper),
                            linetype = "dashed", direction = "hv") +
         ggplot2::geom_step(ggplot2::aes(x = time, y = pt, group = strata,
                                         color = "stratified NPMLE"),
                            direction = "hv") +
         ggplot2::geom_step(ggplot2::aes(x = time, y = unstrat_pt,
                                         color = "unstratified NPMLE"),
                            direction = "hv") +
         ggplot2::scale_color_manual(breaks = c("stratified NPMLE",
                                                "unstratified NPMLE"),
                                     values = c("maroon", "black"),
                                     guide =
                                       ggplot2::guide_legend(override.aes =
                                                               list(lwd = 2))) +
         ggplot2::labs(x = "Time", y = "Selection probability", color = "") +
         ggplot2::coord_cartesian(ylim = c(0, 1.01), xlim = x[["timerange"]],
                                  expand = FALSE) +
         ggplot2::theme_classic() +
         ggplot2::theme(legend.position = "bottom"))
}

#' @rdname test_quasiindep_covariates
#' @export
print.quasiindep_strat_test <- function(x, ...){
  pval1 <- format(x[["strata_ctau"]]$aggregate_pval[1], digits = 2, nsmall = 1)
  result1 <- x[["strata_ctau"]]$aggregate_pval[1] < (1 - x[["level"]])
  pval2 <- format(x[["pval"]], digits = 2, nsmall = 1)
  pval2 <- ifelse(pval2 == "0.0",
                  paste0("<", 1 / x[["B"]]),
                  paste0("=", pval2))

  cat(paste0("\n",
             "Two-stage quasi-independent truncation test involving covariates.",
             "\n",
             "\n",
             "  Unstratified NPMLE fit based on ", x[["n"]], " complete samples (",
             x[["n_miss"]], " dropped due to missingness)",
             "\n",
             "    with ", x[["n_unique"]], " unique event times.",
             "\n",
             "  Min event time: ", x[["eventrange"]][1],
                "\n",
             "  Max event time: ", x[["eventrange"]][2],
             "\n",
             "\n",
             "  Stage 1 (within strata quasi-independence).",
                "\n",
             "  Test statistic: chisq=",
               format(x[["strata_ctau"]]$aggregate_teststat[1],
                      digits = 2, nsmall = 1),
               ", df=", x[["strata_ctau"]]$aggregate_df[1],
               "\n",
             "  Significant evidence ", ifelse(result1, "", "not "),
               "found against quasi-independence (p=", pval1, ")",
             "\n",
             "\n",
             "  Stage 2 (across strata equality of selection probabilities).",
                "\n",
             "  Test statistic: ",
                   format(x[["teststat"]], digits = 2, nsmall = 1),
                "\n",
             "    Time range:   [", x[["timerange"]][1], ", ",
                    x[["timerange"]][2], "]",
                  "\n",
             "    Time weights: ", x[["wt_name"]],
                "\n",
             "  Critical value: ",
                  format(x[["critval"]], digits = 2, nsmall = 1),
                " at level ", x[["level"]],
                "\n",
             "  Significant evidence ", ifelse(x[["testresult"]], "", "not "),
             "found against equality across strata (p", pval2, ").",
                "\n",
             "  Results based on ", x[["B"]], " simulations from the estimated ",
             "null distribution.",
             "\n",
             "\n",
             "Summary of the ", nrow(x[["strata_ctau"]]), " covariate groups used ",
             "in the test:",
             "\n"))

  sort_indx <- order(x[["strata_ctau"]][, "strata_size", drop = TRUE],
                     decreasing = TRUE)
  strata_summary <- x[["strata_ctau"]][sort_indx,
                                       c("strata",
                                         "strata_size"),
                                       drop = FALSE]
  colnames(strata_summary) <- c("stratum", "size")
  if (nrow(x[["strata_ctau"]]) > 10){
    print(strata_summary[1:10, , drop = FALSE],
          row.names = FALSE)
    cat("  ...")
    cat("\n\n")
  }else{
    print(strata_summary, row.names = FALSE)
    cat("\n")
  }
  invisible(x)
}

#' Internal function to compute the estimated influence function for the NPMLE.
#'
#' @description
#' Computes the estimated influence function for the event time cdf,
#' cumulative hazard, normalized inverse selection probability weights, and
#' selection probability as a function of time.
#' Does not check for valid input arguments.
#'
#' @param masses Part of the output from [npmle()].
#' @param J A matrix with `J[i,j]`=`TRUE` if the `i`th follow up time is in the
#'   `j`th truncation interval. Will be computed from `masses` if not supplied.
#' @param ainv Whether the influence values for the normalized inverse
#'   probability weights should be returned (default `FALSE`).
#' @param cumhaz Whether the influence values for the event time cumulative
#'   hazard should be returned (default `FALSE`).
#' @param msel_prob Whether the influence values for the overall/marginal
#'   selection probability should be returned (default `FALSE`).
#' @param infl_at Whether the influence values for the normalized selection
#'   probabilities (at all times) should be returned (default `FALSE`).
#' @param infl_pt Whether the influence values for the selection
#'   probabilities (at all times) should be returned (default `FALSE`).
#' @param L_F The matrix of influence function values for the NPMLE event time
#'   cdf. The rows should correspond to unique ordered observed event times, while
#'   the columns should match the order of `masses`.
#'   If `NULL`, will be computed.
#' @param already_sorted Whether `masses` (and `J`,`L_F`, if supplied) was
#'   computed from sorted event times (default `FALSE`). If `TRUE` then less
#'   computations are required.
#'
#' @details
#' see Vazquez and Xie (2025).
#'
#'
#' @returns A list containing requested influence values for the NPMLE, to be
#' used for constructing standard errors.
#'
#' @keywords internal
#'
#' @noRd
#'
#' @references
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
npmle_infl <- function(masses, J = NULL, ainv = FALSE, cumhaz = FALSE,
                       msel_prob = FALSE, infl_at = FALSE, infl_pt = FALSE,
                       L_F = NULL, already_sorted = FALSE){
  n <- nrow(masses)
  C_lprod <- function(A) {rbind(A, -colSums(A))}
  C_rprod <- function(A) {sapply(seq_len(n - 1),
                                 FUN = function(i) A[, i] - A[, n])}
  Ct_lprod <- function(A) {sapply(seq_len(ncol(A)),
                                  FUN = function(i) A[-n, i] - A[n, i])}

  if (already_sorted == FALSE){
    sort_indx <- order(masses$time)
    unsort_indx <- order(sort_indx)
    masses <- masses[sort_indx, , drop = FALSE]
    if (!is.null(J)){
      J <- J[sort_indx, , drop = FALSE]
      J <- J[, sort_indx, drop = FALSE]
    }
  }else{
    sort_indx <- seq_len(n)
    unsort_indx <- seq_len(n)
  }

  if (is.null(J)){
    J <- apply(masses[, c("ltrunc", "rtrunc")], MARGIN = 1, FUN = function(lr){
      (lr[1] < masses$time) & (masses$time <= lr[2])
    })
  }

  a <- 1 / (n * masses$f)
  S <- stats::toeplitz(rep(1, n), r = c(1, rep(0, n - 1)))
  S_inv <- stats::toeplitz(c(1, -1, rep(0, n - 2)), r = c(1, rep(0, n - 1)))
  B <- S %*% diag(masses$f)
  B_inv <- diag(1 / masses$f) %*% S_inv
  unique_indx <- !duplicated(masses$time, fromLast = TRUE)
  match_unique_indx <- match(masses$time, masses$time[unique_indx])
  Atilde <- J %*% diag(1 / masses$plr^2) %*% t(J) %*% S_inv / n

  # row sorted event time, col original order
  if (is.null(L_F)){
    H <- sapply(unsort_indx, FUN = function(i) {
      ((masses$time[i] <= masses$time[unique_indx]) / a[i]) -
        (cumsum(J[, i] / a^2)[unique_indx] / (n * masses$plr[i]))
    })
    A <- S %*% diag(1 / a^2) %*% Atilde / n
    IA <- -A
    diag(IA) <- 1 + diag(IA)
    L_F <- (B %*% C_lprod(solve(Ct_lprod(C_rprod(B_inv %*% IA %*% B)),
                                Ct_lprod(B_inv %*% H[match_unique_indx, ])))
    )[unique_indx, ]
  }
  var_F <- rowMeans(L_F^2) / n
  if (already_sorted == FALSE) a <- a[unsort_indx]


  L_ainv <- NULL
  var_ainv <- NULL
  if (ainv == TRUE){
    L_ainv <- sapply(unsort_indx,  FUN = function(i){
      -((J[unsort_indx, i] / masses$plr[i]) - a -
          (Atilde[unsort_indx, , drop = FALSE] %*%
             L_F[match_unique_indx, i])) / a^2
    })
    var_ainv <- rowMeans(L_ainv^2) / n
  }

  cumhazard <- NULL
  L_cumhaz <- NULL
  var_cumhaz <- NULL
  if (cumhaz == TRUE){
    npcdf <- cbind(masses$time, pmin(cumsum(masses$f), 1))[unique_indx, ]
    lcont_survival <- 1 - c(0, npcdf[-nrow(npcdf), 2])
    dF <- diff(c(0, npcdf[, 2]))
    cumhazard <- cbind(npcdf[, 1], cumsum(dF / lcont_survival))
    n_unique_times <- nrow(L_F)
    L_cumhaz <- apply(L_F, MARGIN = 2, FUN = function(lf) {
      cumsum((diff(c(0, lf)) / lcont_survival) +
               (c(0, lf[-n_unique_times]) * dF / lcont_survival^2))
    })
    var_cumhaz <- rowMeans(L_cumhaz^2) / n
  }

  msel <- 1 / mean(1 / masses$plr)
  L_msel <- NULL
  influence_msel_prob <- NULL
  influence_at <- NULL
  influence_pt <- NULL
  if (infl_at || infl_pt){
    lr_times <- unique(sort(c(masses$ltrunc, masses$rtrunc)))
    n_lr <- length(lr_times)
    a_times <- lr_times[-n_lr] + diff(lr_times) / 2

    J_at <- apply(cbind(masses$ltrunc, masses$rtrunc), MARGIN = 1,
                  FUN = function(lr) (lr[1] < a_times) & (a_times <= lr[2]))
    at <- c(J_at %*% (1 / masses$plr) / n)
    L_at <- (J_at[, unsort_indx, drop = FALSE] %*%
               diag(1 / masses$plr[unsort_indx])) -
      matrix(rep(at, n), ncol = n) -
      (J_at %*% diag(1 / masses$plr^2) %*%
         t(J) %*% S_inv %*% L_F[match_unique_indx, ] / n)
    var_at <- rowMeans(L_at^2) / n

    if (infl_at == TRUE){
      influence_at$time <- lr_times
      influence_at$at_btwn <- at
      influence_at$var_at_btwn <- var_at
      influence_at$L_at_btwn <- L_at
    }

    if (infl_pt == TRUE){
      influence_pt$time <- lr_times
      influence_pt$pt_btwn <- at * msel

      L_msel <- (colSums(diag(1 / masses$plr^2) %*%
                           t(J) %*% S_inv %*% L_F[match_unique_indx, ] / n) -
                   (1 / masses$plr[unsort_indx]) +
                   (1 / msel)) *
        msel^2
      influence_pt$L_pt_btwn <- (L_at * msel) + (at %*% t(L_msel))
      influence_pt$var_pt_btwn <- rowMeans(influence_pt$L_pt_btwn^2) / n
    }
  }

  if (msel_prob){
    influence_msel_prob$prob <- msel
    influence_msel_prob$L <- L_msel
    if (is.null(influence_msel_prob$L)){
      influence_msel_prob$L <-
        (colSums(diag(1 / masses$plr^2) %*%
                   t(J) %*% S_inv %*% L_F[match_unique_indx, ] / n) -
           (1 / masses$plr[unsort_indx]) +
           (1 / msel)) *
        msel^2
    }
    influence_msel_prob$var <- mean(influence_msel_prob$L^2) / n
  }

  list(a = a, L_F = L_F, var_F = var_F, L_ainv = L_ainv, var_ainv = var_ainv,
       cumhaz = cumhazard, L_cumhaz = L_cumhaz, var_cumhaz = var_cumhaz,
       infl_at = influence_at, infl_pt = influence_pt,
       infl_msel_prob = influence_msel_prob)
}

---
title: "Cox Regression Under Quasi-Independent Double Truncation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Cox Regression Under Quasi-Independent Double Truncation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 5,
  fig.height = 3,
  fig.align = "center"
)
```

```{r setup}
library(survdt)
```

## Inverse probability weighted estimators

The `survdt` package implements several methods for Cox regression with 
quasi-independent double truncation.
We illustrate them through an application to the `aids` dataset, which holds 
records of AIDS incubation times from a retrospective sample of patients with 
transfusion-acquired HIV.
The inclusion criteria were that the patient was diagnosed with AIDS between the
discovery of the disease in 1982 and the end of the study in 1986, with time 
measured in months since HIV infection.
The event time of interest is the AIDS incubation time `incu`, with the truncation 
times contained in `ltrunc` and `rtrunc` respectively. See `?survdt::aids` for more
details.

To fit a Cox model with nonparametric inverse probability weighting, use the
`coxph_indtrunc()` function in the `survdt` package.
The first input argument should be a model formula with a `Survdt` or `Survdt2`
response.
`Survdt()` is used under time-independent covariates (see `?survdt::Survdt`), 
while `Survdt2` is used when there are time-varying covariates (see 
[Time-varying covariates] for an example).
```{r}
aids_fit <- coxph_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp, 
                           data = aids)
aids_fit
confint(aids_fit)
```

The default setting applies robust inverse probability of non-truncation 
weights and fits the model by maximizing a weighted partial likelihood.
Several choices of time-varying weights are available through the input 
argument `ipw_type`.
They offer improved robustness to outliers and extreme inverse probability weights
when compared to standard inverse probability weighting (`ipw_type="W-1"`).

```{r}
coxph_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp, 
               data = aids,
               ipw_type = "W-1")
```

In this case, the coefficient estimates change under different weightings but the
z-statistics are fairly similar.
In [Sensitivity analysis for positivity violations], we find that different
weight choices can have a greater impact in the `aids` data during sensitivity
analysis.
See `?survdt::coxph_indtrunc` for more details on the different weighting options.

Lastly, stratified Cox models can be fit by specifying a `strata_formula`.
See [Stratified Cox models] for an example.

### Plotting survival and hazard estimates

Given a fitted Cox model, the hazard or survival function estimates can be
plotted using the `plot_coxsurv()` function in the `survdt` package.
The first input argument should be a model object from `coxph_indtrunc()`, while
the second input argument should be a data frame with the covariate values to 
plot the conditional survival or hazard function for.

```{r, fig.height = 3.5}
plot_coxsurv(aids_fit, 
             newdata = aids[c(1,40), ],
             target = "survival")

plot_coxsurv(aids_fit, 
             newdata = aids[c(1, 40, 200), ], 
             ci = FALSE, 
             target = "cumhaz")
```

Note that `plot_coxsurv()` prints a message telling us it is computing the 
baseline hazard and its standard errors, since (by default) this was not done 
when fitting the model.
Use `basehaz=TRUE` when running `coxph_indtrunc()` to remove the need to re-compute
the baseline hazard for each plot.

### Assumption checking and model diagnostics

The statistical methods implemented in `coxph_indtrunc()` require two key
assumptions to ensure unbiased inference in the presence of double truncation.
First, the truncation times should be independent of the event time and 
covariates within the observable data region, which is known as quasi-independence.
Second, all possible event times should have a positive probability of being
observed, i.e. not being truncated, which is known as the positivity assumption.
The `survdt` package includes diagnostics for both of these assumptions.
They are described in the following two sections.
Residual diagnostics for the Cox model specification are available through the
`residuals()` function (see `survdt::coxph_indtrunc`).

#### Testing quasi-independent truncation with covariates

The `survdt` package includes the function `test_quasiindep_covariates()` to test
for violations of the quasi-independence assumption.
This is accomplished through a two-stage test based on stratifying by covariate
values.
The first stage checks for dependence within strata, while the second stage checks
across strata.
The test can be repeated with different stratifications when there are many 
covariates involved.
See `?survdt::test_quasiindep_covariates` for more details.

The first input argument should be a model formula with a `Survdt` response and
factor variables defining covariate strata on the right hand side.
Non-factor variables will be automatically coerced to factors.
Below we run the diagnostic test on the `aids` data, with age group defining the
covariate strata.
```{r}
qind_test <- test_quasiindep_covariates(Survdt(incu, ltrunc, rtrunc) ~ age_gp, 
                                        data = aids)
qind_test
```

Both stages of the test fail to find significant evidence of dependent truncation
in the `aids` data, so the quasi-independence assumption required by 
`coxph_indtrunc()` is reasonably satisfied.
The test results for the second stage can be plotted to visualize how strongly
the truncation times could depend on the covariates.
This is done by comparing the estimated selection (non-truncation) probabilities
across strata. 

```{r, fig.height = 3.5}
plot(qind_test)
```

The estimated selection probabilities all lie within the 95% uniform confidence 
band around the unstratified estimates, as expected given the p-value from 
the stage 2 test printed above.

#### Sensitivity analysis for positivity violations

The `survdt` package includes the function `positivity_sens_indtrunc()` to conduct
sensitivity analysis for the Cox model under potential violations of the positivity
assumption.
Given a sequence of potential values for the truncated baseline probability mass, 
it returns a corresponding sequence of adjusted coefficient estimates and associated 
sensitivity intervals which show how the fitted model could change due to 
non-positivity.
See `?survdt::positivity_sens_indtrunc` for more details.

The first input argument should be a model formula with `Survdt` response.
The truncated masses are specified through `trunc_mass`, and should not be too
close to one or the algorithm may not converge.
Many of the other input arguments are shared with `coxph_indtrunc()`.

First, we apply the sensitivity analysis with standard inverse probability 
weighting, which does not have a time-varying component.
```{r error=TRUE, warning=TRUE}
positivity_sens_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp, 
                         data = aids, ipw_type = "W-1",
                         trunc_mass = seq(0, 0.11, by = 0.01))

```

Note that the estimator failed to converge even at small truncated mass values.
This can occur for many reasons (see `?survdt::positivity_sens_indtrunc` for
more information).
In the `aids` data, it turns out that the estimated selection probability for the
largest observed incubation times approaches zero, leading to extreme inverse
probability weights.
Therefore we apply time-varying weights that reduce the influence of event time
outliers and extreme weights by using the option `ipw_type = "W-asurv"`.
This is the default weighting used in `positivity_sens_indtrunc()` and 
`coxph_indtrunc()`.

```{r}
sens <- positivity_sens_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp, 
                                 data = aids,
                                 trunc_mass = seq(0, 0.11, by = 0.01))
sens

```

These robust weights improve the numerical convergence of the sensitivity analysis
estimators by a substantial amount.
The estimates can also be plotted to better visualize the results.

```{r}
plot(sens)
plot(sens, hazratio = TRUE)
```

Clearly the coefficient estimate for adults is highly sensitive to potential
positivity violations, while the coefficient estimate for children is more 
stable.


### Stratified Cox models

The `survdt` package supports stratified Cox models, where the baseline hazard
is allowed to differ across strata.
In the `coxph_indtrunc()` function, stratification factors are specified through
the `strata_formula` input argument, which should be a one-sided formula with
no response variable.
We illustrate this with a simulated sample featuring non-proportional hazards,
which is included in the `survdt` package as `nonprop_sample`.
The data has a binary covariate `group` with a time-varying log hazard ratio of 
$0.7\times 1(t>0.6)$ and a continuous covariate `x` with a constant log hazard
ratio of $-0.5$.

We fit a model stratified by `group`.
```{r}
strat_fit <- coxph_indtrunc(Survdt(event_time, ltrunc, rtrunc) ~ x, 
                            data = nonprop_sample, 
                            strata_formula = ~group)
strat_fit
confint(strat_fit)
```

Plotting the cumulative hazard estimates for the two groups at the same `x`, 
we see that they are indeed approximately equal until time 0.6, after which the
hazard for group 1 is about $\exp(0.7)\approx 2$ times as large as the hazard
for group 0.

```{r, fig.height=3.5}
plot_coxsurv(strat_fit, 
             newdata = data.frame(group = c(0, 1), x = c(1, 1)),
             target = "cumhaz")
```


### Time-varying covariates

The `survdt` package supports time-varying covariates in the `coxph_indtrunc()`
function.
We illustrate this with a simulated sample featuring non-proportional hazards,
which is included in the `survdt` package as `nonprop_sample`.
The data has a binary covariate `group` with a time-varying log hazard ratio of 
$0.7\times 1(t>0.6)$ and a continuous covariate `x` with a constant log hazard
ratio of $-0.5$.

We fit a model with the correct time-varying effect of `group`.
First, we split each row in the data frame into two possible observation windows
corresponding to $t\leq 0.6$ and $t>0.6$.
The `survdt` package includes the function `time_split()` for this purpose.
See `?survdt::time_split` for more details.

```{r}
split_dat <- time_split(nonprop_sample, 
                        cut = 0.6, 
                        stop_name = "event_time", 
                        start = "start",
                        event = "event",
                        id = "id")
```

Then we add a column `group_tgr06` containing the time-varying covariate 
$1(t>0.6)\times$`group`, and fit the Cox model using a `Survdt2` response.
```{r}
split_dat$group_tgr06 <- (split_dat$event_time > 0.6) * split_dat$group
tvar_fit <- coxph_indtrunc(Survdt2(start, event_time, event, ltrunc, rtrunc, id) ~ 
                             group_tgr06 + x, 
                           data = split_dat)
tvar_fit
confint(tvar_fit)
```

The confidence intervals from the estimated Cox model contain the true values 
of 0.7 and -0.5.

---
title: "Nonparametric Analysis of Doubly Truncated Data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Nonparametric Analysis of Doubly Truncated Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 5,
  fig.height = 3,
  fig.align = "center"
)
```

```{r setup}
library(survdt)
```

## Doubly truncated data

Double truncation occurs when a data sample only includes subjects whose event 
time lies within a random truncation interval.
For example, the `survdt` package includes the doubly truncated dataset `aids`,
which holds records of AIDS incubation times from a retrospective sample of 
patients with transfusion-acquired HIV.
The inclusion criteria were that the patient was diagnosed with AIDS between the
discovery of the disease in 1982 and the end of the study in 1986, with time 
measured in months since HIV infection.
This sampling scheme was chosen so that the precise HIV infection time could be 
ascertained retrospectively from a blood transfusion registry.
The event time of interest is the AIDS incubation time `incu`, with the truncation 
times contained in `ltrunc` and `rtrunc` respectively. See `?survdt::aids` for more
details.

The `survdt` package has a special `Survdt` data structure designed to store 
doubly truncated data. Below we create a `Survdt` object using the `aids` dataset
and plot a simple visualization of the sample. See `?survdt::Survdt` for more 
details.

```{r}
y <- Survdt(aids$incu, aids$ltrunc, aids$rtrunc)
head(y)
plot(y)
```

Each row of this plot contains a subject's truncation interval as a gray rectangle
and the subject's event time in black, with rows sorted by event time.

## Nonparametric maximum likelihood estimation

The nonparametric maximum likelihood estimator (NPMLE) of the event time cumulative
distribution function (cdf) is a right-continuous step function with point masses at
the observed event times.
Think of it like an empirical distribution function that is adjusted for double
truncation bias.
It is estimated iteratively via an EM algorithm.
The underlying assumptions for the NPMLE are discussed in 
[Assumptions required for the NPMLE].

The `survdt` package includes the function `npmle()` to compute the NPMLE
and its standard errors.
The first input argument should be a `Survdt` object or expression, which is 
evaluated using the variables found in the `data` argument (if provided).
See `?survdt::npmle` for more details.

Below we fit the NPMLE on the `aids` data and plot both the estimated survival 
function and estimated cumulative hazard function, along with pointwise 95% 
confidence intervals.
Other plot options include the estimated selection (non-truncation) probability 
over time and the estimated inverse probability weights.
See `?survdt::plot.npmle` for more details.
```{r}
npfit <- npmle(Survdt(incu, ltrunc, rtrunc), data = aids, cumhaz = TRUE)
npfit

plot(npfit)
```
```{r}
plot(npfit, target = "cumhaz")
```
These confidence intervals are computed efficiently through closed-form standard
errors, i.e. without relying on bootstrap or other resampling methods, in 
contrast to some other R packages for double truncation.

### Testing for event time differences across multiple groups

The `aids` data has the factor variable `age_gp` which categorizes each subject
based on age at HIV infection.
The three age groups are children (age $\leq 4$), adults (age $4-59$), and 
elderly (age $>59$).
The group sizes are:
```{r}
summary(aids$age_gp)
```

In order to test for differences in incubation times across the three age groups,
the `survdt` package provides two general options.

First, the data can be analyzed through a Cox proportional hazards model using
methods that correct for double truncation bias.
See `vignette("cox-regression-quasiindep", package = "survdt")` and 
`?survdt::coxph_indtrunc` for more details on how to do this with the `survdt`
package.

Second, the data can be analyzed nonparametrically to detect any difference in the 
incubation time distributions without assuming proportional hazards.
The `survdt` package implements this in the function `test_samplediff_indtrunc()`.
The first input argument should be a formula with a `Survdt` response and 
variables defining the groups, i.e. strata, to test across on the right hand side.
The test can be based on the supremum difference between survival functions or 
between cumulative hazards, both estimated by the NPMLE.
The p-value is computed by simulating from the estimated asymptotic null 
distribution of the test statistic.
See `?survdt::test_samplediff_indtrunc` for more details.

Below we test for differences in the AIDS incubation time distribution across 
the three age groups. 
We limit the range of the test to at most 70 months since HIV infection to avoid
the high variability associated with the tail of the distribution (see the plot 
in [Nonparametric maximum likelihood estimation]).

```{r}
agetest <- test_samplediff_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp, 
                                    data = aids,
                                    restr_times = c(NA, 70))
agetest
```

We find highly significant evidence that the incubation time distribution differs
across age groups.
The test result can be visualized through the following plot, which compares the
group-specific survival function estimates with the unstratified estimates.
Under the null hypothesis of equality across strata, the stratified estimates
should all lie within the 95% uniform confidence band, denoted by dashed lines.

```{r, fig.height = 3.5}
plot(agetest)
```
Plotting the test result is recommended in general, since it provides a better
understanding of the estimated effect size and power of the test.

### Assumptions required for the NPMLE

The NPMLE requires two key conditions. 
First, all possible event times must have a positive probability of being observed. 
This can be determined from the study design, as well as by comparing the observed 
event times to the range of possible event times based on external knowledge.
Second, the truncation times must be independent of the event time within the 
observable data region, commonly known as quasi-independence.
See [Testing for quasi-independent truncation] for more information on how to
test this assumption with the `survdt` package.

### Testing for quasi-independent truncation

The `survdt` package includes the function `test_quasiindep_ctau()` to test for
quasi-independence between the event and truncation times using the conditional 
Kendall's tau rank correlation.
The first input argument should be a `Survdt` object or expression, which is 
evaluated using the variables found in the `data` argument (if provided).
It computes the (event time, left truncation time) and 
(event time, right truncation time) rank correlations (`tau`) accounting for
double truncation, as well as a p-value for testing whether both are zero.
See `?survdt::test_quasiindep_ctau` for more details.

Below we apply this quasi-independence test to the `aids` data.

```{r}
test_quasiindep_ctau(Survdt(incu, ltrunc, rtrunc), data = aids)
```

Based on the small estimated correlations and moderate p-value, we do not find 
strong evidence against quasi-independence in the `aids` data.

### Testing for ignorable sampling bias

A unique feature of doubly truncated data, compared to one-sided truncation, is
that it is possible in some cases for there to be no effective sampling bias 
induced by truncation.
This occurs when the sampling probability is constant across all event times.
If the sampling bias is ignorable, standard methods which do not correct for 
double truncation bias may be used to analyze the data.

The `survdt` package includes the function `test_ignorability_indtrunc` to test for
ignorable sampling bias nonparametrically through the NPMLE.
The first input argument can be a `Survdt` object/expression or the fitted NPMLE
object returned by `npmle()`.
The test can be based on the supremum difference between the time-specific and
overall selection probabilities or the supremum difference between the NPMLE CDF
and the empirical CDF (not adjusted for double truncation).
The p-value is computed by simulating from the estimated asymptotic null 
distribution of the test statistic.
See `?survdt::test_ignorability_indtrunc` for more details.

Below we apply these tests to the `aids` data.
The output from `test_ignorability_indtrunc` has a `plot` method to help visualize
the deviation of the NPMLE from its reference value using a uniform 
confidence band.
For the CDF-based test, we limit the range of the test to at most 80 months since 
HIV infection to avoid the high variability associated with the tail of the 
distribution (see the plot in [Nonparametric maximum likelihood estimation]).

```{r, fig.height=3.5}
igtest_prob <- test_ignorability_indtrunc(npfit, test_type = "sel_prob")
igtest_prob
plot(igtest_prob)

igtest_cdf <- test_ignorability_indtrunc(npfit, test_type = "cdf", 
                                         restr_times = c(NA, 80))
igtest_cdf
plot(igtest_cdf)
```

Both tests show highly significant evidence against ignorable sampling bias in 
the `aids` data.
Thus this data should be analyzed with methods that directly account for double
truncation bias.

To illustrate the utility of plotting the test results, we run the CDF-based
test without restricting the time range for the test statistic.

```{r, fig.height=3.5}
igtest_cdf2 <- test_ignorability_indtrunc(npfit, test_type = "cdf")
igtest_cdf2
plot(igtest_cdf2)
```

This unrestricted test returns a much larger p-value, and after plotting the 
result it is evident that this is due to the large variance of the NPMLE cdf at
long incubation times.
Thus the unrestricted CDF test has low power to detect what is clearly a large 
deviation from ignorability.
This issue can occur when the estimated selection probabilities approach zero 
within the event time range.
The plot below confirms that this is the case for the `aids` data.

```{r}
plot(npfit, target = "sel_prob")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npests.R
\name{test_ignorability_indtrunc}
\alias{test_ignorability_indtrunc}
\alias{plot.ignorability_indtrunc_test}
\alias{print.ignorability_indtrunc_test}
\title{Test for non-ignorable sampling bias}
\usage{
test_ignorability_indtrunc(
  y,
  data = NULL,
  test_type = c("sel_prob", "cdf"),
  level = 0.95,
  B = 2000,
  restr_times = NULL,
  wt_fn = NULL
)

\method{plot}{ignorability_indtrunc_test}(x, level, ...)

\method{print}{ignorability_indtrunc_test}(x, ...)
}
\arguments{
\item{y}{A \code{Survdt} object or expression, or an \code{npmle} object. Observations
with missing values are discarded.
See \code{\link[=Survdt]{Survdt()}} and \code{\link[=npmle]{npmle()}}.}

\item{data}{An optional data frame used to evaluate \code{y}.}

\item{test_type}{The test statistic to use: \code{"sel_prob"} (default) or \code{"cdf"}.
See below.}

\item{level}{The confidence level for the test (default 0.95).}

\item{B}{The number of simulations from the estimated null distribution used
to compute the p-value (default 2000).}

\item{restr_times}{An optional numerical vector of length two giving a time
window to restrict the test statistic.
Values of \code{NA} are replaced with the minimum and maximum observed event
time respectively, i.e. no restriction.
The default setting is no restriction.}

\item{wt_fn}{An optional function that takes a numerical vector of times as
input and returns a vector of weights to apply in the test statistic.
If unspecified, all times have equal weight of 1.}

\item{x}{Any R object.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
\code{\link[=test_ignorability_indtrunc]{test_ignorability_indtrunc()}} returns an object of class
\code{ignorability_indtrunc_test}, i.e. a list with the following components:
\item{n}{The complete case sample size.}
\item{n_unique}{The number of unique event times among complete cases.}
\item{n_miss}{The number of discarded observations (with missing values).}
\item{teststat}{The value of the observed test statistic.}
\item{pval}{The p-value for the test.}
\item{critval}{The critical value for the test at the user-specified level.}
\item{testresult}{Whether the test resulted in rejecting the null hypothesis
of ignorable sampling bias.}
\item{sims}{A vector of the \code{B} simulations from the estimated null
distribution of the test statistic.}
\item{test_type}{The matched \code{test_type} value.}
\item{level}{The user-supplied \code{level}.}
\item{B}{The number of simulations specified by the user.}
\item{timerange}{A vector with the minimum and maximum times used in the
test.}
\item{wt_name}{A string giving the name of the user-supplied \code{wt_fn}, if any.}
\item{eventrange}{A vector with the unrestricted range of observed event times.}
\item{ests}{A data frame of estimates to be used by
\code{\link[=plot.ignorability_indtrunc_test]{plot.ignorability_indtrunc_test()}}.}

The \code{print} method for \code{ignorability_indtrunc_test} objects displays a summary
of the test results, while the \code{plot} method returns a visualization comparing
the NPMLE and its reference value under the null hypothesis using a uniform
confidence band, generated through \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
Implements two possible nonparametric tests for non-ignorable sampling bias
based on the NPMLE:
\enumerate{
\item check for non-constant selection probabilities across the event time range;
\item check for a difference between the estimated event time cumulative
distribution functions adjusted and not adjusted for double truncation.
}

Typically used to determine if a doubly truncated sample must be analyzed
with methods that account for truncation bias.
}
\details{
The sampling bias from double truncation is called ignorable when the
selection (non-truncation) probability is constant over all possible event
times.
Since all event times have an equal probability of being included in the sample,
the sampling mechanism does not alter their distribution, so standard analysis
methods can be applied to the data.

Define the normalized selection probability at any time \eqn{t} by
\eqn{a(t) = P(L<T \leq R|T=t)/P(L<T \leq R)}.
Its estimate \eqn{\widehat{a}(t)} is a left-continuous step function with
jumps at the observed truncation times.
By default, \code{\link[=test_ignorability_indtrunc]{test_ignorability_indtrunc()}} tests for non-ignorable sampling
bias through the test statistic
\deqn{\sup_{\min_i T_i \leq t \leq \max_i T_i} |\widehat{a}(t) - 1|}
with the p-value computed by simulating from the estimated asymptotic null
distribution.
Another option is to compare two estimated event time cdfs: the NPMLE
\eqn{\widehat{F}(t)} which adjusts for double truncation and the empirical
cdf \eqn{\widehat{F}^*(t)} which does not adjust for double truncation.
The test statistic would then be
\deqn{\sup_t |\widehat{F}(t) - \widehat{F}^*(t)|.}
}
\section{Restricted time window and weighted tests}{

When the NPMLE has high variance at extreme event times, the test may suffer
from reduced power.
One possible remedy is to restrict the time range of the test to some interval
\eqn{[\tau_1, \tau_2]} contained in \eqn{[\min_i T_i, \max_i T_i]}.
Another option is to down-weight event times with large variance through a
weight function \eqn{w(t)}.
The stabilized test statistic will have the form
\deqn{\sup_{\tau_1 \leq t \leq \tau_2} |w(t)(\widehat{h}(t) - h(t))|}
where \eqn{\widehat{h}(t)} is the NPMLE without assuming the null hypothesis
holds and \eqn{h(t)} is the reference value under the null.
See \code{vignette("nonparametric-analysis", package = "survdt")} for an illustration
with the \link{aids} data assuming vignettes were built when installing \link{survdt}.
}

\section{No effective truncation}{

A doubly truncated sample has no effective truncation when
\eqn{\max_i L_i < \min_i T_i} and \eqn{\max_i T_i \leq \min_i R_i}.
The estimated selection probability is equal to one across all event times
in this case, and its standard errors may not be reliable.
Typically one would conclude that the truncation mechanism is ignorable in
these types of samples, so standard methods can be applied for data analysis.
}

\examples{
igtest <- test_ignorability_indtrunc(Survdt(incu, ltrunc, rtrunc),
                                     data = aids)
igtest
plot(igtest)

}
\references{
de Uña‐Álvarez, J. (2023). Testing for an ignorable sampling bias under random
double truncation. \emph{Statistics in Medicine}, \strong{42}(20), 3732--3744.

Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
estimators for doubly truncated Cox regression with closed-form standard
errors. \emph{Lifetime Data Analysis}, \strong{31}(2), 364–-393.
}
\seealso{
\itemize{
\item \code{\link[=npmle]{npmle()}}: the nonparametric maximum likelihood estimator which forms the
basis for these tests.
\item \code{vignette("nonparametric-analysis", package = "survdt")}: an illustration
of ignorability tests applied to the \link{aids} data, if vignettes were built
when installing \link{survdt}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npests.R
\name{test_quasiindep_ctau}
\alias{test_quasiindep_ctau}
\alias{print.quasiindep_ctau_test}
\title{Test for dependence between truncation times and the event time}
\usage{
test_quasiindep_ctau(y, data = NULL, singular_thresh = 1e-09)

\method{print}{quasiindep_ctau_test}(x, ...)
}
\arguments{
\item{y}{A \code{Survdt} object or expression. Observations with missing values are
discarded. See \code{\link[=Survdt]{Survdt()}}.}

\item{data}{An optional data frame used to evaluate \code{y}.}

\item{singular_thresh}{The numerical threshold for determining whether the
variance matrix for the correlations is singular (default \code{1e-9}).
This can occur when the truncation times are a constant distance from each
other.}

\item{x}{Any R object.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
\code{\link[=test_quasiindep_ctau]{test_quasiindep_ctau()}} returns an object of class \code{quasiindep_ctau_test},
i.e. a list with the following components:
\item{tau}{A numerical vector of length two containing the estimated
conditional Kendall's tau rank correlation (1) between the event time and
left truncation time, and (2) between the event time and right truncation
time.}
\item{teststat}{The chi-squared test statistic for testing the null hypothesis
that both correlations are zero.}
\item{df}{The degrees of freedom for \code{teststat}.}
\item{pval}{The p-value for testing that both correlations are zero.}
\item{var_matrix}{The estimated variance matrix for the two conditional
Kendall's tau correlation estimates.}
\item{singular}{Whether \code{var_matrix} was determined to be singular.}
\item{prob_comp}{The fraction of subject pairs that are comparable.}

The \code{print} method for \code{quasiindep_ctau_test} objects displays a summary of
the test results.
}
\description{
Computes the conditional Kendall's tau rank correlation between the event
time and each of the two truncation times, accounting for double truncation.
Then tests whether both correlations are equal to zero.

Typically used to determine if nonparametric analysis can be done using the
NPMLE, which requires quasi-independence (see \code{\link[=npmle]{npmle()}}).
}
\details{
The conditional Kendall's tau rank correlation between the left truncation
time and event time is estimated by the average of
\deqn{\text{sign}\{(T_i - T_j)(L_i - L_j)\}}
across all comparable pairs of subjects \eqn{(i,j)}.
A pair is called comparable if \eqn{L_j<T_i \leq R_j} and \eqn{L_i<T_j \leq R_i},
which ensures that the order of the pair's times is not influenced by truncation
under quasi-independence.
Similarly, the conditional Kendall's tau rank correlation between the right
truncation time and event time is estimated by the average of
\deqn{\text{sign}\{(T_i - T_j)(R_i - R_j)\}}
across the same comparable pairs of subjects \eqn{(i,j)}.

Under quasi-independent truncation, both conditional Kendall's tau rank
correlations should converge to zero in large samples.
}
\examples{
test_quasiindep_ctau(Survdt(incu, ltrunc, rtrunc), aids)

}
\references{
Martin, E. C., & Betensky, R. A. (2005). Testing quasi-independence of failure
and truncation times via conditional Kendall's tau. \emph{Journal of the American
Statistical Association}, \strong{100}(470), 484--492.
}
\seealso{
\itemize{
\item \code{\link[=test_quasiindep_covariates]{test_quasiindep_covariates()}}: a test for dependent truncation when
covariates are involved, e.g. when aiming to analyze data with
\code{\link[=coxph_indtrunc]{coxph_indtrunc()}}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipw_ests.R
\name{coxph_indtrunc}
\alias{coxph_indtrunc}
\alias{vcov.coxph_indtrunc}
\alias{coef.coxph_indtrunc}
\alias{confint.coxph_indtrunc}
\alias{residuals.coxph_indtrunc}
\alias{print.coxph_indtrunc}
\title{Cox regression under quasi-independent double truncation.}
\usage{
coxph_indtrunc(
  formula,
  data = NULL,
  strata_formula = NULL,
  ipw_type = c("W-asurv", "W-surv", "W-1", "W-a"),
  se = TRUE,
  robust_se = TRUE,
  basehaz = FALSE,
  restr_times = NULL,
  npmle_fit = NULL,
  robust_infl = NULL,
  simple_labels = FALSE,
  keep_x = FALSE
)

\method{vcov}{coxph_indtrunc}(object, ...)

\method{coef}{coxph_indtrunc}(object, ...)

\method{confint}{coxph_indtrunc}(object, parm, level = 0.95, ...)

\method{residuals}{coxph_indtrunc}(
  object,
  type = c("martingale", "deviance"),
  per_subject = type == "deviance",
  ...
)

\method{print}{coxph_indtrunc}(x, ...)
}
\arguments{
\item{formula}{A model formula with a \code{Survdt} or \code{Survdt2} response.
Subjects with any missing values are discarded.
See \code{\link[=Survdt]{Survdt()}} and \code{\link[=Survdt2]{Survdt2()}}.}

\item{data}{An optional data frame used to evaluate \code{formula} and
\code{strata_formula}.}

\item{strata_formula}{An optional one-sided formula with stratification
variables on the right hand side, to fit a stratified Cox model.}

\item{ipw_type}{Type of inverse probability weights.
\enumerate{
\item \code{"W-asurv"} (the default): stabilized robust time-varying weights based
on the NPMLE survival function.
\item \code{"W-surv"}: robust time-varying weights based on the NPMLE survival function.
\item \code{"W-1"}: standard weights with no time-varying component (Rennert and Xie, 2018)
\item \code{"W-a"}: stabilized weights (Mandel et al., 2018)
\item For custom time-varying weights, provide a function that takes a numerical
vector of times as input and returns a numerical vector of positive weights.
}}

\item{se}{Whether to calculate standard errors (default \code{TRUE}).}

\item{robust_se}{Whether to calculate standard errors that are robust to
model misspecification (default \code{TRUE}).
Ignored if \code{se}=\code{FALSE}.}

\item{basehaz}{Whether to return the baseline hazard estimate
(default \code{FALSE}).}

\item{restr_times}{An optional numerical vector of length two giving a time
window to restrict the score function.
Values of \code{NA} are replaced with the minimum and maximum observed event
time respectively, i.e. no restriction.
The default setting is no restriction.}

\item{npmle_fit}{An optional pre-computed \code{npmle} object fit on the complete
cases.
If it was fit with \code{ainv}=\code{FALSE} (or \code{infl}=\code{FALSE} when \code{se}=\code{TRUE}) then
it is ignored and a new NPMLE is fit.
See \code{\link[=npmle]{npmle()}}.}

\item{robust_infl}{A square matrix of influence values for the event weight,
with one column per individual and one row per follow up time, both in the
order that the subject appears in the response variable.
Only used if \code{ipw_type} is a function and \code{robust_se}=\code{TRUE}.}

\item{simple_labels}{Whether to drop the factor names from the stratum labels
(Default \code{FALSE}).}

\item{keep_x}{Whether to include the model matrix in the \code{cox_fit} component
of the return value (Default \code{FALSE}).}

\item{object, x}{A \code{coxph_indtrunc} object.}

\item{...}{Additional arguments passed to other methods.}

\item{parm}{A numeric or character vector specifying a set of model
parameters.
Defaults to all identifiable (non-missing) coefficients.}

\item{level}{Confidence level for confidence intervals.}

\item{type}{The type of residuals to return.}

\item{per_subject}{Whether to return per-subject residuals as opposed to
per-observation residuals, for models with \code{Survdt2} data.
Defaults to \code{TRUE} for deviance residuals and \code{FALSE} otherwise.}
}
\value{
\code{\link[=coxph_indtrunc]{coxph_indtrunc()}} returns an object of class \code{coxph_indtrunc}, i.e. a list
with the following components:
\item{n}{The complete case sample size.}
\item{n_miss}{The number of individuals removed due to missing data.}
\item{n_event}{The number of observed events in the score function.}
\item{coef}{A four-column matrix containing the estimated coefficients,
standard errors, z-statistics, and p-values. Coefficient values of \code{NA}
indicate collinearity.}
\item{var_matrix}{The estimated variance matrix for the coefficient estimates.
Values of \code{NA} indicate collinearity.}
\item{robust_se}{The input value for \code{robust_se}.}
\item{offset}{The offsets from the model formula for complete case observations.}
\item{ipw}{The estimated inverse probability of non-truncation for each
complete case observation.}
\item{time_wt}{The time-varying component of the inverse probability weights
for each complete case observation.}
\item{cc_indx}{A logical index vector for the complete cases in the original
data.}
\item{id_indx}{A vector of ids matching each observation with a subject among
the complete cases.}
\item{sort_indx}{A numerical vector to sort the complete cases into the order
they are used in \code{cox_fit} (see below).}
\item{s}{The stratum value for each complete case observation if a stratified
Cox model was fit.
Otherwise \code{NULL}.}
\item{martingale_resid}{The weighted martingale residuals for the complete
cases, with one entry per observation.}
\item{deviance_resid}{The weighted deviance residuals for the complete cases,
with one entry per subject.}
\item{basehaz}{For input \code{basehaz}=\code{TRUE}, a list with first element \code{s}
containing the stratum values and one additional element for each stratum,
in the order of \code{s}.
Each contains a matrix with following columns:
\code{time}, \code{cumhaz} with the cumulative baseline hazard value (baseline at x=0),
\code{var_logcumhaz} with the estimated variance of the log(cumhaz), and
additional columns, labeled by predictor names, with the estimated covariance
between the log(cumhaz) and the regression coefficients.
For input \code{basehaz}=\code{FALSE}, \code{NULL}.}
\item{timerange}{A vector with the minimum and maximum times used in the
score function.}
\item{eventrange}{A vector with the unrestricted range of observed event times.}
\item{wt_name}{A string giving the name of the weights used in \code{ipw_type}.}
\item{cox_fit}{The \code{coxph} object used to obtain the estimates.
Note that the data was sorted by strata and time before fitting the model,
and the offset may include an additional term of \code{-log(time_wt)}.}
\item{strata_summary}{Some summary statistics for each stratum, computed
prior to applying \code{restr_times}.}
\item{call_}{The call to \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}.}

\code{coxph_indtrunc} objects have methods for \code{vcov} to extract the variance matrix
of estimated coefficients, \code{coef} to extract the estimated coefficients,
\code{confint} to compute confidence intervals for coefficients, \code{residuals} for
model diagnostics, and \code{print} to display a summary of the fitted model.

\code{residuals} returns per-subject residuals in the order that the subject first
appeared in the original data.
The deviance residuals are computed as
\eqn{\text{sign}(\widehat{M}_{wi})\sqrt{
-2[\widehat{M}_{wi} +
\delta_{wi}\log(1 - \widehat{M}_{wi}/\delta_{wi})]}}
where \eqn{\widehat{M}_{wi}} is the weighted martingale residual and
\eqn{\delta_{wi}} is the weighted event indicator, following
Therneau et al. (1990).
}
\description{
Computes an inverse probability weighted Cox regression estimator using
nonparametric weights (see \code{\link[=npmle]{npmle()}}).
Includes options for robust estimators with time-varying weights, which
introduce an event time stabilization weight (see below).
}
\details{
Given a sample of doubly truncated data with event times \eqn{T_i}, truncation
times \eqn{L_i} and \eqn{R_i}, and potentially time-varying covariates
\eqn{X_i(t)}, let \eqn{N_i(t)=1(T_i \leq t)} be the usual counting process
and \eqn{Y_i(t)=1(T_i \geq t)} be the at-risk process.

\code{\link[=coxph_indtrunc]{coxph_indtrunc()}} fits a Cox model by maximizing an inverse probability
weighted partial likelihood, where the double truncation bias is accounted
for through nonparametric estimates of
\eqn{a(t)^{-1}=P(L<T \leq R)/P(L<T \leq R|T=t)} (see \code{\link[=npmle]{npmle()}}).
Assuming \code{strata_formula=NULL} for sake of illustration, setting
\code{ipw_type="W-1"} specifies non-time-varying weights, so the model is fitted by
finding the root of the weighted score function
\deqn{\sum_{i}\int \frac{1}{\widehat{a}(T_i)}
      [X_i(t) - \overline{X}_{\widehat{a}}(t,\beta)]dN_i(t)}
\deqn{\text{where } \overline{X}_{\widehat{a}}(t,\beta)
      = \frac{\sum_j X_j(t)\widehat{a}(T_j)^{-1}Y_j(t)\exp(X_j(t)^T\beta)}
             {\sum_j \widehat{a}(T_j)^{-1}Y_j(t)\exp(X_j(t)^T\beta)}.}

More generally, \code{\link[=coxph_indtrunc]{coxph_indtrunc()}} allows for time-varying weights with a
stabilization component \eqn{\widehat{w}(t)} that determines the relative
importance of each event time.
This can improve robustness and numerical convergence without introducing any
bias, in contrast to weight truncation, for example.
The weighted score function becomes
\deqn{\sum_{i}\int \frac{\widehat{w}(t)}{\widehat{a}(T_i)}
      [X_i(t) - \overline{X}_{\widehat{a}}(t,\beta)]dN_i(t).}
Several choices of time-varying weights are available:
\itemize{
\item \code{ipw_type="W-surv"}: \eqn{\widehat{w}(t)=\widehat{S}(t-)}, where
\eqn{\widehat{S}(t)} is the marginal survival function estimated by
\code{\link[=npmle]{npmle()}}.
Can increase robustness to outliers.
\item \code{ipw_type="W-a"}: \eqn{\widehat{w}(t)=\widehat{a}(t)}.
Can reduce the influence of extreme inverse probability weights.
\item \code{ipw_type="W-asurv"}: \eqn{\widehat{w}(t)=\widehat{a}(t)\widehat{S}(t-)}.
Combines the benefits of both weights.
\item Custom stabilization weights \eqn{\widehat{w}(t)} can be specified by
supplying a function as the input to \code{ipw_type}.
If \code{robust_se=TRUE}, this also requires the influence values for
\eqn{\widehat{w}(t)} to be supplied in \code{robust_infl}, in order to account
for its variability in the standard errors.
}
}
\section{Stratified Cox model}{


A stratified Cox model is specified through \code{strata_formula}.
The formula should look like \code{~svar1 + svar2}... where \code{svar1} and \code{svar2} are
factor variables defining the strata.
This allows the baseline hazard to differ across strata.
}

\section{Restricted time window in score function}{

The user can choose to restrict the time range of the score function during
estimation.
This may be useful when extreme event times have very large weights.
Setting the input argument \code{restr_times=c(t1,t2)} will restrict the range of
the integrals in the score function to the interval \eqn{[}\code{t1}, \code{t2}\eqn{]}
instead of the entire event time range.
Notably, this time restriction will not introduce any bias in the fitted model.
}

\examples{
coxph_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
               data = aids)

}
\references{
Mandel, M., de Uña-Álvarez, J., Simon, D. K., & Betensky, R. A. (2018).
Inverse probability weighted Cox regression for doubly truncated data.
\emph{Biometrics}, \strong{74}(2), 481--487.

Rennert, L., & Xie, S. X. (2018). Cox regression model with doubly truncated
data. \emph{Biometrics}, \strong{74}(2), 725--733.

Therneau, T. M., Grambsch, P. M., & Fleming, T. R. (1990). Martingale-based
residuals for survival models. \emph{Biometrika}, \strong{77}(1), 147--160.

Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
estimators for doubly truncated Cox regression with closed-form standard
errors. \emph{Lifetime Data Analysis}, \strong{31}(2), 364–-393.
}
\seealso{
\itemize{
\item \code{\link[=plot_coxsurv]{plot_coxsurv()}}: compute/plot estimates and pointwise confidence intervals
for the conditional survival function or cumulative hazard from a fitted Cox
model.
\item \code{\link[=positivity_sens_indtrunc]{positivity_sens_indtrunc()}}: sensitivity analysis for positivity violations
in the Cox model.
\item \code{\link[=test_quasiindep_covariates]{test_quasiindep_covariates()}}: test for dependence between truncation times
and the event time or covariates.
The nonparametric weights used by \code{\link[=coxph_indtrunc]{coxph_indtrunc()}} requires quasi-independence.
\item \code{vignette("cox-regression-quasiindep", package = "survdt")}: an
illustration of Cox regression and related diagnostics applied to the
\link{aids} data, if vignettes were built when installing \link{survdt}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survdt-package.R
\docType{package}
\name{survdt-package}
\alias{survdt}
\alias{survdt-package}
\title{survdt: Improved Methods for Survival Analysis under Double Truncation}
\description{
Contains existing and novel methods for nonparametric analysis and Cox regression analysis with doubly truncated survival data, as described in Vazquez and Xie (2025) \doi{10.1007/s10985-025-09650-5}. Includes survival curves and hazard estimates through nonparametric maximum likelihood estimation, various tests for detecting group differences or non-ignorable sampling bias, and inverse probability weighted Cox regression with several options of nonparametric weights. Also implements diagnostics for key modeling assumptions such as quasi-independent truncation and the positivity assumption. Closed-form standard errors are available for all estimates, i.e. bootstrapping is not required.
}
\section{Datasets}{

\itemize{
\item \link{aids}: a doubly truncated sample of AIDS incubation times from patients
with transfusion-acquired HIV.
\item \link{nonprop_sample}: a simulated sample with non-proportional hazards and
quasi-independent double truncation.
}
}

\section{Key functions}{


\strong{Survival objects to hold doubly truncated data}
\itemize{
\item \code{\link[=Survdt]{Survdt()}}: one observation per subject.
\item \code{\link[=Survdt2]{Survdt2()}}: for (start,stop] data with multiple observations per subject.
\item \code{\link[=time_split]{time_split()}}, \code{\link[=survdt2to1]{survdt2to1()}}: convert between the two formats above.
}

\strong{Nonparametric analysis under double truncation}
\itemize{
\item \code{\link[=npmle]{npmle()}}: nonparametric maximum likelihood estimation.
The output has a \code{plot} method to plot the estimated survival function,
cumulative hazard, and selection probabilities along with pointwise
confidence intervals.
The nonparametric maximum likelihood estimator (NPMLE) is used in the two
tests that follow.
\item \code{\link[=test_ignorability_indtrunc]{test_ignorability_indtrunc()}}: test for non-ignorable sampling bias.
\item \code{\link[=test_samplediff_indtrunc]{test_samplediff_indtrunc()}}: test for any difference in event time
distribution across groups.
\item \code{\link[=test_quasiindep_ctau]{test_quasiindep_ctau()}}: test for dependence between truncation times and
the event time.
The NPMLE requires quasi-independence.
}

\strong{Cox regression under double truncation}
\itemize{
\item \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}: inverse probability weighted Cox regression with options
for robust time-varying weights estimated nonparametrically.
The output has common methods such as \code{confint}, \code{residuals}, and \code{vcov}.
\item \code{\link[=plot_coxsurv]{plot_coxsurv()}}: plot conditional survival functions or cumulative hazards
from a fitted Cox model along with pointwise confidence intervals.
\item \code{\link[=test_quasiindep_covariates]{test_quasiindep_covariates()}}: test for dependence between truncation times
and the event time or covariates.
The nonparametric inverse probability weighting used by \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}
requires quasi-independence.
\item \code{\link[=positivity_sens_indtrunc]{positivity_sens_indtrunc()}}: sensitivity analysis for positivity violations.
Check how the estimates and confidence intervals may change under a range
of values for the baseline probability mass potentially lost due to zero
sampling probability.
}
}

\section{Vignettes}{

If vignettes were built when installing \link{survdt}, see
\itemize{
\item \code{vignette("nonparametric-analysis", package = "survdt")}: an example of
various nonparametric analyses applied to the \link{aids} data.
\item \code{vignette("cox-regression-quasiindep", package = "survdt")}: an example of
Cox regression and its assumption diagnostics applied to the \link{aids} data.
}
}

\references{
Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
estimators for doubly truncated Cox regression with closed-form standard
errors. \emph{Lifetime Data Analysis}, \strong{31}(2), 364–-393.
}
\author{
\strong{Maintainer}: Omar Vazquez \email{omar.vazquez@pennmedicine.upenn.edu} (\href{https://orcid.org/0009-0009-4137-3333}{ORCID}) [copyright holder]

Other contributors:
\itemize{
  \item Sharon X. Xie (\href{https://orcid.org/0000-0002-7886-4103}{ORCID}) [thesis advisor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Survdt.R
\name{time_split}
\alias{time_split}
\title{Split rows in a data frame into multiple observations at specified time
intervals}
\usage{
time_split(
  data,
  cut,
  stop_name,
  start_name = "start",
  event_name = "event",
  id_name = NULL,
  origin = 0
)
}
\arguments{
\item{data}{A data frame to be split.}

\item{cut}{A numeric vector of cut times.}

\item{stop_name}{The name of the existing event time or stop time column in
\code{data}.}

\item{start_name}{The name for the start time column to be created in the split
data. If it matches a column in \code{data} (meaning the data was previously split)
then \code{event_name} must also match a column in \code{data}.}

\item{event_name}{The name for the event indicator column to be created in the
split data. See \code{start_name} for special requirements.}

\item{id_name}{An optional name to create an id column in the split data. If
this column already exists in \code{data} its values will be used as ids.
If the data is already in (start, stop] format, do not use this argument.}

\item{origin}{The time that the follow up starts at. Defaults to zero.}
}
\value{
A split data frame with (start, stop] records.
The new column names are taken from the input arguments of \code{time_split()}.
}
\description{
Splits each record in the data frame into a series of observation windows
based on the supplied cut times.
Can be used to reshape data with time-varying covariates to the appropriate
format for \code{\link[=Survdt2]{Survdt2()}}.
The main workhorse is a modified call to \code{\link[survival:survSplit]{survival::survSplit()}}.
}
\details{
A basic use case of \code{\link[=time_split]{time_split()}} is to take a data frame with one row per
subject and transform it into a data frame with multiple rows per subject,
each corresponding to a specific observation time window.
The event time information observed in each window is encoded by four
potentially new variables:
\itemize{
\item \code{start}: the starting time for the observation window.
\item \code{stop}: the end time for the observation window.
\item \code{event}: equal to one when the subject's event occurred at \code{stop}, and zero
otherwise.
\item \code{id}: the label that identifies which subject the data corresponds to.
}

In this use case, the name of the event time column in \code{data} should always be
provided, as well as the name for the new \code{id} column if one does not already
exist.
After \code{cut} is sorted, the \eqn{i}th subject's observation windows will be
\eqn{(}\code{origin}, \code{cut[1]}\eqn{]}, \eqn{(}\code{cut[1]}, \code{cut[2]}\eqn{]}, \eqn{\ldots},
\eqn{(}\code{cut[j]}, \eqn{T_i}\eqn{]} for some
\code{j}\eqn{\leq}\code{length(cut)}.

Another use case of \code{\link[=time_split]{time_split()}} is to introduce additional splits in a
previously split data frame.
The names for the existing \code{start}, \code{stop}, and \code{event} columns should be
provided to \code{\link[=time_split]{time_split()}} in this case.
Requesting a new \code{id} variable for previously cut data will return an error,
since the new ids would not respect the prior groupings.
}
\examples{
splitdat1 <- time_split(aids, cut = c(20, 40),
                        stop_name = "incu",
                        id_name = "id",
                        origin = -1)
tail(splitdat1)

splitdat2 <- time_split(splitdat1, cut = c(10, 30),
                        start_name = "start",
                        stop_name = "incu",
                        event_name = "event",
                        id_name = "id",
                        origin = -1)
tail(splitdat2)
}
\seealso{
\itemize{
\item \code{\link[=Survdt2]{Survdt2()}}: create survival objects with split data.
\item \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}: Cox regression with time-varying covariates.
\item \code{\link[=survdt2to1]{survdt2to1()}}: collapse data back to one row per subject.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npests.R
\name{test_samplediff_indtrunc}
\alias{test_samplediff_indtrunc}
\alias{plot.samplediff_indtrunc_test}
\alias{print.samplediff_indtrunc_test}
\title{Test for survival differences across groups}
\usage{
test_samplediff_indtrunc(
  formula,
  data = NULL,
  test_type = c("survival", "cumhaz"),
  level = 0.95,
  B = 2000,
  restr_times = NULL,
  wt_fn = NULL,
  max_iter = NULL,
  simple_labels = FALSE
)

\method{plot}{samplediff_indtrunc_test}(x, level, ...)

\method{print}{samplediff_indtrunc_test}(x, ...)
}
\arguments{
\item{formula}{A formula with a \code{Survdt} response and stratification variables
on the right hand side.
Note that subtraction will not remove stratification variables here.
Subjects with missing values are discarded.
See \code{\link[=Survdt]{Survdt()}}.}

\item{data}{An optional data frame used to evaluate \code{formula}.}

\item{test_type}{The test statistic to use: \code{"survival"} (default) or
\code{"cumhaz"}.
See below.}

\item{level}{The confidence level for the test (default 0.95).}

\item{B}{The number of simulations from the estimated null distribution used
to compute the p-value (default 2000).}

\item{restr_times}{An optional numerical vector of length two giving a time
window to restrict the test statistic.
Values of \code{NA} are replaced with the minimum and maximum observed event
time respectively, i.e. no restriction.
The default setting is no restriction.}

\item{wt_fn}{An optional function that takes a numerical vector of times as
input and returns a vector of weights to apply in the test statistic.
If unspecified, all times have equal weight of 1.}

\item{max_iter}{The maximum number of iterations to use within each call to
\code{\link[=npmle]{npmle()}}. If missing then the default setting of \code{\link[=npmle]{npmle()}} is used.}

\item{simple_labels}{Whether to drop the factor names from the stratum labels
(Default \code{FALSE}).}

\item{x}{Any R object.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
\code{\link[=test_samplediff_indtrunc]{test_samplediff_indtrunc()}} returns an object of class
\code{samplediff_indtrunc_test}, i.e. a list with the following components:
\item{n}{The complete case sample size.}
\item{n_unique}{The number of unique event times among complete cases.}
\item{n_miss}{The number of discarded observations (with missing values).}
\item{teststat}{The value of the observed test statistic.}
\item{pval}{The p-value for the test.}
\item{critval}{The critical value for the test at the user-specified level.}
\item{testresult}{Whether the test results in rejecting the null
hypothesis of no differences across strata.}
\item{sims}{A vector of the \code{B} simulations from the estimated null
distribution of the test statistic.}
\item{test_type}{The matched \code{test_type} value.}
\item{level}{The user-supplied \code{level}.}
\item{B}{The number of simulations specified by the user.}
\item{timerange}{A vector with the minimum and maximum times used in the test.}
\item{wt_name}{A string giving the name of the user-supplied \code{wt_fn}, if any.}
\item{eventrange}{A vector with the unrestricted range of observed event times.}
\item{strata_ests}{A data frame of estimates to be used by
\code{\link[=plot.samplediff_indtrunc_test]{plot.samplediff_indtrunc_test()}}.}
\item{strata_summary}{Some summary statistics for each stratum.}

The \code{print} method for \code{samplediff_indtrunc_test} objects displays a summary
of the test results, while the \code{plot} method returns a visualization comparing
the stratified and unstratified NPMLE using a uniform confidence band,
generated through \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
Tests for any difference in event time distribution by comparing
the stratified and unstratified estimates obtained using the NPMLE.
}
\details{
Let \eqn{\widehat{S}(t|s)} be the NPMLE for the survival function in stratum
\eqn{s}.
By default, \code{\link[=test_samplediff_indtrunc]{test_samplediff_indtrunc()}} tests for event time distribution
differences through the test statistic
\deqn{\max_s\sup_{\min_i T_i \leq t \leq \max_i T_i} |\widehat{S}(t|s) - \widehat{S}(t)|}
where \eqn{\widehat{S}(t)} is the unstratified NPMLE.
Another option is to compare the stratified cumulative hazards
\eqn{\widehat{\Lambda}(t|s)} with the unstratified NPMLE \eqn{\widehat{\Lambda}(t)}
through the test statistic
\deqn{\max_s\sup_{\min_i T_i \leq t \leq \max_i T_i}
      |\widehat{\Lambda}(t|s) - \widehat{\Lambda}(t)|.}
The p-value in all cases is computed by simulating from the estimated asymptotic
null distribution.
}
\section{Restricted time window and weighted tests}{

When the NPMLE has high variance at extreme event times, the test may suffer
from reduced power.
One possible remedy is to restrict the time range of the test to some interval
\eqn{[\tau_1, \tau_2]} contained in \eqn{[\min_i T_i, \max_i T_i]}.
Another option is to down-weight event times with large variance through a
weight function \eqn{w(t)}.
The stabilized test statistic will have the form
\deqn{\sup_{\tau_1 \leq t \leq \tau_2} |w(t)(\widehat{h}(t) - h(t))|}
where \eqn{\widehat{h}(t)} is the NPMLE without assuming the null hypothesis
holds and \eqn{h(t)} is the reference value under the null.
See \code{vignette("nonparametric-analysis", package = "survdt")} for an illustration
with the \link{aids} data assuming vignettes were built when installing \link{survdt}.
}

\examples{
difftest <- test_samplediff_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
                                     data = aids,
                                     restr_times = c(NA, 70))
difftest
plot(difftest)

}
\references{
Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
estimators for doubly truncated Cox regression with closed-form standard
errors. \emph{Lifetime Data Analysis}, \strong{31}(2), 364–-393.
}
\seealso{
\itemize{
\item \code{\link[=npmle]{npmle()}}: the nonparametric maximum likelihood estimator which forms the
basis for these tests.
\item \code{vignette("nonparametric-analysis", package = "survdt")}: an illustration
of group tests applied to the \link{aids} data, if vignettes were built when
installing \link{survdt}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Survdt2.R
\name{Survdt2}
\alias{Survdt2}
\alias{survdt2to1}
\alias{[.Survdt2}
\alias{is.na.Survdt2}
\alias{is.Survdt2}
\alias{length.Survdt2}
\alias{as.data.frame.Survdt2}
\alias{plot.Survdt2}
\title{Create a survival object for use with time-varying covariates}
\usage{
Survdt2(start, stop, event, ltrunc, rtrunc, id)

survdt2to1(x)

\method{[}{Survdt2}(x, i, j, ...)

\method{is.na}{Survdt2}(x)

is.Survdt2(x)

\method{length}{Survdt2}(x)

\method{as.data.frame}{Survdt2}(x, ...)

\method{plot}{Survdt2}(x, ...)
}
\arguments{
\item{start}{A vector of start times for the observation windows.}

\item{stop}{A vector of stop times for the observation windows.}

\item{event}{A vector of 0/1 event indicators (1=event).}

\item{ltrunc}{A vector of left truncation times.}

\item{rtrunc}{A vector of right truncation times.}

\item{id}{A vector of subject ids.}

\item{x}{Any R object.}

\item{i, j}{Row and columns index vectors. The result is a \code{Survdt2} object if
\code{j} is missing, otherwise \code{x} is treated as a matrix.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
\itemize{
\item \code{\link[=Survdt2]{Survdt2()}} returns an object of class \code{Survdt2}, with columns \code{start},
\code{stop}, \code{event}, \code{ltrunc}, \code{rtrunc}, and \code{id}.
Indexing a \code{Survdt2} object by rows will return another \code{Survdt2} object,
otherwise it is treated as a matrix.
The length of a \code{Survdt2} object is the total number of observations across
all subjects, and \code{is.na()} returns a logical index vector for the non-complete
rows.
Calling \code{as.data.frame} on a \code{Survdt2} object will return a data frame
stripped of the \code{Survdt2} class attribute.
\item \code{\link[=is.Survdt2]{is.Survdt2()}} returns \code{TRUE} if \code{x} inherits from the class \code{Survdt2} and
\code{FALSE} otherwise.
\item \code{\link[=survdt2to1]{survdt2to1()}} takes a \code{Survdt2} object as input and returns the result from
calling \code{\link[=Survdt]{Survdt()}} on collapsed data with one row per \code{id}.
The subjects in the collapsed data are sorted in the order they first appear
in the \code{Survdt2} object.
If all event indicators for a subject are zero and/or \code{NA}, the event time
in the collapsed data is set to \code{NA}.
Lastly, the \code{id} values are preserved within the row names of the \code{Survdt}
object.
\item The plot method for \code{Survdt2} first collapses the data into \code{Survdt}
format using \code{\link[=survdt2to1]{survdt2to1()}} and then calls \code{\link[=plot.Survdt]{plot.Survdt()}}.
}
}
\description{
Creates a survival object for doubly truncated data with the follow
up time for each subject split across multiple observation windows using
(start, stop] encoding.
}
\details{
The \eqn{i}th subject's survival data consists of the left and right truncation
times \eqn{L_i} and \eqn{R_i} as well as a series of (start, stop] observation
windows \eqn{(t_{i0}}, \eqn{t_{i1}]}, \eqn{(t_{i1}}, \eqn{t_{i2}]},
\eqn{\ldots}, \eqn{(t_{i,j-1}}, \eqn{t_{ij}}\eqn{]} for some \eqn{j}.
Furthermore, each of these observation windows has an associated event indicator
that is equal to one if the event time is equal to the stop time, and zero
otherwise.
Each subject should have an observation window with an event, and multiple
events per subject are not allowed.
Attempting to run \code{\link[=Survdt2]{Survdt2()}} with inputs that do not follow this specification
will result in an error.

This data structure is used for Cox regression with time-varying covariates,
since the subject's covariate values can vary across observation windows.
}
\examples{
# first create (start, stop] data frame;
# could then apply time transformations
# to covariates
splitdat <- time_split(aids, cut = c(20, 40),
                       stop_name = "incu",
                       id_name = "id",
                       origin = -1)

y1 <- with(splitdat,
           Survdt2(start, incu, event, ltrunc, rtrunc, id))
plot(y1)

y2 <- with(aids,
           Survdt(incu, ltrunc, rtrunc))
max(abs(unclass(survdt2to1(y1)) - unclass(y2)))


coxph_indtrunc(Survdt2(start, incu, event, ltrunc, rtrunc, id) ~ age_gp,
               data = splitdat)

}
\seealso{
\itemize{
\item \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}: inverse probability weighted Cox regression with
time-varying covariates.
\item \code{\link[=Survdt]{Survdt()}}: the more basic survival object with one row per subject.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Survdt.R
\name{Survdt}
\alias{Survdt}
\alias{[.Survdt}
\alias{is.na.Survdt}
\alias{is.Survdt}
\alias{length.Survdt}
\alias{as.data.frame.Survdt}
\alias{plot.Survdt}
\title{Create a basic survival object for doubly truncated data.}
\usage{
Survdt(time, ltrunc, rtrunc)

\method{[}{Survdt}(x, i, j, ...)

\method{is.na}{Survdt}(x)

is.Survdt(x)

\method{length}{Survdt}(x)

\method{as.data.frame}{Survdt}(x, ...)

\method{plot}{Survdt}(x, colorvar = NULL, sort = TRUE, ...)
}
\arguments{
\item{time}{A vector of follow up times.}

\item{ltrunc}{A vector of left truncation times.}

\item{rtrunc}{A vector of right truncation times.}

\item{x}{Any R object.}

\item{i, j}{Row and columns index vectors. The result is a \code{Survdt} object if
\code{j} is missing, otherwise \code{x} is treated as a matrix.}

\item{...}{Additional arguments passed to other methods.}

\item{colorvar}{An optional vector used to color the dots in the plot. Will
be coerced to factor before using.}

\item{sort}{Whether to sort the data rows by follow up time (and \code{colorvar},
if provided) before plotting (default \code{TRUE}).}
}
\value{
\itemize{
\item \code{\link[=Survdt]{Survdt()}} returns an object of class \code{Survdt}, with columns \code{time},
\code{ltrunc}, and \code{rtrunc}.
Indexing a \code{Survdt} object by rows will return another \code{Survdt} object,
otherwise it is treated as a matrix.
The length of a \code{Survdt} object is its sample size, and \code{is.na()} returns
a logical index vector for the non-complete cases.
Calling \code{as.data.frame()} on a \code{Survdt} object will return a data frame
stripped of the \code{Survdt} class attribute.
\item \code{\link[=is.Survdt]{is.Survdt()}} returns \code{TRUE} if \code{x} inherits from the class \code{Survdt} and
\code{FALSE} otherwise.
\item The plot method for \code{Survdt} objects returns a simple visualization from
\code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} with x-axis time, y-axis subject, rectangles for the
truncation intervals, and dots for the event times.
The dots can optionally be colored by a factor variable supplied in
\code{colorvar}.
Incomplete cases are dropped from the plot (with a warning).
}
}
\description{
Creates a survival object to contain the event times, left truncation times,
and right truncation times from a doubly truncated sample, with one observation
per subject.

Typically used as the input for nonparametric analysis methods or as the
response variable in a regression model formula.
}
\details{
For a given event time of interest \eqn{T}, a sample is doubly truncated when
it only includes subjects with \eqn{L_i<T_i \leq R_i} for some random
truncation times \eqn{L_i} and \eqn{R_i}.
Analyzing this type of data requires information on the \eqn{i}th subject's
event time \eqn{T_i}, left truncation time \eqn{L_i}, and right truncation
time \eqn{R_i}.

\code{\link[=Survdt]{Survdt()}} provides the basic data structure for organizing such data and
checking its validity in the \code{survdt} package.
Attempting to run \code{\link[=Survdt]{Survdt()}} with inputs that do not satisfy \eqn{L<T \leq R}
will result in an error.
}
\examples{
y <- with(aids, Survdt(incu, ltrunc, rtrunc))
plot(y)
plot(y, colorvar = aids$age_gp)
npmle(y)

}
\seealso{
\itemize{
\item \code{\link[=npmle]{npmle()}}, \code{\link[=test_samplediff_indtrunc]{test_samplediff_indtrunc()}}: nonparametric estimation and
analysis for doubly truncated data.
\item \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}: inverse probability weighted Cox regression.
\item \code{\link[=Survdt2]{Survdt2()}}: to encode time-varying covariates; also \code{\link[=time_split]{time_split()}} for
converting data from \code{Survdt} structure to \code{Survdt2}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npests.R
\name{test_quasiindep_covariates}
\alias{test_quasiindep_covariates}
\alias{plot.quasiindep_strat_test}
\alias{print.quasiindep_strat_test}
\title{Test for dependence between truncation times and the event time or covariates}
\usage{
test_quasiindep_covariates(
  formula,
  data = NULL,
  level = 0.95,
  B = 2000,
  restr_times = NULL,
  wt_fn = NULL,
  max_iter = NULL,
  simple_labels = FALSE
)

\method{plot}{quasiindep_strat_test}(x, level, ...)

\method{print}{quasiindep_strat_test}(x, ...)
}
\arguments{
\item{formula}{A formula with a \code{Survdt} response and stratification variables
on the right hand side.
Note that subtraction will not remove stratification variables here.
Subjects with missing values are discarded.
See \code{\link[=Survdt]{Survdt()}}.}

\item{data}{An optional data frame used to evaluate \code{formula}.}

\item{level}{The confidence level for the test (default 0.95).}

\item{B}{The number of simulations from the estimated null distribution used
to compute the p-value (default 2000).}

\item{restr_times}{An optional numerical vector of length two giving a time
window to restrict the test statistic.
Values of \code{NA} are replaced with the minimum and maximum observed event
time respectively, i.e. no restriction.
The default setting is no restriction.}

\item{wt_fn}{An optional function that takes a numerical vector of times as
input and returns a vector of weights to apply in the test statistic.
If unspecified, all times have equal weight of 1.}

\item{max_iter}{The maximum number of iterations to use within each call to
\code{\link[=npmle]{npmle()}}. If missing then the default setting of \code{\link[=npmle]{npmle()}} is used.}

\item{simple_labels}{Whether to drop the factor names from the stratum labels
(Default \code{FALSE}).}

\item{x}{Any R object.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
An object of class \code{quasiindep_strat_test}, i.e. a list with the following
components:
\item{n}{The complete case sample size.}
\item{n_unique}{The number of unique event times among complete cases.}
\item{n_miss}{The number of discarded observations (with missing values).}
\item{teststat}{The value of the observed test statistic for the across-strata
test (stage 2).}
\item{pval}{The p-value for the across-strata test.}
\item{critval}{The critical value for the across-strata test at the
user-specified level.}
\item{testresult}{Whether the across-strata test results in rejecting the null
hypothesis of quasi-independence.}
\item{sims}{A vector of the \code{B} simulations from the estimated null distribution
of the across-strata test statistic.}
\item{level}{The user-supplied \code{level}.}
\item{B}{The number of simulations specified by the user.}
\item{timerange}{A vector with the minimum and maximum times used in the
across-strata test.}
\item{wt_name}{A string giving the name of the user-suppled \code{wt_fn}, if any.}
\item{eventrange}{A vector with the unrestricted range of observed event times.}
\item{strata_ests}{A data frame of estimates to be used by
\code{\link[=plot.quasiindep_strat_test]{plot.quasiindep_strat_test()}}.}
\item{strata_ctau}{A data frame of the within-strata test results (stage 1).
Includes stratum-specific tests by conditional Kendall's tau as well as the
aggregate p-value for testing within-strata quasi-independence in all strata
simultaneously.}

The \code{print} method for \code{quasiindep_strat_test} objects displays a summary of the
test results, while the \code{plot} method returns a visualization comparing the
stratified and unstratified NPMLE from stage 2 using a uniform confidence band,
generated through \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
Implements a two-stage test for dependence between the truncation times and
the event time and covariates that first tests within covariate strata and then
across strata.

Typically used to check the underlying assumptions for inverse probability
weighted regression with NPMLE weights, e.g. as used in \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}.
}
\details{
Implements a nonparametric test for dependent truncation to check if the
selection probability depends on either the event times or covariates.
The test has two stages:
\enumerate{
\item Test for dependence between event and truncation times stratified by
covariate groups using conditional Kendall's tau
(see \code{\link[=test_quasiindep_ctau]{test_quasiindep_ctau()}}).
The aggregate p-value for this stage is computed from a chi-squared
distribution.
\item Test whether the selection probability depends on covariates by comparing
the stratified and unstratified selection probabilities estimated using the
NPMLE.
Let \eqn{\widehat{\pi}(t|s)} be the estimate in stratum \eqn{s}, and
\eqn{\widehat{\pi}(t)} be the unstratified estimate.
Both are left-continuous step functions with jumps at observed truncation
times.
By default, \code{\link[=test_quasiindep_covariates]{test_quasiindep_covariates()}} uses the test statistic
\deqn{\max_s\sup_{\min_i T_i\leq t \leq \max_i T_i}
         |\widehat{\pi}(t|s)-\widehat{\pi}(t)|}
to compare selection probabilities across strata.
The p-value for this stage is computed by simulating from the estimated
asymptotic null distribution.
}
}
\section{Restricted time window and weighted tests}{

When the NPMLE has high variance at extreme event times, the test may suffer
from reduced power.
One possible remedy is to restrict the time range of the test to some interval
\eqn{[\tau_1, \tau_2]} contained in \eqn{[\min_i T_i, \max_i T_i]}.
Another option is to down-weight event times with large variance through a
weight function \eqn{w(t)}.
The stabilized test statistic will have the form
\deqn{\sup_{\tau_1 \leq t \leq \tau_2} |w(t)(\widehat{h}(t) - h(t))|}
where \eqn{\widehat{h}(t)} is the NPMLE without assuming the null hypothesis
holds and \eqn{h(t)} is the reference value under the null.
See \code{vignette("nonparametric-analysis", package = "survdt")} for an illustration
with the \link{aids} data assuming vignettes were built when installing \link{survdt}.
}

\section{No effective truncation}{

A doubly truncated sample has no effective truncation when
\eqn{\max_i L_i < \min_i T_i} and \eqn{\max_i T_i \leq \min_i R_i}.
The estimated selection probability is equal to one across all event times
in this case, and its standard errors may not be reliable.
Typically one would conclude that the truncation mechanism is ignorable in
these types of samples, so standard methods can be applied for data analysis.
}

\examples{
indtest <- test_quasiindep_covariates(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
                                      data = aids)
indtest
plot(indtest)

}
\references{
Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted estimators
for doubly truncated Cox regression with closed-form standard errors.
\emph{Lifetime Data Analysis}, \strong{31}(2), 364–-393.
}
\seealso{
\itemize{
\item \code{\link[=test_quasiindep_ctau]{test_quasiindep_ctau()}}: the conditional Kendall's tau test for quasi-dependence
without covariates, which is the basis for stage 1.
\item \code{\link[=npmle]{npmle()}}: the nonparametric maximum likelihood estimator which is the basis
for stage 2.
\item \code{vignette("cox-regression-quasiindep", package = "survdt")}: an illustration
of this quasi-independence test applied to the \link{aids} data, if vignettes were
built when installing \link{survdt}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipw_ests.R
\name{plot_coxsurv}
\alias{plot_coxsurv}
\title{Plot survival or hazard estimates from a Cox model fit}
\usage{
plot_coxsurv(
  object,
  newdata,
  ci = TRUE,
  level = 0.95,
  ctype = c("log-log", "log", "plain"),
  target = c("survival", "cumhaz")
)
}
\arguments{
\item{object}{A \code{coxph_indtrunc} object. See \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}.}

\item{newdata}{A data frame with the covariate values for the plot.
The original model formula is applied to extract predictors.
Rows that are duplicated or have missing values are discarded.}

\item{ci}{Whether to plot pointwise confidence intervals (default \code{TRUE}).
Will re-fit the model if \code{basehaz} or its standard errors were not computed.}

\item{level}{Confidence level for the pointwise confidence intervals
(default 0.95).}

\item{ctype}{Type of confidence interval.
\itemize{
\item \code{"plain"}: linear confidence from a normal approximation.
\item \code{"log"}: back-transformed confidence intervals from the natural log scale.
Guaranteed to be non-negative.
\item \code{"log-log"} (the default): for the survival function, back-transformed
confidence intervals from the (natural) log minus log scale, which are
guaranteed to be between zero and one.
For the cumulative hazard, the same confidence intervals as \code{"log"} are
used.
}}

\item{target}{Which quantity to plot.
\itemize{
\item \code{"survival"} (the default): the conditional survival function.
\item \code{"cumhaz"}: the conditional cumulative hazard.
}}
}
\value{
After displaying the plot, a list with two elements is returned invisibly.
The first element \code{ests} is a data frame with the columns \code{time}, \code{est},
\code{lower}, and \code{upper}, containing the point estimates and their computed
confidence intervals.
The second element \code{plot} is the plotted object returned by \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
Lines are colored based on the covariate value.
}
\description{
Given a fitted Cox model from \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}, plot the conditional survival
or cumulative hazard estimates along with pointwise confidence intervals.
The baseline hazard and its standard errors are computed by \code{\link[=plot_coxsurv]{plot_coxsurv()}}
if they were not included in the \code{coxph_indtrunc} fit.
}
\details{
In order to construct the linear predictors at the requested covariate values,
the design matrix obtained from evaluating the original model formula on
\code{newdata} should have the same column names as the design matrix for the
fitted model.
Issues can occur for categorical variables that were converted to factors by
\code{\link[=coxph_indtrunc]{coxph_indtrunc()}} for estimation.
If an error occurs, check that each categorical variable in \code{newdata} is a
factor variable that is not missing any levels used to fit the model.
}
\examples{
fit <- coxph_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
                      data = aids,
                      basehaz = TRUE)
plot_coxsurv(fit, newdata = aids[c(1,40), ])

plot_coxsurv(fit, newdata = aids[c(1,40), ],
             target = "cumhaz")
}
\seealso{
\code{\link[=coxph_indtrunc]{coxph_indtrunc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipw_ests.R
\name{positivity_sens_indtrunc}
\alias{positivity_sens_indtrunc}
\alias{print.coxph_pos_indtrunc}
\alias{plot.coxph_pos_indtrunc}
\title{Sensitivity analysis for positivity violations in a Cox model with
quasi-independent double truncation.}
\usage{
positivity_sens_indtrunc(
  formula,
  data = NULL,
  strata_formula = NULL,
  trunc_mass = seq(0, 0.9, by = 0.1),
  ipw_type = c("W-asurv", "W-surv", "W-1", "W-a"),
  se = TRUE,
  level = 0.95,
  restr_times = NULL,
  npmle_fit = NULL,
  robust_infl = NULL,
  max_iter = 100,
  error = 1e-07,
  beta_init = NULL,
  center_covariates = FALSE,
  simple_labels = FALSE
)

\method{print}{coxph_pos_indtrunc}(x, ...)

\method{plot}{coxph_pos_indtrunc}(x, parm, level, hazratio = FALSE, dropNA = TRUE, ...)
}
\arguments{
\item{formula}{A model formula with a \code{Survdt} response.
Subjects with missing values are discarded.
Time-varying covariates are not supported.
See \code{\link[=Survdt]{Survdt()}}.}

\item{data}{An optional data frame used to evaluate \code{formula} and
\code{strata_formula}.}

\item{strata_formula}{An optional one-sided formula with stratification
variables on the right hand side, for stratified Cox models.}

\item{trunc_mass}{A numerical vector containing the grid of truncated
baseline mass values to use in the sensitivity analysis.}

\item{ipw_type}{Type of inverse probability weights.
\enumerate{
\item \code{"W-asurv"} (the default): stabilized robust time-varying weights based
on the NPMLE survival function.
\item \code{"W-surv"}: robust time-varying weights based on the NPMLE survival function.
\item \code{"W-1"}: standard weights with no time-varying component (Rennert and Xie, 2018)
\item \code{"W-a"}: stabilized weights (Mandel et al., 2018)
\item For custom time-varying weights, provide a function that takes a numerical
vector of times as input and returns a numerical vector of positive weights.
}}

\item{se}{Whether to calculate (robust) standard errors (default \code{TRUE}).}

\item{level}{Confidence level for constructing sensitivity intervals
(default 0.95).}

\item{restr_times}{An optional numerical vector of length two giving a time
window to restrict the score function.
Values of \code{NA} are replaced with the minimum and maximum observed event
time respectively, i.e. no restriction.
The default setting is no restriction.}

\item{npmle_fit}{An optional pre-computed \code{npmle} object fit on the complete
cases.
If it was fit with \code{ainv}=\code{FALSE} (or \code{infl}=\code{FALSE} when \code{se}=\code{TRUE}) then
it is ignored and a new NPMLE is fit.
See \code{\link[=npmle]{npmle()}}.}

\item{robust_infl}{A square matrix of influence values for the event weight,
with one column per individual and one row per follow up time, both in the
order that they appear in the data.
Only used if \code{ipw_type} is a function and \code{se}=\code{TRUE}.}

\item{max_iter}{Maximum number of iterations before non-convergence
is declared at a given truncated mass (default 100).}

\item{error}{Tolerance for determining numerical convergence of coefficient
estimates (default \code{1e-7}).}

\item{beta_init}{Optional numerical vector with initial values for each
coefficient estimate.}

\item{center_covariates}{Whether to center the covariates before the
sensitivity analysis (default \code{FALSE}).
Covariates with values of 0/1 are not centered.}

\item{simple_labels}{Whether to drop the factor names from the stratum labels
(Default \code{FALSE}).}

\item{x}{A \code{coxph_pos_indtrunc} object.}

\item{...}{Additional arguments passed to other methods.}

\item{parm}{A numeric or character vector specifying a set of model
parameters.
Defaults to all identifiable (non-missing) coefficients.}

\item{hazratio}{Whether to plot the hazard ratios exp(beta) and their
sensitivity intervals instead of the regression coefficients beta (default
\code{FALSE}).}

\item{dropNA}{Whether to remove truncation masses where the estimator did not
converge, resulting in missing values, from the plot (default \code{TRUE}).}
}
\value{
\code{\link[=positivity_sens_indtrunc]{positivity_sens_indtrunc()}} returns an object of class \code{coxph_pos_indtrunc},
i.e. a list with the following components:
\item{n}{The complete case sample size.}
\item{n_miss}{The number of individuals removed due to missing data.}
\item{n_event}{The number of observed events used in the score function.}
\item{level}{The user-specified confidence level for constructing sensitivity
intervals.}
\item{trunc_mass}{The truncated baseline masses supplied in \code{trunc_mass}
(sorted).
The baseline covariate values are provided in \code{col_centerings}.}
\item{coef}{A matrix of coefficient estimates with \code{j}th column corresponding
to \code{trunc_mass[j]}.
Missing values indicate non-convergence or collinearity.}
\item{se}{A matrix of standard errors with \code{j}th column corresponding to
\code{trunc_mass[j]}.}
\item{lower,upper}{Matrices containing the lower and upper sensitivity
interval endpoints, with \code{j}th column corresponding to \code{trunc_mass[j]}.}
\item{timerange}{A vector with the minimum and maximum times used in the
score function.}
\item{eventrange}{A vector with the unrestricted range of observed event times.}
\item{wt_name}{A string giving the name of the weights used in \code{ipw_type}.}
\item{col_centerings}{A numerical vector with the values used to center the
columns of the design matrix if \code{center_covariates}=\code{TRUE}.
Otherwise, a vector of zeros.}
\item{strata_summary}{Some summary statistics for each stratum, computed prior
to applying \code{restr_times}.}
\item{call_}{The call to \code{\link[=positivity_sens_indtrunc]{positivity_sens_indtrunc()}}.}

The \code{print} method for \code{coxph_pos_indtrunc} objects displays a summary of the
sensitivity analysis results, while the \code{plot} method returns a visualization
of the estimators and their sensitivity intervals across different truncated
mass values, generated through \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
Computes inverse probability weighted sensitivity analysis estimators for the
Cox model using nonparametric weights (see \code{\link[=npmle]{npmle()}}).
Given a grid of values for the potential amount of baseline probability mass
lost due to zero sampling probability, shows how a positivity violation could
influence the results of a fitted Cox model.
}
\details{
Consider a sample of doubly truncated data with event times \eqn{T_i},
truncation times \eqn{L_i} and \eqn{R_i}, and covariates \eqn{X_i}.

A positivity violation occurs when the sampling probability
\eqn{P(L<T \leq R|T=t)} is zero for \eqn{t\leq a} and \eqn{t > b}, where the
observation bounds \eqn{a}
and \eqn{b} lie within the event time support.
The truncated baseline probability mass is defined as the amount of probability
mass right-truncated by the positivity violation at the baseline covariate
level, conditional on not being left-truncated by the positivity violation,
i.e. \eqn{P(T> b|X=0,T > a)}.
Note that the interpretation of this truncated mass depends on how the baseline
covariate value is defined, e.g. whether the covariates are centered.

At each specified truncated mass, \code{\link[=positivity_sens_indtrunc]{positivity_sens_indtrunc()}} fits a Cox model
that accounts for the positivity violation through an adjustment to the
at-risk process (see Vazquez and Xie, 2025).
This modified at-risk process is inserted into the weighted score function from
\code{\link[=coxph_indtrunc]{coxph_indtrunc()}}.

The end result of the sensitivity analysis is a series of adjusted coefficient
estimators at the grid of truncated masses, which show how sensitive the Cox
model estimates are to potential positivity violations, as well as sensitivity
intervals giving a range of plausible values for the coefficients at the
user-supplied confidence \code{level} for each truncated mass.

In general, the sensitivity analysis estimators may not converge at
some truncated mass values.
Usually this is due to large weights, near-zero risk scores
\eqn{\exp(X_i^T\beta)}, and/or truncated masses too close to one.
Some options to consider are:
\itemize{
\item Changing the baseline covariate value by shifting the covariates or reordering
factor levels.
\item Using time-varying weights (see \code{ipw_type}).
\item Restricting the time window in the score function through \code{restr_times}
(see below).
\item Altering the range and step size of \code{trunc_mass}.
}
}
\section{Stratified Cox model}{


A stratified Cox model is specified through \code{strata_formula}.
The formula should look like \code{~svar1 + svar2}... where \code{svar1} and \code{svar2} are
factor variables defining the strata.
This allows the baseline hazard to differ across strata.
}

\section{Restricted time window in score function}{

The user can choose to restrict the time range of the score function during
estimation.
This may be useful when extreme event times have very large weights.
Setting the input argument \code{restr_times=c(t1,t2)} will restrict the range of
the integrals in the score function to the interval \eqn{[}\code{t1}, \code{t2}\eqn{]}
instead of the entire event time range.
Notably, this time restriction will not introduce any bias in the fitted model.
}

\examples{
sensfit <- positivity_sens_indtrunc(Survdt(incu, ltrunc, rtrunc) ~ age_gp,
                                    data = aids,
                                    trunc_mass = seq(0, 0.11, by = 0.02),
                                    ipw_type = "W-asurv")
sensfit
plot(sensfit)

}
\references{
Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
estimators for doubly truncated Cox regression with closed-form standard
errors. \emph{Lifetime Data Analysis}, \strong{31}(2), 364–-393.
}
\seealso{
\itemize{
\item \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}: fitting a Cox model under quasi-independent truncation.
\item \code{vignette("cox-regression-quasiindep", package = "survdt")}: an illustration
of this sensitivity analysis applied to the \link{aids} data, if vignettes were
built when installing \link{survdt}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{nonprop_sample}
\alias{nonprop_sample}
\title{Simulated data with non-proportional hazards and independent double truncation}
\format{
A data frame with 490 rows and 5 columns:
\describe{
\item{\code{ltrunc}}{The left truncation time.}
\item{\code{rtrunc}}{The right truncation time.}
\item{\code{event_time}}{The event time.}
\item{\code{group}}{A binary covariate with non-proportional hazards.}
\item{\code{x}}{A continuous covariate with proportional hazards.}
}
}
\usage{
nonprop_sample
}
\description{
The event times were simulated from a Cox model with hazard function
\deqn{\lambda(t|x,\textit{group}) = \exp\{0.7\textit{group}\times 1(t \leq 0.6) - 0.5x\}\lambda_0(t-0.1)}
where \eqn{\lambda_0(t)} is the hazard function from a beta(2,2) distribution.
The covariates \eqn{\textit{group}} and \eqn{x} were independent bernoulli(0.5)
and uniform(0,1) random variates, respectively.

The left truncation time had a uniform(0,0.6) distribution,
while the time between the left and right truncation time was an independent
uniform(0.1,0.7) random variate.
All times were rounded to two decimal places.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npests.R
\name{npmle}
\alias{npmle}
\alias{is.npmle}
\alias{print.npmle}
\title{Compute the nonparametric MLE under double truncation}
\usage{
npmle(
  y,
  data = NULL,
  se = TRUE,
  cumhaz = FALSE,
  ainv = TRUE,
  infl = FALSE,
  max_iter = 1000,
  error = 1e-09,
  init = NULL,
  convergence_warning = TRUE,
  check_identifiability = TRUE
)

is.npmle(x)

\method{print}{npmle}(x, ...)
}
\arguments{
\item{y}{A \code{Survdt} object or expression. Observations with missing values
are discarded.
See \code{\link[=Survdt]{Survdt()}}.}

\item{data}{An optional data frame used to evaluate \code{y}.}

\item{se}{Whether to compute standard errors for each estimate
(default \code{TRUE}).}

\item{cumhaz}{Whether to also compute the NPMLE of the event time cumulative
hazard (default \code{FALSE}).}

\item{ainv}{Whether to return the NPMLE of the normalized inverse selection
probabilities at the observed event times (default \code{TRUE}).}

\item{infl}{Whether to return the influence values for each estimate
(default \code{FALSE}).}

\item{max_iter}{The maximum number of iterations for the EM algorithm.}

\item{error}{Tolerance for determining numerical convergence, defined as the
relative change in the log likelihood being less than \code{error} or the absolute
change being less than sqrt(\code{error}) at any iteration.}

\item{init}{Optional event time cdf point masses to initialize the EM
algorithm.}

\item{convergence_warning}{Whether to issue a warning when numerical
convergence is not achieved by \code{max_iter} iterations (default \code{TRUE}).}

\item{check_identifiability}{Whether to check and return an error if the
NPMLE is not identifiable (default \code{TRUE}, see below).}

\item{x}{Any R object.}

\item{...}{Additional arguments passed to or from other methods.}
}
\value{
\code{\link[=npmle]{npmle()}} returns an object of class \code{npmle}, i.e. a list with the following
components:
\item{n}{The complete case sample size.}
\item{n_miss}{The number of discarded observations (with missing values).}
\item{cdf}{A matrix containing the estimated event time cdf evaluated at
ordered unique event times, with columns \code{time}, \code{cdf}, and \code{se}.}
\item{cumhaz}{A matrix containing the estimated event time cumulative hazard
evaluated at ordered unique event times, with columns \code{time}, \code{cumhaz}, and
\code{se}.
Set to \code{NULL} if \code{cumhaz}=\code{FALSE}.}
\item{ainv}{A matrix containing the estimated normalized inverse selection
probabilities for each event time (not sorted and including duplicates),
with columns \code{time}, \code{ainv}, and \code{se}.
Set to \code{NULL} if \code{ainv}=\code{FALSE}.}
\item{infl_cdf}{A matrix containing the estimated influence function for the
cdf, with \verb{[i,j]}th element corresponding to the \code{i}th ordered unique event
time and \code{j}th unsorted individual.
Set to \code{NULL} if \code{infl}=\code{FALSE}.}
\item{infl_cumhaz}{A matrix containing the estimated influence function for
the cumulative hazard, with \verb{[i,j]}th element corresponding to the \code{i}th
ordered unique event time and \code{j}th unsorted individual.
Set to \code{NULL} if \code{cumhaz=FALSE} or \code{infl}=\code{FALSE}.}
\item{infl_ainv}{A matrix containing the estimated influence function for the
normalized inverse selection probabilities, with \verb{[i,j]}th element
corresponding to the \code{i}th event time (including ties) and \code{j}th individual,
both unsorted.
Set to \code{NULL} if \code{ainv=FALSE} or \code{infl}=\code{FALSE}.}
\item{masses}{A data frame containing the point masses for the event time
(\code{f}) and truncation time (\code{g}) cdfs and the estimated selection
probabilities for each event time (\code{pt}) and truncation interval (\code{plr}),
with one row per observation.
Not sorted.}
\item{converged}{Whether the EM algorithm converged before \code{max_iter} was
reached.}
\item{n_iter}{The total number of EM algorithm iterations that ran.}
\item{overall_sampling_prob}{A named vector with the estimated overall
probability of non-truncation in \code{prob} and its standard error in \code{se}.}

\code{\link[=is.npmle]{is.npmle()}} returns \code{TRUE} if \code{x} inherits from the class \code{npmle} and
\code{FALSE} otherwise. The \code{print} method for \code{npmle} objects displays some sample
information and the first few steps of the estimated cdf.
}
\description{
Computes the nonparametric maximum likelihood estimator (NPMLE) of the
pre-truncation event time and truncation time distributions using an EM
algorithm.
}
\details{
Given a doubly truncated sample \eqn{(T_1,L_1,R_1)}, \eqn{(T_2,L_2,R_2)},
\eqn{\ldots}, \eqn{(T_n,L_n,R_n)}, the NPMLE of the (pre-truncation) event
time cdf is a right-continuous step function
\deqn{\widehat{F}(t) = \sum_{i=1}^n 1(T_i\leq t) \widehat{f}_i}
with point masses at the observed event times.
Assuming (quasi-)independent truncation, these point masses maximize the
likelihood function
\deqn{L(f_1, \ldots, f_n) = \prod_{i=1}^n \frac{f_i}{\sum_j 1(L_i<T_j \leq R_i) f_j}.}
Efron and Petrosian (1999) provided a simple EM algorithm to compute the NPMLE.

The point masses of the NPMLE can be expressed as
\eqn{\widehat{f}_i = 1/[\widehat{a}(T_i)n]},
where \eqn{a(t) = P(L<T \leq R|T=t)/P(L<T \leq R)} is the normalized
probability of observing time \eqn{t} within a truncation interval.
These inverse selection probabilities are used to correct for double truncation
bias in \code{\link[=coxph_indtrunc]{coxph_indtrunc()}}.

Once \eqn{\widehat{F}(t)} is obtained, other quantities such as the survival
function, cumulative hazard, and bivariate truncation time cdf point masses
are straightforward to compute.
}
\section{Identifiability}{

Xiao and Hudgens (2019) provided a necessary and sufficient condition for the
likelihood function \eqn{L(f_1, \ldots, f_n)} to have a unique maximizer.
By default, this condition is checked by \code{\link[=npmle]{npmle()}} before estimation.
}

\section{Standard errors}{

By default, \code{\link[=npmle]{npmle()}} computes variance estimates for the NPMLE
using the closed-form standard errors derived by Vazquez and Xie (2025).
This is generally more computationally efficient than the resampling methods
used by other packages.
If fit with \code{infl}=\code{TRUE}, the \code{npmle} object may be supplied as an input to
other \link{survdt} functions, which will then not need to re-compute the influence
values.
}

\examples{
npmle(Survdt(incu, ltrunc, rtrunc),
      data = aids)


}
\references{
Efron, B., & Petrosian, V. (1999). Nonparametric methods for doubly truncated
data. \emph{Journal of the American Statistical Association}, \strong{94}(447), 824--834.

de Uña-Álvarez, J., & Van Keilegom, I. (2021). Efron–Petrosian integrals for
doubly truncated data with covariates: An asymptotic analysis. \emph{Bernoulli},
\strong{27}(1), 249--273.

Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
estimators for doubly truncated Cox regression with closed-form standard
errors. \emph{Lifetime Data Analysis}, \strong{31}(2), 364–-393.

Xiao, J., & Hudgens, M. G. (2019). On nonparametric maximum likelihood
estimation with double truncation. \emph{Biometrika}, \strong{106}(4), 989--996.
}
\seealso{
\itemize{
\item \code{\link[=plot.npmle]{plot.npmle()}}: compute/plot estimates and pointwise confidence
intervals for the event time survival function and cumulative hazard,
normalized inverse selection probability weights, and selection probability
as a function of time.
\item \code{\link[=test_ignorability_indtrunc]{test_ignorability_indtrunc()}}: test for ignorable sampling bias using the
NPMLE.
\item \code{\link[=test_samplediff_indtrunc]{test_samplediff_indtrunc()}}: test for differences in the event time
distributions across groups using the NPMLE.
\item \code{\link[=test_quasiindep_ctau]{test_quasiindep_ctau()}}: test for violations of the quasi-independent
truncation assumption.
\item \code{vignette("nonparametric-analysis", package = "survdt")}: an example of
various nonparametric analyses applied to the \link{aids} data, if vignettes
were built when installing \link{survdt}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{aids}
\alias{aids}
\title{AIDS incubation data}
\format{
A data frame with 295 rows and 5 columns:
\describe{
\item{\code{incu}}{The incubation time, i.e. months from HIV infection to AIDS
diagnosis.}
\item{\code{age}}{The age at HIV infection in years.}
\item{\code{ltrunc}}{The time from HIV infection to the start of data collection
(1982), in months.
Note that \code{ltrunc} is negative for patients infected before 1982.}
\item{\code{rtrunc}}{The time from HIV infection to the end of data collection
(July 1986), in months.}
\item{\code{age_gp}}{A factor with three \code{age} groups: \verb{<=4}, \code{4-59}, and \verb{>59}
(the baseline level).}
}
}
\source{
Wang, M. -C. (1989). A semiparametric model for randomly truncated data.
\emph{Journal of the American Statistical Association}, \strong{84}(407), 742--748.
}
\usage{
aids
}
\description{
A doubly truncated dataset of AIDS incubation times from patients with
transfusion-acquired HIV collected by the Centers for Disease Control and
Prevention (CDC). The sample contains patients who were diagnosed between
the discovery of the disease (1982) and the end of study (July 1986), with
infection times determined retrospectively from a blood transfusion registry.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npests.R
\name{plot.npmle}
\alias{plot.npmle}
\title{Plot estimates from an NPMLE fit}
\usage{
\method{plot}{npmle}(
  x,
  ci = TRUE,
  level = 0.95,
  ctype = c("log-log", "log", "plain"),
  target = c("survival", "cumhaz", "ainv", "sel_prob"),
  ...
)
}
\arguments{
\item{x}{An \code{npmle} object. See \code{\link[=npmle]{npmle()}}.}

\item{ci}{Whether to plot pointwise confidence intervals (default \code{TRUE}).}

\item{level}{Confidence level for the pointwise confidence intervals (default
0.95).}

\item{ctype}{Type of confidence interval.
\itemize{
\item \code{"plain"}: linear confidence intervals from a normal approximation.
\item \code{"log"}: back-transformed confidence intervals from the natural log scale.
Guaranteed to be non-negative.
\item \code{"log-log"} (the default): for the survival function and selection
probability, back-transformed confidence intervals from the (natural) log
minus log scale, which are guaranteed to be between zero and one.
For the normalized inverse probability weights and cumulative hazard,
the same confidence intervals as \code{"log"} are used.
}}

\item{target}{Which NPMLE quantity to plot.
\itemize{
\item \code{"survival"} (the default): the event time survival function.
\item \code{"cumhaz"}: the event time cumulative hazard function.
\item \code{"ainv"}: the normalized inverse probability weights at each observed
event time. See \code{\link[=npmle]{npmle()}}.
\item \code{"sel_prob"}: the selection (non-truncation) probability as a function of
time.
}}

\item{...}{Additional arguments passed to other methods.}
}
\value{
After displaying the plot, a list with two elements is returned
invisibly.
The first element \code{ests} is a data frame with the columns \code{time},
\code{est}, \code{lower}, and \code{upper}, containing the point estimates and their computed
confidence intervals.
The second element \code{plot} is the plotted object returned by \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
Given a fitted NPMLE from \code{\link[=npmle]{npmle()}}, plot various quantities related to the
event time or truncation time distributions along with pointwise confidence
intervals.
The estimates and standard errors are computed by \code{\link[=plot.npmle]{plot.npmle()}} if they
were not included in the \code{npmle} fit.
}
\examples{
fit <- npmle(Survdt(incu, ltrunc, rtrunc),
             data = aids)
plot(fit)

plot(fit, target = "cumhaz")
}
\seealso{
\code{\link[=npmle]{npmle()}}
}
